@@ -376,9 +376,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
     def test_sign_repository_indexes(self) -> None:
         """Generate and sign indexes for a package repository."""
         suite_name = "signing-experiment"
-        self.create_suite(
-            suite_name, {"components": ["main"], "architectures": ["amd64"]}
-        )
+        self.create_suite(suite_name, architectures=["amd64"])
         self.add_suite_to_archive(suite_name)
 
         workflow_template_result = Client.execute_command(
