@@ -883,14 +883,21 @@ ArtifactDataWithArch: TypeAlias = (
 )
 
 
-def get_architecture(artifact_data: ArtifactDataWithArch) -> str:
+def get_architectures(artifact_data: ArtifactDataWithArch) -> set[str]:
     """Retrieve the architecture of the package."""
+    architectures: set[str] = set()
     match artifact_data:
         case DebianBinaryPackages() | DebianPackageBuildLog():
-            return cast(str, artifact_data.architecture)
+            assert artifact_data.architecture is not None
+            architectures.add(artifact_data.architecture)
         case DebianBinaryPackage():
-            return cast(str, artifact_data.deb_fields.get("Architecture"))
+            architectures.add(artifact_data.deb_fields["Architecture"])
         case DebianUpload():
-            return cast(str, artifact_data.changes_fields.get("Architecture"))
-
-    raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+            for architecture in artifact_data.changes_fields[
+                "Architecture"
+            ].split():
+                if architecture != "source":
+                    architectures.add(architecture)
+        case _:
+            raise TypeError(f"Unexpected type: {type(artifact_data).__name__}")
+    return architectures
