@@ -98,8 +98,7 @@ class ArtifactPlayground:
         version: str = "1.0-1",
         src_architectures: set[str] | None = None,
         binary: bool = False,
-        binaries: list[str] | None = None,
-        bin_architecture: str = "all",
+        binaries: list[tuple[str, str]] | None = None,
         section: str = "unknown",
         priority: str = "optional",
     ) -> Upload:
@@ -107,7 +106,7 @@ class ArtifactPlayground:
         architectures: set[str] = set()
         files: list[Path] = []
         if binaries is None:
-            binaries = [src_name]
+            binaries = [(src_name, "all")]
         if source:
             architectures.add("source")
             dsc = destdir / f"{src_name}_{version}.dsc"
@@ -129,14 +128,14 @@ class ArtifactPlayground:
                 dsc,
                 src_files,
                 source=src_name,
-                binaries=binaries,
+                binaries=[name for (name, architecture) in binaries],
                 version=version,
                 architectures=src_architectures,
             )
             files += [dsc] + src_files
         if binary:
-            architectures.add(bin_architecture)
-            for bin_name in binaries:
+            for bin_name, bin_architecture in binaries:
+                architectures.add(bin_architecture)
                 deb = destdir / f"{bin_name}_{version}_{bin_architecture}.deb"
                 cls.write_deb_file(
                     deb,
@@ -157,7 +156,11 @@ class ArtifactPlayground:
             changes,
             files,
             source=src_name,
-            binaries=binaries,
+            binaries=(
+                [name for (name, architecture) in binaries]
+                if binaries
+                else None
+            ),
             version=version,
             section=section,
         )
