@@ -83,9 +83,14 @@ class FileUploader:
     # client_max_body_size as well
     _MAX_CHUNK_SIZE_BYTES = 50 * 1024 * 1024
 
-    def __init__(self, token: str, *, logger: logging.Logger) -> None:
+    def __init__(
+        self, token: str, scope: str, *, logger: logging.Logger
+    ) -> None:
         """Initialize object."""
-        self._headers = {"Token": token}
+        self._headers = {
+            "Token": token,
+            "X-Debusine-Scope": scope,
+        }
         self._logger = logger
 
     def _upload_chunk(
