@@ -239,6 +239,7 @@ class ArtifactResponse(ServerObject):
 
     id: int
     url: str
+    scope: str
     workspace: str
     category: str
     created_at: dt.datetime
@@ -609,6 +610,15 @@ class CollectionData(ServerObject):
     stats: list[dict[str, Any]] | None = None
 
 
+class EmitMetricRequest(StrictBaseModel):
+    """A request for the server to emit a metric on behalf of the client."""
+
+    metric_type: Literal["counter", "gauge", "summary", "histogram"]
+    name: str
+    labels: dict[str, str]
+    value: float
+
+
 def model_to_json_serializable_dict(
     model: pydantic.BaseModel, exclude_unset: bool = False
 ) -> dict[Any, Any]:
