@@ -281,11 +281,12 @@ class DebusineTests(TestCase):
         self.api_url = "https://debusine.debian.org/api"
 
         self.token = "token"
+        self.scope = "scope"
         self.logger = mock.create_autospec(spec=logging.Logger)
         # Add a trailing slash to the API URL to ensure that all parts of
         # the client strip it correctly.
         self.debusine = Debusine(
-            self.api_url + "/", self.token, logger=self.logger
+            self.api_url + "/", self.token, scope=self.scope, logger=self.logger
         )
 
     def test_webui_url(self) -> None:
@@ -686,6 +687,50 @@ class DebusineTests(TestCase):
             ),
         )
 
+    @responses.activate
+    def test_artifact_get_with_workspace(self) -> None:
+        """Test artifact_get() with a workspace."""
+        artifact_id = 2
+        workspace = "test"
+        download_artifact_tar_gz_url = "https://example.com/path/"
+        category = "Testing"
+        created_at = dt.datetime.now(dt.UTC)
+        data: dict[str, Any] = {}
+
+        get_artifact_response = create_artifact_response(
+            id=artifact_id,
+            workspace=workspace,
+            category=category,
+            created_at=created_at,
+            data=data,
+            download_tar_gz_url=download_artifact_tar_gz_url,
+        )
+
+        responses.add(
+            responses.GET,
+            (
+                f"{self.api_url}/1.0/artifact/{workspace}/"
+                f"{get_artifact_response.id}/"
+            ),
+            json=model_to_json_serializable_dict(get_artifact_response),
+        )
+
+        artifact = self.debusine.artifact_get(artifact_id, workspace=workspace)
+
+        self.assert_token_key_included_in_all_requests(self.token)
+
+        self.assertEqual(
+            artifact,
+            create_artifact_response(
+                id=artifact_id,
+                category=category,
+                data=data,
+                created_at=created_at,
+                workspace=workspace,
+                download_tar_gz_url=download_artifact_tar_gz_url,
+            ),
+        )
+
     @responses.activate
     def test_artifact_create_success(self) -> None:
         """Test artifact_create succeeds."""
@@ -972,7 +1017,7 @@ class DebusineTests(TestCase):
         }
 
         # Set endpoint to download artifact's information
-        url_artifact_information = self.api_url + f"/1.0/artifact/{artifact_id}"
+        url_artifact_information = f"{self.api_url}/1.0/artifact/{artifact_id}/"
         artifact_response = create_artifact_response(
             id=artifact_id,
             download_tar_gz_url=url_download_artifact,
@@ -1202,7 +1247,7 @@ class DebusineTests(TestCase):
         )
 
         # Set endpoint to download artifact's information
-        url_artifact_information = self.api_url + f"/1.0/artifact/{artifact_id}"
+        url_artifact_information = f"{self.api_url}/1.0/artifact/{artifact_id}/"
         artifact_response = create_artifact_response(
             id=artifact_id,
             files=FilesResponseType(
@@ -1824,6 +1869,30 @@ class DebusineTests(TestCase):
         self.assertEqual(res, [data])
         responses.assert_call_count(url, 1)
 
+    @responses.activate
+    def test_emit_metric(self) -> None:
+        url = f"{self.debusine.api_url}/open-metrics/emit/"
+        responses.add(responses.POST, url, status=requests.codes.no_content)
+
+        self.debusine.emit_metric(
+            metric_type="counter",
+            name="debusine_test",
+            labels={"foo": "bar"},
+            value=1,
+        )
+
+        responses.assert_call_count(url, 1)
+        assert responses.calls[0].request.body is not None
+        self.assertEqual(
+            json.loads(responses.calls[0].request.body),
+            {
+                "metric_type": "counter",
+                "name": "debusine_test",
+                "labels": {"foo": "bar"},
+                "value": 1,
+            },
+        )
+
 
 class DebusineAsyncTests(server.DebusineAioHTTPTestCase, TestCase):
     """
