@@ -113,9 +113,10 @@ class FileUploaderTests(TestCase):
         super().setUp()
         self.url = "https://localhost:8011/1.0/artifact/2/README.md"
         self.token = "21efb7587bd10b8eb2c56b4280b4c4a92a9"
+        self.scope = "scope"
 
         self.file_uploader = FileUploader(
-            self.token, logger=logging.getLogger(__name__)
+            self.token, scope=self.scope, logger=logging.getLogger(__name__)
         )
 
         self.file_content = b"test-content"
@@ -161,6 +162,17 @@ class FileUploaderTests(TestCase):
         content_range = f"bytes {uploaded_size}-{file_size - 1}/{file_size}"
         return content_range
 
+    @responses.activate
+    def test_authentication(self) -> None:
+        """Client sets the correct headers to authenticate the request."""
+        responses.add(responses.PUT, self.url)
+        self.file_uploader.upload(self.file_path, self.url)
+
+        request = responses.calls[0].request
+
+        self.assertEqual(request.headers["Token"], self.token)
+        self.assertEqual(request.headers["X-Debusine-Scope"], self.scope)
+
     @responses.activate
     def test_file_uploader_check_file_was_completed(self) -> None:
         """
