@@ -10,11 +10,11 @@
 """Data models for db users and authentication."""
 
 import abc
+import datetime as dt
 import enum
 import hashlib
 import re
 import secrets
-from datetime import datetime, timedelta
 from enum import StrEnum
 from typing import (
     Annotated,
@@ -61,8 +61,11 @@ from debusine.server import notifications
 from debusine.utils.typing_utils import copy_signature_from
 
 if TYPE_CHECKING:
+    from django.http import HttpRequest
     from django_stubs_ext.db.models import TypedModelMeta
 
+    from debusine.web.views.ui.auth import GroupUI, UserUI
+
     # https://github.com/typeddjango/django-stubs/issues/2112
     AbstractUserMeta = TypedModelMeta
 else:
@@ -81,7 +84,7 @@ class TokenManager(models.Manager["Token"]):
         # should be enough for most providers to be able to launch an
         # instance.
         return self.create(
-            expire_at=timezone.now() + timedelta(minutes=15), enabled=True
+            expire_at=timezone.now() + dt.timedelta(minutes=15), enabled=True
         )
 
     def get_tokens(
@@ -124,7 +127,7 @@ class TokenManager(models.Manager["Token"]):
         except Token.DoesNotExist:
             return None
 
-    def expired(self, at: datetime) -> QuerySet["Token"]:
+    def expired(self, at: dt.datetime) -> QuerySet["Token"]:
         """
         Return queryset with tokens that have expired.
 
@@ -181,6 +184,11 @@ class Token(models.Model):
 
     key: str
 
+    objects = TokenManager()
+
+    class Meta(TypedModelMeta):
+        base_manager_name = "objects"
+
     def save(self, *args: Any, **kwargs: Any) -> None:
         """Save the token. If it's a new token it generates a key."""
         if not self.hash:
@@ -214,8 +222,6 @@ class Token(models.Model):
         """Hash the given secret."""
         return hashlib.sha256(secret.encode()).hexdigest()
 
-    objects = TokenManager()
-
 
 SYSTEM_USER_NAME = "_system"
 
@@ -274,6 +280,7 @@ class User(AbstractUser):
     objects = UserManager()  # type: ignore[misc]
 
     class Meta(AbstractUserMeta):
+        base_manager_name = "objects"
         constraints = [
             # System users must not have an email address.
             CheckConstraint(
@@ -309,6 +316,12 @@ class User(AbstractUser):
         assert user is not None  # enforced by decorator
         return self.pk == user.pk
 
+    def ui(self, request: "HttpRequest") -> "UserUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.auth import UserUI
+
+        return UserUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return an absolute URL to this user."""
         return reverse("user:detail", kwargs={"username": self.username})
@@ -508,6 +521,7 @@ class Group(models.Model):
     objects = GroupManager.from_queryset(GroupQuerySet)()
 
     class Meta(TypedModelMeta):
+        base_manager_name = "objects"
         constraints = [
             UniqueConstraint(
                 fields=["name", "scope"],
@@ -531,6 +545,12 @@ class Group(models.Model):
         else:
             self.add_audit_log(GroupAuditLogUpdated.create_diff(old, self))
 
+    def ui(self, request: "HttpRequest") -> "GroupUI":
+        """Return a UI helper for this instance."""
+        from debusine.web.views.ui.auth import GroupUI
+
+        return GroupUI.get(request, self)
+
     def get_absolute_url(self) -> str:
         """Return an absolute URL to this user."""
         # Prevent circular import
@@ -689,7 +709,7 @@ class GroupAuditLogEntryBase(pydantic.BaseModel, abc.ABC):
         """Set up stricter pydantic Config."""
 
         validate_assignment = True
-        extra = pydantic.Extra.forbid
+        extra = "forbid"
 
     #: Username of the user performing the action
     actor: str = ""
@@ -854,6 +874,9 @@ class GroupAuditLog(models.Model):
 
     objects = GroupAuditLogManager()
 
+    class Meta(TypedModelMeta):
+        base_manager_name = "objects"
+
     @property
     def changes_model(self) -> GroupAuditLogEntry:
         """Return the pydantic model for changes."""
