@@ -182,8 +182,19 @@ class Collection(models.Model):
         max_length=255, validators=[validate_collection_name]
     )
     category = models.CharField(max_length=255)
-    full_history_retention_period = models.DurationField(null=True, blank=True)
-    metadata_only_retention_period = models.DurationField(null=True, blank=True)
+    full_history_retention_period = models.DurationField(
+        null=True,
+        blank=True,
+        help_text="Period after removing an item from the collection"
+        " during which the related artifact is not allowed to expire",
+    )
+    metadata_only_retention_period = models.DurationField(
+        null=True,
+        blank=True,
+        help_text="Period after removing an item from the collection during"
+        " which its metadata is kept in the history of the collection",
+    )
+
     workspace = models.ForeignKey(
         Workspace, on_delete=models.PROTECT, related_name="collections"
     )
@@ -191,6 +202,8 @@ class Collection(models.Model):
         max_length=8,
         choices=RetainsArtifacts.choices,
         default=RetainsArtifacts.ALWAYS,
+        help_text="Controls if artifacts in this collection should be retained"
+        " rather than being allowed to expire",
     )
 
     child_artifacts = models.ManyToManyField(
