@@ -176,6 +176,13 @@ class WorkerPoolTests(TestCase):
         self.assertIsNotNone(worker.activation_token.expire_at)
         self.assertTrue(worker.activation_token.enabled)
 
+    def test_launch_workers_disabled(self) -> None:
+        worker_pool = self.playground.create_worker_pool(enabled=False)
+        with self.assertRaisesRegex(
+            ValueError, r"Pool is disabled, refusing to launch workers."
+        ):
+            worker_pool.launch_workers(1)
+
     def test_launch_workers_integrity_error(self) -> None:
         """Exercise the branch from transaction.atomic() to exit."""
         with (
@@ -226,16 +233,18 @@ class WorkerPoolTests(TestCase):
         worker_pool = self.playground.create_worker_pool()
         worker = self.playground.create_worker(worker_pool=worker_pool)
         assert worker.activation_token
-        self.assertTrue(worker.activation_token.enabled)
+        token = worker.activation_token
+        self.assertTrue(token.enabled)
 
         worker_pool.terminate_worker(worker)
 
-        worker.activation_token.refresh_from_db()
-        self.assertFalse(worker.activation_token.enabled)
+        token.refresh_from_db()
+        self.assertFalse(token.enabled)
 
         worker.refresh_from_db()
         # The provider should have done this on termination:
         self.assertIsNone(worker.instance_created_at)
+        self.assertIsNone(worker.activation_token)
 
     def test_terminate_worker_with_token(self) -> None:
         worker_pool = self.playground.create_worker_pool()
@@ -243,17 +252,19 @@ class WorkerPoolTests(TestCase):
         assert worker.activation_token
         self.playground.create_worker_token(worker=worker)
         assert worker.token
+        token = worker.token
         worker.activation_token = None
         worker.save()
 
         worker_pool.terminate_worker(worker)
 
-        worker.token.refresh_from_db()
-        self.assertFalse(worker.token.enabled)
+        token.refresh_from_db()
+        self.assertFalse(token.enabled)
 
         worker.refresh_from_db()
         # The provider should have done this on termination:
         self.assertIsNone(worker.instance_created_at)
+        self.assertIsNone(worker.token)
 
     def test_terminate_worker_wrong_pool(self) -> None:
         worker_pool = self.playground.create_worker_pool()
@@ -304,14 +315,56 @@ class WorkerPoolTests(TestCase):
             ).exists()
         )
 
-    def test_terminate_worker_retries_running_work_request(self) -> None:
-        """Terminate a worker retries a running work request."""
+    def test_terminate_worker_deassigns_running_work_request(self) -> None:
+        """Terminate a worker deassigns a running work request, if possible."""
         worker_pool = self.playground.create_worker_pool()
         worker = self.playground.create_worker(worker_pool=worker_pool)
-        work_request_pending = self.playground.create_worker_task()
-        work_request_running = self.playground.create_worker_task()
-        work_request_running.assign_worker(worker)
-        work_request_running.mark_running()
+        work_request_pending = self.playground.create_worker_task(
+            mark_pending=True, worker=worker
+        )
+        work_request_running = self.playground.create_worker_task(
+            mark_running=True, worker=worker
+        )
+        work_request_completed = self.playground.create_worker_task(
+            result=WorkRequest.Results.SUCCESS, worker=worker
+        )
+
+        with self.assertNoLogs(
+            logger="debusine.db.models.worker_pools", level=logging.DEBUG
+        ):
+            worker_pool.terminate_worker(worker)
+
+        # The running work request was de-assigned.
+        work_request_running.refresh_from_db()
+        self.assertEqual(
+            work_request_running.status, WorkRequest.Statuses.PENDING
+        )
+        self.assertFalse(hasattr(work_request_running, "superseded"))
+
+        # the pending work request was de-assigned
+        work_request_pending.refresh_from_db()
+        self.assertEqual(
+            work_request_pending.status, WorkRequest.Statuses.PENDING
+        )
+        self.assertIsNone(work_request_pending.worker)
+        self.assertFalse(hasattr(work_request_pending, "superseded"))
+
+        # the completed work request was left alone.
+        work_request_completed.refresh_from_db()
+        self.assertEqual(
+            work_request_completed.status, WorkRequest.Statuses.COMPLETED
+        )
+        self.assertEqual(work_request_completed.worker, worker)
+        self.assertFalse(hasattr(work_request_completed, "superseded"))
+
+    def test_terminate_worker_retries_running_work_request(self) -> None:
+        """Terminate a worker retries a running work request, if needed."""
+        worker_pool = self.playground.create_worker_pool()
+        worker = self.playground.create_worker(worker_pool=worker_pool)
+        work_request = self.playground.create_worker_task(
+            mark_running=True, worker=worker
+        )
+        self.playground.create_build_log_artifact(work_request=work_request)
 
         with self.assertNoLogs(
             logger="debusine.db.models.worker_pools", level=logging.DEBUG
@@ -320,33 +373,26 @@ class WorkerPoolTests(TestCase):
 
         # The running work request was retried, though not yet assigned to a
         # worker.
-        work_request_running.refresh_from_db()
-        self.assertEqual(
-            work_request_running.status, WorkRequest.Statuses.ABORTED
-        )
-        self.assertTrue(hasattr(work_request_running, "superseded"))
-        self.assertIsNone(work_request_running.superseded.worker)
-        self.assertEqual(
-            work_request_running.superseded.workflow_data.retry_count, 1
-        )
-
-        # The pending work request was left alone.
-        work_request_pending.refresh_from_db()
-        self.assertEqual(
-            work_request_pending.status, WorkRequest.Statuses.PENDING
-        )
-        self.assertFalse(hasattr(work_request_pending, "superseded"))
+        work_request.refresh_from_db()
+        self.assertEqual(work_request.status, WorkRequest.Statuses.ABORTED)
+        self.assertTrue(hasattr(work_request, "superseded"))
+        self.assertIsNone(work_request.superseded.worker)
+        self.assertEqual(work_request.superseded.workflow_data.retry_count, 1)
 
     def test_terminate_worker_cannot_retry_running_work_request(self) -> None:
         """If a running work request cannot be retried, it is logged."""
         worker_pool = self.playground.create_worker_pool()
         worker = self.playground.create_worker(worker_pool=worker_pool)
-        work_request_running = self.playground.create_worker_task()
-        work_request_running.assign_worker(worker)
+        work_request_running = self.playground.create_worker_task(worker=worker)
         work_request_running.task_data = {"invalid": True}
         work_request_running.configured_task_data = {"invalid": True}
         work_request_running.save()
-        work_request_running.mark_running()
+        self.playground.advance_work_request(
+            work_request_running, mark_running=True
+        )
+        self.playground.create_build_log_artifact(
+            work_request=work_request_running
+        )
 
         with self.assertLogsContains(
             "Cannot retry previously-running work request: "
