@@ -96,7 +96,7 @@ class WorkerManagerTests(TestCase):
         )
         workers["dynamic_connected_busy"].connected_at = timezone.now()
         workers["dynamic_connected_busy"].save()
-        wr = self.playground.create_worker_task()
+        wr = self.playground.create_worker_task(mark_pending=True)
         wr.assign_worker(workers["dynamic_connected_busy"])
         wr.mark_running()
 
@@ -105,7 +105,7 @@ class WorkerManagerTests(TestCase):
         )
         workers["dynamic_connected_now_idle"].connected_at = timezone.now()
         workers["dynamic_connected_now_idle"].save()
-        wr = self.playground.create_worker_task()
+        wr = self.playground.create_worker_task(mark_pending=True)
         wr.assign_worker(workers["dynamic_connected_now_idle"])
         wr.mark_completed(WorkRequest.Results.SUCCESS)
 
@@ -149,7 +149,9 @@ class WorkerManagerTests(TestCase):
         )
 
         # A task is assigned to the worker
-        work_request = self.playground.create_worker_task(worker=worker)
+        work_request = self.playground.create_worker_task(
+            worker=worker, mark_pending=True
+        )
 
         # The worker is not ready (it is busy with a task assigned)
         self.assertQuerySetEqual(Worker.objects.waiting_for_work_request(), [])
@@ -186,13 +188,17 @@ class WorkerManagerTests(TestCase):
 
         # Assign two tasks to the worker; it can still accept more work
         for _ in range(2):
-            work_request = self.playground.create_worker_task(worker=worker)
+            work_request = self.playground.create_worker_task(
+                worker=worker, mark_pending=True
+            )
             self.assertQuerySetEqual(
                 Worker.objects.waiting_for_work_request(), [worker]
             )
 
         # Assign a third task to the worker; it is now as busy as it allows
-        work_request = self.playground.create_worker_task(worker=worker)
+        work_request = self.playground.create_worker_task(
+            worker=worker, mark_pending=True
+        )
         self.assertQuerySetEqual(Worker.objects.waiting_for_work_request(), [])
 
         # The third task finished
@@ -311,6 +317,17 @@ class WorkerManagerTests(TestCase):
             ["pool-001", "pool-003", "pool-004"],
         )
 
+    def test_get_or_create_scheduler(self) -> None:
+        """WorkerManager.get_or_create_scheduler returns a scheduler worker."""
+        worker = Worker.objects.get_or_create_scheduler()
+
+        self.assertEqual(worker.name, "scheduler")
+        self.assertIsNone(worker.token)
+        self.assertEqual(worker.worker_type, WorkerType.SCHEDULER)
+        self.assertIsNotNone(worker.pk)
+
+        self.assertEqual(Worker.objects.get_or_create_scheduler(), worker)
+
     def test_get_or_create_celery(self) -> None:
         """WorkerManager.get_or_create_celery returns a Celery worker."""
         worker = Worker.objects.get_or_create_celery()
@@ -399,8 +416,7 @@ class WorkerManagerTests(TestCase):
         worker.registered_at = timezone.now() - dt.timedelta(days=2)
         worker.instance_created_at = timezone.now() - dt.timedelta(days=1)
         worker.save()
-        work_request = self.playground.create_worker_task(worker=worker)
-        work_request.mark_running()
+        self.playground.create_worker_task(worker=worker, mark_running=True)
         worker.refresh_from_db()
         self.assertTrue(worker.is_busy())
 
@@ -410,18 +426,21 @@ class WorkerManagerTests(TestCase):
         self.assertGreaterEqual(idle_time, 0)
 
     def test_with_idle_time_completed_task(self) -> None:
-        """Compute idle_time with a running task."""
+        """Compute idle_time with a completed task."""
         worker = Worker.objects.create_with_fqdn(
             'worker-a', token=Token.objects.create(enabled=True)
         )
         worker.registered_at = timezone.now() - dt.timedelta(days=2)
         worker.instance_created_at = timezone.now() - dt.timedelta(days=1)
         worker.save()
-        work_request = self.playground.create_worker_task(worker=worker)
-        work_request.mark_running()
-        work_request.mark_completed(WorkRequest.Results.SUCCESS)
-        work_request.completed_at = timezone.now() - dt.timedelta(hours=6)
-        work_request.save()
+        for hours_ago in (10, 6):
+            work_request = self.playground.create_worker_task(
+                worker=worker, result=WorkRequest.Results.SUCCESS
+            )
+            work_request.completed_at = timezone.now() - dt.timedelta(
+                hours=hours_ago
+            )
+            work_request.save()
 
         idle_time = (
             Worker.objects.with_idle_time().get(pk=worker.pk)
@@ -514,7 +533,9 @@ class WorkerTests(TestCase):
         )
         self.assertFalse(worker.is_busy())
 
-        work_request = self.playground.create_worker_task(worker=worker)
+        work_request = self.playground.create_worker_task(
+            worker=worker, mark_pending=True
+        )
 
         self.assertEqual(work_request.status, WorkRequest.Statuses.PENDING)
         self.assertTrue(worker.is_busy())
@@ -539,8 +560,9 @@ class WorkerTests(TestCase):
         self.assertFalse(worker.is_at_capacity())
         for _ in range(3):
             self.assertFalse(worker.is_at_capacity())
-            work_request = self.playground.create_worker_task(worker=worker)
-            work_request.assign_worker(worker)
+            work_request = self.playground.create_worker_task(
+                worker=worker, mark_pending=True
+            )
         self.assertTrue(worker.is_at_capacity())
         work_request.mark_aborted()
         self.assertFalse(worker.is_at_capacity())
