@@ -33,6 +33,7 @@ from django.db.models import (
     OuterRef,
     Q,
     QuerySet,
+    Subquery,
 )
 from django.db.models.expressions import Case, When
 from django.db.models.functions import Extract, Greatest, Now
@@ -122,6 +123,16 @@ class WorkerQuerySet(QuerySet["Worker", A], Generic[A]):
         without a running work request assigned.
         """
         tasks = WorkRequest.objects.filter(worker=OuterRef("pk"))
+
+        last_completed_at = Subquery(
+            tasks.annotate(last_completed_at=Max("completed_at")).values(
+                "last_completed_at"
+            )
+        )
+        # Work-around https://code.djangoproject.com/ticket/28296
+        last_completed_at.query.set_group_by()
+        last_completed_at.query.clear_ordering()
+
         return self.annotate(
             idle_time=Case(
                 When(
@@ -130,9 +141,7 @@ class WorkerQuerySet(QuerySet["Worker", A], Generic[A]):
                 ),
                 When(
                     Exists(tasks),
-                    then=Extract(
-                        Now() - Max(tasks.values("completed_at")), "epoch"
-                    ),
+                    then=Extract(Now() - last_completed_at, "epoch"),
                 ),
                 default=Extract(
                     Now() - Greatest("registered_at", "instance_created_at"),
@@ -251,6 +260,23 @@ class WorkerManager(models.Manager["Worker"]):
                     pass
             index += 1
 
+    def get_or_create_scheduler(self) -> "Worker":
+        """
+        Return a new Worker representing the scheduler.
+
+        The scheduler of course can't dispatch tasks to itself, but
+        representing it as a worker is convenient because it allows the
+        scheduler to call back to the server to emit metrics.
+        """
+        try:
+            return self.get(name="scheduler", worker_type=WorkerType.SCHEDULER)
+        except Worker.DoesNotExist:
+            return self.create(
+                name="scheduler",
+                worker_type=WorkerType.SCHEDULER,
+                registered_at=timezone.now(),
+            )
+
     def get_or_create_celery(self) -> "Worker":
         """Return a new Worker representing the Celery task queue."""
         try:
@@ -313,10 +339,7 @@ class Worker(models.Model):
     dynamic_metadata_updated_at = models.DateTimeField(blank=True, null=True)
 
     worker_type = models.CharField(
-        max_length=8,
-        choices=WorkerType.choices,
-        default=WorkerType.EXTERNAL,
-        editable=False,
+        choices=WorkerType.choices, default=WorkerType.EXTERNAL, editable=False
     )
     # Only Celery workers currently support concurrency levels greater than
     # 1.
@@ -338,9 +361,16 @@ class Worker(models.Model):
             # Non-Celery workers must have a token.
             CheckConstraint(
                 name="%(app_label)s_%(class)s_celery_or_token",
-                check=Q(worker_type=WorkerType.CELERY)
+                check=Q(
+                    worker_type__in={WorkerType.CELERY, WorkerType.SCHEDULER}
+                )
                 | Q(activation_token__isnull=False)
-                | Q(token__isnull=False),
+                | Q(token__isnull=False)
+                # Exclude worker_pool tombstones
+                | Q(
+                    worker_pool_id__isnull=False,
+                    instance_created_at__isnull=True,
+                ),
             )
         ]
 
