@@ -1329,8 +1329,7 @@ class Playground(ArtifactPlayground):
         version: str = ...,
         src_architectures: set[str] | None = ...,
         binary: Literal[False],
-        binaries: list[str] | None = ...,
-        bin_architecture: str = ...,
+        binaries: list[tuple[str, str]] | None = ...,
         section: str = ...,
         priority: str = ...,
         workspace: Workspace | None = ...,
@@ -1348,8 +1347,7 @@ class Playground(ArtifactPlayground):
         version: str = ...,
         src_architectures: set[str] | None = ...,
         binary: Literal[True] = True,
-        binaries: list[str] | None = ...,
-        bin_architecture: str = ...,
+        binaries: list[tuple[str, str]] | None = ...,
         section: str = ...,
         priority: str = ...,
         workspace: Workspace | None = ...,
@@ -1367,8 +1365,7 @@ class Playground(ArtifactPlayground):
         version: str = ...,
         src_architectures: set[str] | None = ...,
         binary: Literal[True] = True,
-        binaries: list[str] | None = ...,
-        bin_architecture: str = ...,
+        binaries: list[tuple[str, str]] | None = ...,
         section: str = ...,
         priority: str = ...,
         workspace: Workspace | None = ...,
@@ -1386,8 +1383,7 @@ class Playground(ArtifactPlayground):
         version: str = "1.0-1",
         src_architectures: set[str] | None = None,
         binary: bool = True,
-        binaries: list[str] | None = None,
-        bin_architecture: str = "all",
+        binaries: list[tuple[str, str]] | None = None,
         section: str = "unknown",
         priority: str = "optional",
         workspace: Workspace | None = None,
@@ -1397,7 +1393,7 @@ class Playground(ArtifactPlayground):
     ) -> UploadArtifacts:
         """Create a set of artifacts for a package upload."""
         if binaries is None:
-            binaries = [src_name]
+            binaries = [(src_name, "all")]
         local_artifacts: dict[str, list[LocalArtifact[Any]]] = {
             "source": [],
             "binaries": [],
@@ -1413,7 +1409,6 @@ class Playground(ArtifactPlayground):
                 src_architectures=src_architectures,
                 binary=binary,
                 binaries=binaries,
-                bin_architecture=bin_architecture,
                 section=section,
                 priority=priority,
             )
@@ -1843,7 +1838,7 @@ class Playground(ArtifactPlayground):
         self,
         source: Artifact,
         *,
-        binary_name: str | None = None,
+        binaries: list[str] | None = None,
         architecture: str = "all",
         build_architecture: str | None = None,
         workflow: WorkRequest | None = None,
@@ -1891,30 +1886,35 @@ class Playground(ArtifactPlayground):
             buildlog_artifact, source, ArtifactRelation.Relations.RELATES_TO
         )
 
+        if binaries is None:
+            binaries = [
+                item["package"]
+                for item in source.data["dsc_fields"]["Package-List"]
+            ]
+
         # Add the binary upload
         upload_artifacts = self.create_upload_artifacts(
             src_name=name,
             version=version,
             source=False,
             binary=True,
-            binaries=[binary_name or name],
-            bin_architecture=architecture,
+            binaries=[(binary, architecture) for binary in binaries],
             workspace=workspace,
             work_request=work_request,
             created_by=work_request.created_by,
             create_files=True,
         )
-        assert len(upload_artifacts.binaries) == 1
-        self.create_artifact_relation(
-            upload_artifacts.binaries[0],
-            source,
-            ArtifactRelation.Relations.BUILT_USING,
-        )
-        self.create_artifact_relation(
-            buildlog_artifact,
-            upload_artifacts.binaries[0],
-            ArtifactRelation.Relations.RELATES_TO,
-        )
+        for binary in upload_artifacts.binaries:
+            self.create_artifact_relation(
+                binary,
+                source,
+                ArtifactRelation.Relations.BUILT_USING,
+            )
+            self.create_artifact_relation(
+                buildlog_artifact,
+                binary,
+                ArtifactRelation.Relations.RELATES_TO,
+            )
 
         if workflow is not None:
             work_request.add_event_reaction(
@@ -1923,7 +1923,7 @@ class Playground(ArtifactPlayground):
                     collection="internal@collections",
                     name_template=f"build-{architecture}",
                     variables={
-                        "binary_names": [binary_name or name],
+                        "binary_names": binaries,
                         "architecture": architecture,
                         "source_package_name": name,
                     },
