@@ -9,15 +9,23 @@
 
 """debusine celery configuration."""
 
+import logging
 import os
+import secrets
+from pathlib import Path
 from types import SimpleNamespace
-from typing import Any
+from typing import Any, TYPE_CHECKING
 
 from celery import Celery, Task, states
 from celery.result import AsyncResult
 from celery.signals import before_task_publish, worker_init, worker_shutdown
 from celery.worker import WorkController
 
+from debusine.client.debusine import Debusine
+
+if TYPE_CHECKING:
+    from debusine.db.models import Token
+
 os.environ.setdefault("DJANGO_SETTINGS_MODULE", "debusine.project.settings")
 
 
@@ -77,6 +85,45 @@ def store_pending_state(
     )
 
 
+class CeleryToken:
+    """Manage a Celery worker's token."""
+
+    token: "Token"
+    client: Debusine
+
+    def __init__(self, name: str) -> None:
+        """Read an existing token key, or generate one if there is none."""
+        from django.conf import settings
+
+        from debusine.db.models import Token
+        from debusine.utils import atomic_writer
+
+        path = Path(settings.DEBUSINE_TOKENS_DIRECTORY, name)
+        try:
+            key = path.read_text()
+        except FileNotFoundError:
+            key = secrets.token_hex(32)
+            with atomic_writer(path, mode="w", chmod=0o600) as fd:
+                fd.write(key)
+
+        self.token, _ = Token.objects.get_or_create(
+            hash=Token._generate_hash(key)
+        )
+        self.client = Debusine(
+            f"https://{settings.DEBUSINE_FQDN}/api",
+            key,
+            logger=logging.getLogger(),
+        )
+
+
+_celery_token: CeleryToken | None = None
+
+
+def get_celery_token() -> CeleryToken | None:
+    """Get information about the current Celery worker token."""
+    return _celery_token
+
+
 # mypy complains that worker_init.connect is untyped, which is true, but we
 # can't fix that here.
 @worker_init.connect  # type: ignore[misc]
@@ -88,16 +135,27 @@ def connect_worker(*, sender: WorkController, **kwargs: Any) -> None:
     from debusine.tasks.models import WorkerType
     from debusine.worker.system_information import system_metadata
 
+    worker: Worker | None = None
+
     if sender.hostname.startswith("celery@"):
         worker = Worker.objects.get_or_create_celery()
         worker.concurrency = sender.concurrency
         worker.save()
-
         metadata = {
             **system_metadata(WorkerType.CELERY),
             **analyze_worker_all_executors(),
             **BaseTask.analyze_worker_all_tasks(),
         }
+    elif sender.hostname.startswith("scheduler@"):
+        worker = Worker.objects.get_or_create_scheduler()
+        metadata = {**system_metadata(WorkerType.SCHEDULER)}
+
+    if worker is not None:
+        global _celery_token
+        _celery_token = CeleryToken(sender.hostname.split("@")[0])
+        if worker.token is None:
+            worker.token = _celery_token.token
+        worker.save()
 
         worker.set_dynamic_metadata(metadata)
         worker.mark_connected()
@@ -112,3 +170,8 @@ def disconnect_worker(*, sender: WorkController, **kwargs: Any) -> None:
 
     if sender.hostname.startswith("celery@"):
         Worker.objects.get_or_create_celery().mark_disconnected()
+    elif sender.hostname.startswith("scheduler@"):
+        Worker.objects.get_or_create_scheduler().mark_disconnected()
+
+    global _celery_token
+    _celery_token = None
