@@ -10,22 +10,22 @@
 """Unit tests for Debusine Celery configuration."""
 
 import json
-from collections.abc import Generator
-from contextlib import contextmanager
 from multiprocessing import Lock
+from pathlib import Path
 from typing import Any
 from unittest import mock
 
 from celery import shared_task, signals, states
 from celery.contrib.testing.worker import start_worker
 from celery.utils.nodenames import gethostname, nodename
+from django.conf import settings
 from django.test import override_settings
 from django_celery_results.models import TaskResult
 
-from debusine.db.models import Worker
-from debusine.project.celery import app, make_app
+from debusine.db.models import Token, Worker
+from debusine.project.celery import CeleryToken, app, get_celery_token, make_app
 from debusine.tasks.models import WorkerType
-from debusine.test.django import TransactionTestCase
+from debusine.test.django import TestCase, TransactionTestCase
 
 _lock = Lock()
 
@@ -129,6 +129,46 @@ class TestStorePendingState(TransactionTestCase):
             signals.worker_shutdown.send(sender=celery_worker)
 
 
+class TestCeleryToken(TestCase):
+    """Tests for :py:class:`CeleryToken`."""
+
+    def test_existing_token(self) -> None:
+        tokens_dir = self.create_temporary_directory()
+        key = "existing-key"
+        Path(tokens_dir, "test").write_text(key)
+
+        with override_settings(
+            DEBUSINE_FQDN="debusine.example.org",
+            DEBUSINE_TOKENS_DIRECTORY=str(tokens_dir),
+        ):
+            celery_token = CeleryToken("test")
+
+        self.assertEqual(celery_token.token.hash, Token._generate_hash(key))
+        self.assertEqual(
+            celery_token.client.base_api_url, "https://debusine.example.org/api"
+        )
+        self.assertEqual(celery_token.client.token, key)
+        self.assertIsNone(celery_token.client.scope)
+
+    def test_new_token(self) -> None:
+        tokens_dir = self.create_temporary_directory()
+
+        with override_settings(
+            DEBUSINE_FQDN="debusine.example.org",
+            DEBUSINE_TOKENS_DIRECTORY=str(tokens_dir),
+        ):
+            celery_token = CeleryToken("test")
+
+        key = Path(tokens_dir, "test").read_text()
+        self.assertEqual(len(key), 64)
+        self.assertEqual(celery_token.token.hash, Token._generate_hash(key))
+        self.assertEqual(
+            celery_token.client.base_api_url, "https://debusine.example.org/api"
+        )
+        self.assertEqual(celery_token.client.token, key)
+        self.assertIsNone(celery_token.client.scope)
+
+
 @override_settings(CELERY_BROKER_URL="memory://")
 class TestConnectWorker(TransactionTestCase):
     """Tests for :py:func:`connect_worker`."""
@@ -136,31 +176,21 @@ class TestConnectWorker(TransactionTestCase):
     def setUp(self) -> None:
         super().setUp()
         self.app = make_app()
-
-    @contextmanager
-    def patch_anon_nodename(self, hostname: str) -> Generator[None]:
-        """
-        Patch Celery's hostname.
-
-        Celery >= 5.5 allows passing a custom hostname to
-        :py:func:`start_worker`, but earlier versions don't.
-        """
-        with mock.patch(
-            "celery.contrib.testing.worker.anon_nodename", return_value=hostname
-        ):
-            yield
+        Worker.objects.filter(
+            worker_type__in={WorkerType.CELERY, WorkerType.SCHEDULER}
+        ).delete()
 
     def test_server_task_worker(self) -> None:
         """The main worker creates a Worker instance that runs server tasks."""
         with (
-            self.patch_anon_nodename(
-                hostname=nodename("celery", gethostname())
-            ),
             mock.patch("debusine.django.django_utils.test_data_override"),
             # Set a concurrency that's unlikely to be the actual number of
             # CPUs we have.
             start_worker(
-                self.app, concurrency=5, perform_ping_check=False
+                self.app,
+                concurrency=5,
+                perform_ping_check=False,
+                hostname=nodename("celery", gethostname()),
             ) as celery_worker,
         ):
             worker = Worker.objects.get(worker_type=WorkerType.CELERY)
@@ -169,50 +199,152 @@ class TestConnectWorker(TransactionTestCase):
                 worker.dynamic_metadata["system:worker_type"], WorkerType.CELERY
             )
             self.assertIsNotNone(worker.connected_at)
+            self.assertFalse(
+                Worker.objects.filter(worker_type=WorkerType.SCHEDULER).exists()
+            )
+
+            token = Token.objects.get_token_or_none(
+                Path(settings.DEBUSINE_TOKENS_DIRECTORY, "celery").read_text()
+            )
+            assert token is not None
+            self.assertEqual(token.worker, worker)
+            celery_token = get_celery_token()
+            assert celery_token is not None
+            self.assertEqual(celery_token.token, token)
 
             signals.worker_shutdown.send(sender=celery_worker)
 
             worker.refresh_from_db()
             self.assertIsNone(worker.connected_at)
+            self.assertFalse(
+                Worker.objects.filter(worker_type=WorkerType.SCHEDULER).exists()
+            )
+
+    def test_server_task_worker_token_already_exists(self) -> None:
+        existing_worker = Worker.objects.get_or_create_celery()
+        existing_worker.token = CeleryToken("celery").token
+        existing_worker.save()
 
-    def test_scheduler_worker(self) -> None:
-        """The scheduler worker does not create a Worker instance."""
         with (
-            self.patch_anon_nodename(
-                hostname=nodename("scheduler", gethostname())
-            ),
             mock.patch("debusine.django.django_utils.test_data_override"),
+            # Set a concurrency that's unlikely to be the actual number of
+            # CPUs we have.
             start_worker(
-                app, concurrency=1, perform_ping_check=False
+                self.app,
+                concurrency=5,
+                perform_ping_check=False,
+                hostname=nodename("celery", gethostname()),
             ) as celery_worker,
         ):
-            self.assertFalse(
-                Worker.objects.filter(worker_type=WorkerType.CELERY).exists()
+            worker = Worker.objects.get(worker_type=WorkerType.CELERY)
+            self.assertEqual(worker, existing_worker)
+
+            token = Token.objects.get_token_or_none(
+                Path(settings.DEBUSINE_TOKENS_DIRECTORY, "celery").read_text()
             )
+            assert token is not None
+            self.assertEqual(token.worker, worker)
+            self.assertEqual(token, existing_worker.token)
+            celery_token = get_celery_token()
+            assert celery_token is not None
+            self.assertEqual(celery_token.token, token)
 
             signals.worker_shutdown.send(sender=celery_worker)
 
+    def test_scheduler_worker(self) -> None:
+        """The scheduler worker creates a Worker instance."""
+        with (
+            mock.patch("debusine.django.django_utils.test_data_override"),
+            start_worker(
+                app,
+                concurrency=1,
+                perform_ping_check=False,
+                hostname=nodename("scheduler", gethostname()),
+            ) as celery_worker,
+        ):
+            worker = Worker.objects.get(worker_type=WorkerType.SCHEDULER)
+            self.assertEqual(worker.concurrency, 1)
+            self.assertEqual(
+                worker.dynamic_metadata["system:worker_type"],
+                WorkerType.SCHEDULER,
+            )
+            self.assertIsNotNone(worker.connected_at)
             self.assertFalse(
                 Worker.objects.filter(worker_type=WorkerType.CELERY).exists()
             )
 
+            token = Token.objects.get_token_or_none(
+                Path(
+                    settings.DEBUSINE_TOKENS_DIRECTORY, "scheduler"
+                ).read_text()
+            )
+            assert token is not None
+            self.assertEqual(token.worker, worker)
+            celery_token = get_celery_token()
+            assert celery_token is not None
+            self.assertEqual(celery_token.token, token)
+
+            signals.worker_shutdown.send(sender=celery_worker)
+
+            worker.refresh_from_db()
+            self.assertIsNone(worker.connected_at)
+            self.assertFalse(
+                Worker.objects.filter(worker_type=WorkerType.CELERY).exists()
+            )
+
+    def test_scheduler_worker_token_already_exists(self) -> None:
+        existing_worker = Worker.objects.get_or_create_scheduler()
+        existing_worker.token = CeleryToken("scheduler").token
+        existing_worker.save()
+
+        with (
+            mock.patch("debusine.django.django_utils.test_data_override"),
+            start_worker(
+                app,
+                concurrency=1,
+                perform_ping_check=False,
+                hostname=nodename("scheduler", gethostname()),
+            ) as celery_worker,
+        ):
+            worker = Worker.objects.get(worker_type=WorkerType.SCHEDULER)
+            self.assertEqual(worker, existing_worker)
+
+            token = Token.objects.get_token_or_none(
+                Path(
+                    settings.DEBUSINE_TOKENS_DIRECTORY, "scheduler"
+                ).read_text()
+            )
+            assert token is not None
+            self.assertEqual(token.worker, worker)
+            self.assertEqual(token, existing_worker.token)
+            celery_token = get_celery_token()
+            assert celery_token is not None
+            self.assertEqual(celery_token.token, token)
+
+            signals.worker_shutdown.send(sender=celery_worker)
+
     def test_provisioner_worker(self) -> None:
         """The provisioner worker does not create a Worker instance."""
         with (
-            self.patch_anon_nodename(
-                hostname=nodename("provisioner", gethostname())
-            ),
             mock.patch("debusine.django.django_utils.test_data_override"),
             start_worker(
-                app, concurrency=1, perform_ping_check=False
+                app,
+                concurrency=1,
+                perform_ping_check=False,
+                hostname=nodename("provisioner", gethostname()),
             ) as celery_worker,
         ):
             self.assertFalse(
-                Worker.objects.filter(worker_type=WorkerType.CELERY).exists()
+                Worker.objects.filter(
+                    worker_type__in={WorkerType.CELERY, WorkerType.SCHEDULER}
+                ).exists()
             )
+            self.assertIsNone(get_celery_token())
 
             signals.worker_shutdown.send(sender=celery_worker)
 
             self.assertFalse(
-                Worker.objects.filter(worker_type=WorkerType.CELERY).exists()
+                Worker.objects.filter(
+                    worker_type__in={WorkerType.CELERY, WorkerType.SCHEDULER}
+                ).exists()
             )
