@@ -546,3 +546,10 @@ def reconstruct_lookup(
         return _reconstruct_collection_lookup(
             result.collection.id, workflow_root=workflow_root
         )
+
+
+def artifact_ids(results: Sequence[LookupResult]) -> list[int]:
+    """Return the artifact ids."""
+    return [
+        result.artifact.id for result in results if result.artifact is not None
+    ]
