@@ -9,7 +9,7 @@
 
 """Unit tests for TaskHistoryManager."""
 
-from datetime import datetime, timedelta
+import datetime as dt
 
 from django.db.models import IntegerField, Q
 from django.db.models.functions import Cast
@@ -68,7 +68,9 @@ class TaskHistoryManagerTests(TestCase):
         collection: Collection | None = None,
     ) -> CollectionItem:
         """Create and return a historical task run in a collection."""
-        created_at = datetime(2020, 1, 1) + timedelta(seconds=work_request_id)
+        created_at = dt.datetime(2020, 1, 1) + dt.timedelta(
+            seconds=work_request_id
+        )
         return CollectionItem.objects.create_from_bare_data(
             BareDataCategory.HISTORICAL_TASK_RUN,
             parent_collection=collection or self.collection,
@@ -345,9 +347,7 @@ class TaskHistoryManagerTests(TestCase):
         # Matching items are pruned, but the most recent success is kept
         # despite not being within the window of old items to keep.
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .order_by("id"),
+            self.collection.child_items.active().order_by("id"),
             [items[0], *items[2:]],
         )
 
@@ -397,9 +397,7 @@ class TaskHistoryManagerTests(TestCase):
         # Matching items are pruned, but the most recent failure is kept
         # despite not being within the window of old items to keep.
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .order_by("id"),
+            self.collection.child_items.active().order_by("id"),
             [items[0], *items[2:]],
         )
 
@@ -432,10 +430,7 @@ class TaskHistoryManagerTests(TestCase):
         )
 
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .order_by("id"),
-            items[3:],
+            self.collection.child_items.active().order_by("id"), items[3:]
         )
 
     def test_remove_item_bare_data(self) -> None:
@@ -641,10 +636,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
         )
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
-            [items[0]],
+            self.collection.child_items.active().filter(condition), [items[0]]
         )
         self.assertCountEqual(
             lookup_multiple(
@@ -703,9 +695,7 @@ class TaskHistoryManagerTests(TestCase):
             ),
         )
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
+            self.collection.child_items.active().filter(condition),
             items[:2],
             ordered=False,
         )
@@ -758,9 +748,7 @@ class TaskHistoryManagerTests(TestCase):
         # Ideally we'd check the query itself, but digging into the
         # WorkRequest subquery is hard, so we just check the results.
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
+            self.collection.child_items.active().filter(condition),
             items[:2],
             ordered=False,
         )
@@ -822,9 +810,7 @@ class TaskHistoryManagerTests(TestCase):
         # Ideally we'd check the query itself, but digging into the
         # WorkRequest subquery is hard, so we just check the results.
         self.assertQuerySetEqual(
-            CollectionItem.objects.active()
-            .filter(parent_collection=self.collection)
-            .filter(condition),
+            self.collection.child_items.active().filter(condition),
             items[:4],
             ordered=False,
         )
