@@ -426,7 +426,7 @@ class OnWorkRequestCompletedConsumer(DebusineAsyncWebsocketConsumer):
             wr
             async for wr in WorkRequest.objects.filter(
                 **filter_kwargs
-            ).order_by("completed_at")
+            ).order_by("completed_at", "id")
         ]
 
     async def _send_pending_work_requests_since_last_completed(self) -> None:
