@@ -15,7 +15,7 @@ from django.contrib.auth import get_user_model
 from django.core.management import CommandParser
 
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import Users
+from debusine.server.management.management_utils import Users
 
 
 class Command(DebusineBaseCommand):
