@@ -454,7 +454,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
         self.assertTrue(
             asset.roles.filter(group=group1, role=AssetRoles.OWNER).exists()
         )
-        self.assertFalse(asset.usage.all().exists())
+        self.assertFalse(asset.usage.exists())
 
     def test_grant_role_usage(self) -> None:
         workspace1 = self.playground.create_workspace(name="workspace1")
@@ -490,7 +490,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
             call_command(
                 "asset", "grant_role", str(asset.id), "unknown", group1.name
             )
-        self.assertFalse(asset.roles.all().exists())
+        self.assertFalse(asset.roles.exists())
 
     def test_grant_role_unknown_group(self) -> None:
         asset = self.playground.create_signing_key_asset()
@@ -500,7 +500,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
             call_command(
                 "asset", "grant_role", str(asset.id), "owner", "unknown"
             )
-        self.assertFalse(asset.roles.all().exists())
+        self.assertFalse(asset.roles.exists())
 
     def test_grant_role_unknown_scope(self) -> None:
         asset = self.playground.create_signing_key_asset()
@@ -515,7 +515,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 "signer",
                 group1.name,
             )
-        self.assertFalse(asset.usage.all().exists())
+        self.assertFalse(asset.usage.exists())
 
     def test_grant_role_unknown_workspace(self) -> None:
         asset = self.playground.create_signing_key_asset()
@@ -532,7 +532,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 "signer",
                 group1.name,
             )
-        self.assertFalse(asset.usage.all().exists())
+        self.assertFalse(asset.usage.exists())
 
     def test_grant_role_invalid_scope(self) -> None:
         asset = self.playground.create_signing_key_asset()
@@ -553,7 +553,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 "signer",
                 group1.name,
             )
-        self.assertFalse(asset.usage.all().exists())
+        self.assertFalse(asset.usage.exists())
 
     def test_grant_role_without_workspace(self) -> None:
         asset = self.playground.create_cloud_provider_account_asset()
@@ -569,7 +569,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 "owner",
                 group1.name,
             )
-        self.assertFalse(asset.roles.all().exists())
+        self.assertFalse(asset.roles.exists())
 
     def test_grant_role_usage_without_workspace(self) -> None:
         workspace1 = self.playground.create_workspace(name="workspace1")
@@ -604,7 +604,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
         self.assertEqual(stdout, "")
         self.assertEqual(stderr, "")
 
-        self.assertFalse(asset.roles.all().exists())
+        self.assertFalse(asset.roles.exists())
 
     def test_revoke_role_usage(self) -> None:
         asset = self.playground.create_signing_key_asset()
@@ -629,7 +629,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
         self.assertEqual(stdout, "")
         self.assertEqual(stderr, "")
 
-        self.assertFalse(asset_usage.roles.all().exists())
+        self.assertFalse(asset_usage.roles.exists())
 
     def test_revoke_role_without_workspace(self) -> None:
         asset = self.playground.create_cloud_provider_account_asset()
@@ -645,7 +645,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
                 "owner",
                 group1.name,
             )
-        self.assertFalse(asset.roles.all().exists())
+        self.assertFalse(asset.roles.exists())
 
     def test_revoke_role_usage_without_workspace(self) -> None:
         workspace1 = self.playground.create_workspace(name="workspace1")
