@@ -53,14 +53,10 @@ class ManageWorkerCommandTests(TestCase):
         self.token.refresh_from_db()
 
         work_request_running = self.playground.create_worker_task(
-            worker=self.worker,
-            status=WorkRequest.Statuses.RUNNING,
-            started_at=timezone.now(),
+            worker=self.worker, mark_running=True, started_at=timezone.now()
         )
         work_request_pending = self.playground.create_worker_task(
-            worker=self.worker,
-            status=WorkRequest.Statuses.PENDING,
-            started_at=timezone.now(),
+            worker=self.worker, mark_pending=True, started_at=timezone.now()
         )
         work_request_finished = self.playground.create_worker_task(
             worker=self.worker,
