@@ -360,7 +360,7 @@ class AddFileStoreScopeCommandTests(TestCase):
             "--download-priority",
             "200",
             "--populate",
-            "--drain",
+            "--no-drain",
             "--drain-to",
             "other_store",
             "--soft-max-size",
@@ -376,7 +376,7 @@ class AddFileStoreScopeCommandTests(TestCase):
         self.assertEqual(file_store_extra.upload_priority, 50)
         self.assertEqual(file_store_extra.download_priority, 200)
         self.assertTrue(file_store_extra.populate)
-        self.assertTrue(file_store_extra.drain)
+        self.assertFalse(file_store_extra.drain)
         self.assertEqual(file_store_extra.drain_to, "other_store")
         self.assertFalse(file_store_extra.read_only)
         self.assertFalse(file_store_extra.write_only)
@@ -477,9 +477,9 @@ class EditFileStoreScopeCommandTests(TestCase):
                 "upload_priority": 100,
                 "download_priority": 100,
                 "populate": True,
-                "drain": True,
+                "drain": False,
                 "drain_to": "other_store",
-                "read_only": True,
+                "read_only": False,
                 "write_only": True,
                 "soft_max_size": 1048576,
             },
@@ -496,9 +496,9 @@ class EditFileStoreScopeCommandTests(TestCase):
         self.assertEqual(file_store_extra.upload_priority, 100)
         self.assertEqual(file_store_extra.download_priority, 100)
         self.assertTrue(file_store_extra.populate)
-        self.assertTrue(file_store_extra.drain)
+        self.assertFalse(file_store_extra.drain)
         self.assertEqual(file_store_extra.drain_to, "other_store")
-        self.assertTrue(file_store_extra.read_only)
+        self.assertFalse(file_store_extra.read_only)
         self.assertTrue(file_store_extra.write_only)
         self.assertEqual(file_store_extra.soft_max_size, 1048576)
 
