@@ -145,7 +145,7 @@ class VacuumStorageCommandTransactionTests(
         artifact_2, _ = self.playground.create_artifact(
             paths=["README"], create_files=True
         )
-        file_in_artifact_2 = artifact_2.fileinartifact_set.all().first()
+        file_in_artifact_2 = artifact_2.fileinartifact_set.first()
         assert file_in_artifact_2 is not None
         file_in_artifact_2.complete = False
         file_in_artifact_2.save()
