@@ -8,8 +8,8 @@
 # contained in the LICENSE file.
 
 """Tests for the management command workspace."""
+import datetime as dt
 import io
-from datetime import timedelta
 from typing import ClassVar
 
 import yaml
@@ -100,7 +100,9 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             )
             self.assertEqual(role.role, WorkspaceRole.Roles.OWNER)
             self.assertFalse(workspace.public)
-            self.assertEqual(workspace.default_expiration_delay, timedelta(30))
+            self.assertEqual(
+                workspace.default_expiration_delay, dt.timedelta(30)
+            )
             self.assertQuerySetEqual(
                 workspace.collections.values_list("category", flat=True),
                 SINGLETON_COLLECTION_CATEGORIES,
@@ -120,7 +122,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         )
         workspace = Workspace.objects.get(scope=self.scope, name="myws-new")
         self.assertTrue(workspace.public)
-        self.assertEqual(workspace.default_expiration_delay, timedelta(0))
+        self.assertEqual(workspace.default_expiration_delay, dt.timedelta(0))
         self.assertQuerySetEqual(
             workspace.collections.values_list("category", flat=True), []
         )
@@ -137,7 +139,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         )
         workspace.refresh_from_db()
         self.assertFalse(workspace.public)
-        self.assertEqual(workspace.default_expiration_delay, timedelta(10))
+        self.assertEqual(workspace.default_expiration_delay, dt.timedelta(10))
         role = WorkspaceRole.objects.get(
             resource=workspace, group__name="Owners-myws2"
         )
@@ -150,7 +152,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
             "localtest/myws-new",
         )
         self.assertFalse(workspace.public)
-        self.assertEqual(workspace.default_expiration_delay, timedelta(10))
+        self.assertEqual(workspace.default_expiration_delay, dt.timedelta(10))
 
     def test_rename_scenario(self) -> None:
         """Test rename command."""
@@ -424,7 +426,7 @@ class WorkspaceCommandsTests(TabularOutputTests, TestCase):
         sbuild_template = self.playground.create_workflow_template(
             name="Build package",
             task_name="sbuild",
-            task_data={},
+            static_parameters={},
             workspace=workspace,
         )
 
