@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,31 +9,36 @@
 
 """Tests for the TokenLastSeenAtMiddleware."""
 from collections.abc import Mapping
-from typing import Any, TYPE_CHECKING
+from typing import Any, TYPE_CHECKING, cast
 
+from asgiref.typing import (
+    ASGIReceiveCallable,
+    ASGISendCallable,
+    WebSocketReceiveEvent,
+)
 from channels.db import database_sync_to_async
 from django.utils import timezone
 
 from debusine.db.context import context
+from debusine.server.consumers import ChannelScopeWithToken
 from debusine.server.middlewares.token_last_seen_at import (
     TokenLastSeenAtMiddlewareChannels,
 )
 from debusine.test.django import TransactionTestCase
 
 if TYPE_CHECKING:
-    from django.core.handlers.asgi import _ReceiveCallback, _SendCallback
-
-    _ReceiveCallback  # fake usage for vulture
-    _SendCallback  # fake usage for vulture
+    from channels.consumer import _ChannelScope
 
 
 class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
     """Test TokenLastSeenAtMiddlewareChannels."""
 
     @classmethod
-    async def _noop_receive(cls) -> dict[str, Any]:  # pragma: no cover
+    async def _noop_receive(cls) -> WebSocketReceiveEvent:  # pragma: no cover
         """No-op receive callback for testing."""
-        return {}
+        return WebSocketReceiveEvent(
+            type="websocket.receive", bytes=None, text=""
+        )
 
     @classmethod
     async def _noop_send(cls, message: Mapping[str, Any]) -> None:
@@ -44,27 +49,31 @@ class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
         token = await database_sync_to_async(
             self.playground.create_bare_token
         )()
+        assert token is not None
 
         self.assertIsNone(token.last_seen_at)
 
         before = timezone.now()
 
-        scope_upstream = {
-            "type": "http",
-            "method": "GET",
-            "headers": [(b"token", token.key.encode("utf-8"))],
-        }
+        scope_upstream = cast(
+            "_ChannelScope",
+            {
+                "type": "websocket",
+                "headers": [(b"token", token.key.encode("utf-8"))],
+            },
+        )
 
         async def mock_app(
-            scope_downstream: dict[str, Any],
-            receive: "_ReceiveCallback",  # noqa: U100
-            send: "_SendCallback",  # noqa: U100
+            scope_downstream: ChannelScopeWithToken,
+            receive: ASGIReceiveCallable,  # noqa: U100
+            send: ASGISendCallable,  # noqa: U100
         ) -> None:
             """Mock ASGI app to inspect the modified scope."""
             await token.arefresh_from_db()
             self.assertEqual(scope_downstream["token"], token)
             self.assertIsNone(context.worker_token)
             # self.assertEqual(context.worker_token, token)
+            assert token.last_seen_at is not None
             self.assertGreaterEqual(token.last_seen_at, before)
             self.assertLess(token.last_seen_at, timezone.now())
 
@@ -77,26 +86,30 @@ class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
         token = await database_sync_to_async(
             self.playground.create_worker_token
         )()
+        assert token is not None
 
         self.assertIsNone(token.last_seen_at)
 
         before = timezone.now()
 
-        scope_upstream = {
-            "type": "http",
-            "method": "GET",
-            "headers": [(b"token", token.key.encode("utf-8"))],
-        }
+        scope_upstream = cast(
+            "_ChannelScope",
+            {
+                "type": "websocket",
+                "headers": [(b"token", token.key.encode("utf-8"))],
+            },
+        )
 
         async def mock_app(
-            scope_downstream: dict[str, Any],
-            receive: "_ReceiveCallback",  # noqa: U100
-            send: "_SendCallback",  # noqa: U100
+            scope_downstream: ChannelScopeWithToken,
+            receive: ASGIReceiveCallable,  # noqa: U100
+            send: ASGISendCallable,  # noqa: U100
         ) -> None:
             """Mock ASGI app to inspect the modified scope."""
             await token.arefresh_from_db()
             self.assertEqual(scope_downstream["token"], token)
             self.assertEqual(context.worker_token, token)
+            assert token.last_seen_at is not None
             self.assertGreaterEqual(token.last_seen_at, before)
             self.assertLess(token.last_seen_at, timezone.now())
 
@@ -106,12 +119,14 @@ class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
 
     async def test_request_without_token(self) -> None:
         """Request without a token."""
-        scope_upstream = {"type": "http", "method": "GET", "headers": {}}
+        scope_upstream = cast(
+            "_ChannelScope", {"type": "websocket", "headers": {}}
+        )
 
         async def mock_app(
-            scope_downstream: dict[str, Any],
-            receive: "_ReceiveCallback",  # noqa: U100
-            send: "_SendCallback",  # noqa: U100
+            scope_downstream: ChannelScopeWithToken,
+            receive: ASGIReceiveCallable,  # noqa: U100
+            send: ASGISendCallable,  # noqa: U100
         ) -> None:
             """Mock ASGI app to inspect the modified scope."""
             self.assertIsNone(scope_downstream["token"])
@@ -123,16 +138,18 @@ class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
 
     async def test_request_token_key_does_not_exist(self) -> None:
         """Request where the Token key in the header does not exist."""
-        scope_upstream = {
-            "type": "http",
-            "method": "GET",
-            "headers": {"token": "DOES-NOT-EXIST"},
-        }
+        scope_upstream = cast(
+            "_ChannelScope",
+            {
+                "type": "websocket",
+                "headers": {"token": "DOES-NOT-EXIST"},
+            },
+        )
 
         async def mock_app(
-            scope_downstream: dict[str, Any],
-            receive: "_ReceiveCallback",  # noqa: U100
-            send: "_SendCallback",  # noqa: U100
+            scope_downstream: ChannelScopeWithToken,
+            receive: ASGIReceiveCallable,  # noqa: U100
+            send: ASGISendCallable,  # noqa: U100
         ) -> None:
             """Mock ASGI app to inspect the modified scope."""
             self.assertFalse(hasattr(scope_downstream, "token"))
@@ -147,16 +164,18 @@ class TokenLastSeenAtMiddlewareChannelsTests(TransactionTestCase):
             self.playground.create_worker_token
         )(enabled=False)
 
-        scope_upstream = {
-            "type": "http",
-            "method": "GET",
-            "headers": [(b"token", token.key.encode("utf-8"))],
-        }
+        scope_upstream = cast(
+            "_ChannelScope",
+            {
+                "type": "websocket",
+                "headers": [(b"token", token.key.encode("utf-8"))],
+            },
+        )
 
         async def mock_app(
-            scope_downstream: dict[str, Any],
-            receive: "_ReceiveCallback",  # noqa: U100
-            send: "_SendCallback",  # noqa: U100
+            scope_downstream: ChannelScopeWithToken,
+            receive: ASGIReceiveCallable,  # noqa: U100
+            send: ASGISendCallable,  # noqa: U100
         ) -> None:
             """Mock ASGI app to inspect the modified scope."""
             self.assertEqual(scope_downstream["token"], token)
