@@ -9,8 +9,8 @@
 
 """OpenMetrics statistics for Debusine."""
 
+import datetime as dt
 from collections.abc import Iterable, Sequence
-from datetime import datetime, timedelta
 from functools import cache
 from itertools import product
 from typing import Any, cast
@@ -30,7 +30,7 @@ from django.db.models import (
     Sum,
 )
 from django.db.models.expressions import Combinable
-from django.db.models.functions import Extract
+from django.db.models.functions import Coalesce, Extract
 from django.utils import timezone
 from prometheus_client.core import (
     GaugeMetricFamily,
@@ -255,7 +255,7 @@ class DebusineCollector(Collector):
             buckets[f'b{days}d'] = Count(
                 "created_by",
                 distinct=True,
-                filter=Q(created_at__gte=now - timedelta(days=days)),
+                filter=Q(created_at__gte=now - dt.timedelta(days=days)),
             )
         buckets['binf'] = Count("created_by", distinct=True)
         return buckets
@@ -340,6 +340,9 @@ class DebusineCollector(Collector):
                 "task_name",
                 "scope",
                 "status",
+                "build_architecture",
+                # TODO: For compatibility; remove this once existing graphs
+                # have been updated.
                 "host_architecture",
                 "backend",
             ),
@@ -350,7 +353,7 @@ class DebusineCollector(Collector):
             "task_name",
             "workspace__scope__name",
             "status",
-            "task_data__host_architecture",
+            "task_data__build_architecture",
             "task_data__backend",
         ).annotate(count=Count("id")):
             gauge.add_metric(
@@ -359,13 +362,16 @@ class DebusineCollector(Collector):
                     stat["task_name"],
                     stat["workspace__scope__name"],
                     stat["status"],
-                    stat["task_data__host_architecture"] or "",
+                    stat["task_data__build_architecture"] or "",
+                    # TODO: For compatibility; remove this once existing
+                    # graphs have been updated.
+                    stat["task_data__build_architecture"] or "",
                     stat["task_data__backend"] or "",
                 ),
                 stat["count"],
             )
-            if stat["task_data__host_architecture"]:
-                seen_architectures.add(stat["task_data__host_architecture"])
+            if stat["task_data__build_architecture"]:
+                seen_architectures.add(stat["task_data__build_architecture"])
         for task_type in TaskTypes:
             architectures: Iterable[str] = ("",)
             backends: Iterable[str] = ("",)
@@ -378,6 +384,9 @@ class DebusineCollector(Collector):
                 self._scopes(),
                 WorkRequest.Statuses,
                 architectures,
+                # TODO: For compatibility; remove this once existing graphs
+                # have been updated.
+                architectures,
                 backends,
             )
 
@@ -392,34 +401,46 @@ class DebusineCollector(Collector):
             "busy",
             "worker_type",
             "worker_pool",
-            "host_architecture",
+            "native_architecture",
         ]
-        architectures: set[str] = set()
-        host_architectures: set[str] = set()
-        for archs, host_arch in (
-            Worker.objects.values_list(
-                "static_metadata__system:architectures",
+        # TODO: Drop system:host_architecture fallback once all workers have
+        # been updated.
+        workers = Worker.objects.annotate(
+            static_native_architecture=Coalesce(
+                "static_metadata__system:native_architecture",
                 "static_metadata__system:host_architecture",
+            ),
+            dynamic_native_architecture=Coalesce(
+                "dynamic_metadata__system:native_architecture",
+                "dynamic_metadata__system:host_architecture",
+            ),
+        )
+        architectures: set[str] = set()
+        native_architectures: set[str] = set()
+        for archs, native_arch in (
+            workers.values_list(
+                "static_metadata__system:architectures",
+                "static_native_architecture",
             )
             .distinct(
                 "static_metadata__system:architectures",
-                "static_metadata__system:host_architecture",
+                "static_native_architecture",
             )
             .union(
-                Worker.objects.values_list(
+                workers.values_list(
                     "dynamic_metadata__system:architectures",
-                    "dynamic_metadata__system:host_architecture",
+                    "dynamic_native_architecture",
                 ).distinct(
                     "dynamic_metadata__system:architectures",
-                    "dynamic_metadata__system:host_architecture",
+                    "dynamic_native_architecture",
                 )
             )
         ):
             if archs:
                 for arch in archs:
                     architectures.add(arch)
-            if host_arch:
-                host_architectures.add(host_arch)
+            if native_arch:
+                native_architectures.add(native_arch)
 
         for arch in architectures:
             labels.append(f"architecture_{arch}")
@@ -427,17 +448,17 @@ class DebusineCollector(Collector):
         gauge = ZeroGauge(name, description, labels=labels)
 
         for stat in (
-            Worker.objects.values(
+            workers.values(
                 "worker_type",
                 "worker_pool__name",
-                "static_metadata__system:host_architecture",
-                "dynamic_metadata__system:host_architecture",
+                "static_native_architecture",
+                "dynamic_native_architecture",
                 "static_metadata__system:architectures",
                 "dynamic_metadata__system:architectures",
             )
             .annotate(
-                connected=ExpressionWrapper(
-                    Q(connected_at__isnull=False), output_field=BooleanField()
+                connected=Exists(
+                    Worker.objects.connected().filter(pk=OuterRef("pk"))
                 ),
                 busy=Exists(
                     WorkRequest.objects.filter(
@@ -456,8 +477,8 @@ class DebusineCollector(Collector):
                 str(int(stat["busy"])),
                 stat["worker_type"],
                 stat["worker_pool__name"] or "",
-                stat["static_metadata__system:host_architecture"]
-                or stat["dynamic_metadata__system:host_architecture"]
+                stat["static_native_architecture"]
+                or stat["dynamic_native_architecture"]
                 or "",
             ]
             stat_archs = (
@@ -476,10 +497,10 @@ class DebusineCollector(Collector):
                 (worker_type,),
             ]
             if worker_type == WorkerType.EXTERNAL:
-                # worker_pool, host_architecture, architecture_*
+                # worker_pool, native_architecture, architecture_*
                 label_values += [
                     self._worker_pools(include_null=True),
-                    host_architectures,
+                    native_architectures,
                 ]
                 label_values += [("0", "1")] * len(architectures)
             else:
@@ -497,7 +518,7 @@ class DebusineCollector(Collector):
         )
         now = timezone.now()
         runtime = cast(
-            Combinable, now - cast(datetime, F("instance_created_at"))
+            Combinable, now - cast(dt.datetime, F("instance_created_at"))
         )
         # We are summing two separate joins, so do it in subqueries to
         # avoid cartesian explosion
