@@ -13,14 +13,16 @@ Backend infrastructure for provisioning cloud workers.
 See docs/reference/devel-blueprints/dynamic-compute.rst
 """
 
+import logging
 import math
+from collections import defaultdict
 from functools import cached_property
 
 from celery import shared_task
 from django.contrib.postgres.functions import TransactionNow
 from django.db import transaction
-from django.db.models import F, IntegerField, Sum
-from django.db.models.functions import Cast, Extract, Greatest, Least
+from django.db.models import Count, F, IntegerField, Q, Sum
+from django.db.models.functions import Cast, Coalesce, Extract, Greatest, Least
 from django.utils import timezone
 
 from debusine.db.models import (
@@ -42,6 +44,8 @@ TASK_DURATION_SECONDS = 10 * 60
 # TODO: Hardcoded target latency for a scope, until we can read it from Scope
 TARGET_LATENCY_SECONDS = 3600
 
+log = logging.getLogger(__name__)
+
 
 def terminate_idle(minutes: int = 10) -> bool:
     """
@@ -50,14 +54,24 @@ def terminate_idle(minutes: int = 10) -> bool:
     :returns: true if idle workers were found
     """
     # List cloud workers that have been idle for more than 10 minutes
-    idle_workers = Worker.objects.with_idle_time().filter(
-        idle_time__gt=minutes * 60,
-        worker_pool__isnull=False,
+    idle_workers = (
+        Worker.objects.active()
+        .with_idle_time()
+        .filter(
+            idle_time__gt=minutes * 60,
+            worker_pool__isnull=False,
+            worker_pool__enabled=True,
+        )
     )
     has_idle = False
     for worker in idle_workers:
         assert worker.worker_pool is not None
         has_idle = True
+        log.info(
+            "Terminating idle worker %s (idle for %d seconds)",
+            worker.name,
+            worker.idle_time,
+        )
         worker.worker_pool.terminate_worker(worker)
     return has_idle
 
@@ -78,14 +92,30 @@ class WorkerAllocator:
 
     def __init__(self) -> None:
         """Prefetch common information from the database."""
-        #: Number of available static workers
-        self.static_workers_count = (
-            Worker.objects.connected().filter(worker_pool__isnull=True).count()
-        )
-
         self.start_of_month = timezone.now().replace(
             day=1, hour=0, minute=0, second=0, microsecond=0
         )
+        self.static_workers = self.static_workers_count()
+
+    def static_workers_count(self) -> dict[str, int]:
+        """Count connected static workers that can build architectures."""
+        workers: dict[str, int] = defaultdict(int)
+        for row in (
+            Worker.objects.connected()
+            .filter(worker_pool__isnull=True)
+            .annotate(
+                architectures=Coalesce(
+                    "static_metadata__system:architectures",
+                    "dynamic_metadata__system:architectures",
+                )
+            )
+            .values("architectures")
+            .annotate(count=Count("id"))
+            .order_by()
+        ):
+            for architecture in row["architectures"]:
+                workers[architecture] += row["count"]
+        return workers
 
     def limit_by_max_active_instances(
         self, pool: WorkerPool, amount: int
@@ -95,7 +125,7 @@ class WorkerAllocator:
         if pool_limits.max_active_instances is None:
             return amount
 
-        current_worker_count = Worker.objects.filter(worker_pool=pool).count()
+        current_worker_count = pool.workers_running.count()
         available = pool_limits.max_active_instances - current_worker_count
         if available <= 0:
             return 0
@@ -284,59 +314,75 @@ class ScopeWorkerAllocator:
         )
 
         if amount > 0:
+            log.info("Launching %d workers in %s", amount, pool.name)
             pool.launch_workers(amount)
         return amount
 
-    @cached_property
-    def pending_task_count(self) -> int:
-        """Return the number of pending tasks."""
-        return WorkRequest.objects.filter(
-            workspace__scope=self.scope,
-            task_data__build_architecture="amd64",
-        ).count()
+    def pending_task_count(self) -> dict[str, int]:
+        """Return the number of pending tasks by architecture."""
+        pending: dict[str, int] = {}
+        for row in (
+            WorkRequest.objects.pending()
+            .filter(
+                workspace__scope=self.scope,
+            )
+            .values('task_data__build_architecture')
+            .annotate(count=Count('id'))
+            .order_by()
+        ):
+            pending[row["task_data__build_architecture"]] = row["count"]
+        return pending
 
-    @cached_property
-    def workers_required(self) -> int:
+    def workers_required(self) -> dict[str, int]:
         """
         Compute how many workers can process the scope's pending load.
 
         This computes the absolute number of workers required to process the
-        scope's pending load within the expected latency targets.
+        scope's pending load for the given architectures, within the expected
+        latency targets.
         """
         # If we have no pending tasks, we do not require workers
-        if (pending_tasks := self.pending_task_count) == 0:
-            return 0
+        pending_tasks = self.pending_task_count()
 
-        # Estimated number of worker-seconds needed to run all but the last
-        # pending task.
-        pending_seconds = TASK_DURATION_SECONDS * max(pending_tasks - 1, 0)
+        workers: dict[str, int] = {}
+        for arch, tasks in pending_tasks.items():
+            if tasks == 0:
+                workers[arch] = 0
+                continue
 
-        # Each worker will contribute one second per second of work towards
-        # draining the queue.  There's no point in allocating more workers
-        # than the number of pending tasks.
-        return max(
-            1,
-            min(
-                int(math.ceil(pending_seconds / TARGET_LATENCY_SECONDS)),
-                pending_tasks,
-            ),
-        )
+            # Estimated number of worker-seconds needed to run all but the last
+            # pending task.
+            pending_seconds = TASK_DURATION_SECONDS * max(tasks - 1, 0)
 
-    @cached_property
-    def workers_available(self) -> int:
+            # Each worker will contribute one second per second of work towards
+            # draining the queue.  There's no point in allocating more workers
+            # than the number of pending tasks.
+            workers[arch] = max(
+                1,
+                min(
+                    int(math.ceil(pending_seconds / TARGET_LATENCY_SECONDS)),
+                    tasks,
+                ),
+            )
+        return workers
+
+    def workers_available(self) -> dict[str, int]:
         """Compute the number of workers currently available to the scope."""
-        # Number of workers already available that can process the queue
-        count = self.worker_allocator.static_workers_count
+        workers = defaultdict(int, self.worker_allocator.static_workers)
 
-        # Add instance-wide pool workers
-        count += Worker.objects.filter(worker_pool__instance_wide=True).count()
-
-        # Add scope-assigned pool workers
-        count += Worker.objects.filter(
-            worker_pool__scopeworkerpool__scope=self.scope
-        ).count()
-
-        return count
+        for row in (
+            Worker.objects.active()
+            .filter(
+                Q(worker_pool__instance_wide=True)
+                | Q(worker_pool__scopeworkerpool__scope=self.scope)
+            )
+            .values("worker_pool__architectures")
+            .annotate(count=Count("id"))
+            .order_by()
+        ):
+            for architecture in row["worker_pool__architectures"]:
+                workers[architecture] += row["count"]
+        return workers
 
     @cached_property
     def allocation_pools(self) -> list[WorkerPool]:
@@ -367,24 +413,26 @@ class ScopeWorkerAllocator:
     def process_scope(self) -> None:
         """Perform allocation for this scope."""
         # Number of workers required to process the queue in time
-        workers_required = self.workers_required
-
-        # Number of workers currently available
-        workers_available = self.workers_available
-
-        # If we have enough workers already, we are done
-        if workers_required <= workers_available:
-            return
-
-        # Number of workers that we need to allocate
-        new_workers_required = workers_required - workers_available
+        workers_required_by_arch = self.workers_required()
+        workers_available_by_arch = self.workers_available()
 
         # Allocate from one pool at a time
         for pool in self.allocation_pools:
-            if new_workers_required <= 0:
-                break
-            allocated = self.allocate_workers(pool, new_workers_required)
-            new_workers_required -= allocated
+            for architecture in pool.architectures:
+                workers_available = workers_available_by_arch.get(
+                    architecture, 0
+                )
+                workers_required = workers_required_by_arch.get(architecture, 0)
+
+                # Number of workers that we need to allocate
+                new_workers_required = workers_required - workers_available
+
+                # If we have enough workers already, we are done
+                if new_workers_required <= 0:
+                    continue
+
+                allocated = self.allocate_workers(pool, new_workers_required)
+                workers_required_by_arch[architecture] -= allocated
 
 
 def create_new_workers() -> None:
