@@ -25,6 +25,7 @@ from django.http import HttpRequest
 from django.test import Client, RequestFactory, override_settings
 from django.urls import resolve, reverse
 from django.utils import timezone
+from django_prometheus.testutils import assert_metric_diff, save_registry
 
 from debusine.db.models import Identity, User
 from debusine.db.playground import scenarios
@@ -52,6 +53,10 @@ class SalsaSignonViews(TestCase):
 
     scenario = scenarios.DefaultScopeUser()
 
+    def setUp(self) -> None:
+        super().setUp()
+        self.registry = save_registry()
+
     def _collect_urls(
         self, response: TestResponseType, match_re: str
     ) -> list[str]:
@@ -239,6 +244,13 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": identities[0]}
         )
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="False",
+        )
 
         # The user is not authenticated, because the identity is not bound
         self.assertFalse(response.wsgi_request.user.is_authenticated)
@@ -278,6 +290,13 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": ident}
         )
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="True",
+        )
 
         # The user has been correctly authenticated
         self.assertTrue(response.wsgi_request.user.is_authenticated)
@@ -318,6 +337,13 @@ class SalsaSignonViews(TestCase):
         # The identity is not activated
         assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(response.wsgi_request.signon.identities, {})
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="False",
+        )
 
     def test_oidc_callback_login_user_inactive(self) -> None:
         """Do not log in an inactive user."""
@@ -355,6 +381,13 @@ class SalsaSignonViews(TestCase):
         # The identity is not activated
         assert isinstance(response.wsgi_request, RequestSignonProtocol)
         self.assertEqual(response.wsgi_request.signon.identities, {})
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="False",
+        )
 
         # The user is still marked as inactive
         self.scenario.user.refresh_from_db()
@@ -399,6 +432,13 @@ class SalsaSignonViews(TestCase):
         self.assertEqual(
             response.wsgi_request.signon.identities, {"salsa": ident}
         )
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="True",
+        )
 
     def test_oidc_callback_wrong_provider(self) -> None:
         """The OIDC callback with a wrong provider."""
@@ -411,6 +451,41 @@ class SalsaSignonViews(TestCase):
 
         self.assertEqual(Identity.objects.count(), 0)
         self.assertFalse(response.wsgi_request.user.is_authenticated)
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="wrong",
+            activated="False",
+        )
+
+    def test_oidc_callback_validation_error(self) -> None:
+        ident = self._make_identity(
+            subject="123",
+            last_used=timezone.now() - dt.timedelta(days=1),
+        )
+        cb_url = reverse("signon:oidc_callback", kwargs={"name": ident.issuer})
+
+        with self.assertLogs("debusine.server.signon") as log:
+            response = self.client.get(cb_url)
+        self.assertEqual(response.status_code, 403)
+
+        self.assertFalse(response.wsgi_request.user.is_authenticated)
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer=ident.issuer,
+            activated="False",
+        )
+        self.assertEqual(
+            log.output,
+            [
+                f"INFO:debusine.server.signon:OIDC validation for provider "
+                f"'{ident.issuer}' failed: "
+                f"Request state mismatch: expected state not found in session"
+            ],
+        )
 
     def test_oidc_callback_map_identity_to_user_failed(self) -> None:
         """Test user interaction when activating an identity fails."""
@@ -431,6 +506,13 @@ class SalsaSignonViews(TestCase):
         self.assertRedirects(response, reverse("homepage:homepage"))
         messages = [m.message for m in get_messages(response.wsgi_request)]
         self.assertEqual(messages, ["Login failed: test message"])
+        assert_metric_diff(
+            self.registry,
+            1,
+            "debusine_signon_oidc_callbacks_total",
+            issuer="salsa",
+            activated="False",
+        )
 
         ident.refresh_from_db()
         self.assertIsNone(ident.user)
