@@ -10,6 +10,8 @@
 """Unit tests for the task to copy items into target collections."""
 
 from collections.abc import Sequence
+from pathlib import Path
+from textwrap import dedent
 from typing import Any, ClassVar
 
 from django.conf import settings
@@ -19,7 +21,10 @@ from debusine.artifacts.models import (
     BareDataCategory,
     CollectionCategory,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import (
+    LookupChildType,
+    model_to_json_serializable_dict,
+)
 from debusine.db.models import (
     Collection,
     CollectionItem,
@@ -36,10 +41,17 @@ from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
 )
+from debusine.tasks import BaseTask
 from debusine.tasks.models import LookupMultiple, WorkerType
 from debusine.test.django import TestCase
 
 
+def cleanup_task(task: BaseTask[Any, Any]) -> None:
+    """Delete debug log files directory if it exists."""
+    if task._debug_log_files_directory:
+        task._debug_log_files_directory.cleanup()
+
+
 class CopyCollectionItemsTests(TestCase):
     """Tests for :py:class:`CopyCollectionItems`."""
 
@@ -127,33 +139,47 @@ class CopyCollectionItemsTests(TestCase):
             artifact, user=self.user, variables=artifact_item_data
         )
 
-    def lookup_source_items(
+    def prepare_copy_item_test(
         self, lookup: dict[str, Any]
-    ) -> Sequence[LookupResult]:
-        """Look up source items in preparation for copying them."""
-        return lookup_multiple(
-            LookupMultiple.parse_obj(
-                {
-                    "collection": f"_@{CollectionCategory.PACKAGE_BUILD_LOGS}",
-                    **lookup,
-                }
-            ),
+    ) -> tuple[CopyCollectionItems, Sequence[LookupResult]]:
+        """Set up a task and lookup results to test ``copy_item``."""
+        source_items = LookupMultiple.parse_obj(
+            {
+                "collection": f"_@{CollectionCategory.PACKAGE_BUILD_LOGS}",
+                **lookup,
+            }
+        )
+        task = CopyCollectionItems(
+            task_data=model_to_json_serializable_dict(
+                CopyCollectionItemsData(
+                    copies=[
+                        CopyCollectionItemsCopies(
+                            source_items=source_items,
+                            target_collection=self.target_build_logs.id,
+                        )
+                    ]
+                ),
+                exclude_unset=True,
+            )
+        )
+        self.addCleanup(cleanup_task, task)
+        results = lookup_multiple(
+            source_items,
             self.source_workspace,
             user=self.user,
             expect_type=LookupChildType.ANY,
         )
+        return task, results
 
     def test_copy_item_bare(self) -> None:
         """Copy a bare data item."""
         bare_item = self.create_build_log_placeholder()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.BARE}
         )
         self.assertEqual(len(source_items), 1)
 
-        CopyCollectionItems.copy_item(
-            source_items[0], self.target_build_logs, user=self.user
-        )
+        task.copy_item(source_items[0], self.target_build_logs, user=self.user)
 
         self.assertTrue(
             self.target_build_logs.child_items.filter(
@@ -170,7 +196,7 @@ class CopyCollectionItemsTests(TestCase):
     def test_copy_item_bare_needs_unembargo(self) -> None:
         """Copying bare data from private to public needs unembargo=True."""
         self.create_build_log_placeholder()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.BARE}
         )
         self.assertEqual(len(source_items), 1)
@@ -182,21 +208,21 @@ class CopyCollectionItemsTests(TestCase):
             f"Copying from {self.source_workspace} to {self.target_workspace} "
             f"requires unembargo=True",
         ):
-            CopyCollectionItems.copy_item(
+            task.copy_item(
                 source_items[0], self.target_build_logs, user=self.user
             )
 
     def test_copy_item_bare_with_unembargo(self) -> None:
         """Copying bare data from private to public can unembargo it."""
         bare_item = self.create_build_log_placeholder()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.BARE}
         )
         self.assertEqual(len(source_items), 1)
         self.source_workspace.public = False
         self.source_workspace.save()
 
-        CopyCollectionItems.copy_item(
+        task.copy_item(
             source_items[0],
             self.target_build_logs,
             user=self.user,
@@ -218,12 +244,12 @@ class CopyCollectionItemsTests(TestCase):
     def test_copy_item_bare_variables(self) -> None:
         """Pass additional variables while copying a bare item."""
         bare_item = self.create_build_log_placeholder()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.BARE}
         )
         self.assertEqual(len(source_items), 1)
 
-        CopyCollectionItems.copy_item(
+        task.copy_item(
             source_items[0],
             self.target_build_logs,
             user=self.user,
@@ -256,7 +282,7 @@ class CopyCollectionItemsTests(TestCase):
         artifact = artifact_item.artifact
         assert artifact is not None
         build_log_file = artifact.files.get()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -265,9 +291,7 @@ class CopyCollectionItemsTests(TestCase):
         ).get()
 
         work_request.set_current()
-        CopyCollectionItems.copy_item(
-            source_items[0], self.target_build_logs, user=self.user
-        )
+        task.copy_item(source_items[0], self.target_build_logs, user=self.user)
 
         copied_collection_item = self.target_build_logs.child_items.get(
             name=f"debian_bookworm_amd64_hello_1.0-1_{work_request.id}",
@@ -312,7 +336,7 @@ class CopyCollectionItemsTests(TestCase):
             workspace=self.source_workspace
         )
         self.create_build_log(work_request=work_request)
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -324,7 +348,7 @@ class CopyCollectionItemsTests(TestCase):
             f"Copying from {self.source_workspace} to {self.target_workspace} "
             f"requires unembargo=True",
         ):
-            CopyCollectionItems.copy_item(
+            task.copy_item(
                 source_items[0], self.target_build_logs, user=self.user
             )
 
@@ -341,7 +365,7 @@ class CopyCollectionItemsTests(TestCase):
         artifact_item = self.create_build_log(work_request=work_request)
         artifact = artifact_item.artifact
         assert artifact is not None
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -349,7 +373,7 @@ class CopyCollectionItemsTests(TestCase):
         self.source_workspace.save()
 
         work_request.set_current()
-        CopyCollectionItems.copy_item(
+        task.copy_item(
             source_items[0],
             self.target_build_logs,
             user=self.user,
@@ -384,7 +408,7 @@ class CopyCollectionItemsTests(TestCase):
             users=[work_request.created_by],
         )
         self.create_build_log(extra_data={"nested": {"foo": "bar"}})
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -395,7 +419,7 @@ class CopyCollectionItemsTests(TestCase):
         )
 
         work_request.set_current()
-        CopyCollectionItems.copy_item(
+        task.copy_item(
             source_items[0],
             target_collection,
             user=self.user,
@@ -417,7 +441,7 @@ class CopyCollectionItemsTests(TestCase):
     def test_copy_item_artifact_failed_to_expand_variables(self) -> None:
         """Fail if the given variables cannot be expanded."""
         self.create_build_log()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -425,7 +449,7 @@ class CopyCollectionItemsTests(TestCase):
         with self.assertRaisesRegex(
             CannotCopy, r"Cannot expand variables: \{'\$key': 'nonexistent'\}"
         ):
-            CopyCollectionItems.copy_item(
+            task.copy_item(
                 source_items[0],
                 self.target_build_logs,
                 user=self.user,
@@ -443,7 +467,7 @@ class CopyCollectionItemsTests(TestCase):
             users=[work_request.created_by],
         )
         artifact_item = self.create_build_log(skip_add_files_in_store=True)
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -454,7 +478,7 @@ class CopyCollectionItemsTests(TestCase):
             f"Cannot copy incomplete 'hello_1.0-1_amd64.buildlog' from "
             f"artifact ID {artifact_item.artifact_id}",
         ):
-            CopyCollectionItems.copy_item(
+            task.copy_item(
                 source_items[0], self.target_build_logs, user=self.user
             )
 
@@ -469,7 +493,7 @@ class CopyCollectionItemsTests(TestCase):
             users=[work_request.created_by],
         )
         artifact_item = self.create_build_log()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -481,7 +505,7 @@ class CopyCollectionItemsTests(TestCase):
             f"Cannot copy 'hello_1.0-1_amd64.buildlog' from artifact ID "
             f"{artifact_item.artifact_id}: not in any available file store",
         ):
-            CopyCollectionItems.copy_item(
+            task.copy_item(
                 source_items[0], self.target_build_logs, user=self.user
             )
 
@@ -513,7 +537,7 @@ class CopyCollectionItemsTests(TestCase):
         artifact = artifact_item.artifact
         assert artifact is not None
         build_log_file = artifact.files.get()
-        source_items = self.lookup_source_items(
+        task, source_items = self.prepare_copy_item_test(
             {"child_type": LookupChildType.ARTIFACT}
         )
         self.assertEqual(len(source_items), 1)
@@ -522,9 +546,7 @@ class CopyCollectionItemsTests(TestCase):
         ).get()
 
         work_request.set_current()
-        CopyCollectionItems.copy_item(
-            source_items[0], self.target_build_logs, user=self.user
-        )
+        task.copy_item(source_items[0], self.target_build_logs, user=self.user)
 
         copied_collection_item = self.target_build_logs.child_items.get(
             name=f"debian_bookworm_amd64_hello_1.0-1_{work_request.id}",
@@ -574,16 +596,20 @@ class CopyCollectionItemsTests(TestCase):
         sbuild = self.playground.simulate_package_build(
             source_package, workflow=workflow
         )
+        binary_package = sbuild.artifact_set.get(
+            category=ArtifactCategory.BINARY_PACKAGE
+        )
         workflow.internal_collection.manager.add_artifact(
-            sbuild.artifact_set.get(category=ArtifactCategory.BINARY_PACKAGE),
+            binary_package,
             user=sbuild.created_by,
             workflow=workflow,
             name="binary-amd64",
         )
+        build_log = sbuild.artifact_set.get(
+            category=ArtifactCategory.PACKAGE_BUILD_LOG
+        )
         self.source_build_logs.manager.add_artifact(
-            sbuild.artifact_set.get(
-                category=ArtifactCategory.PACKAGE_BUILD_LOG
-            ),
+            build_log,
             user=sbuild.created_by,
             workflow=workflow,
             variables={
@@ -644,7 +670,32 @@ class CopyCollectionItemsTests(TestCase):
         copy_collection_items.assign_worker(worker)
         task = copy_collection_items.get_task()
 
-        self.assertTrue(task.execute())
+        with self.playground.assign_role(
+            self.source_workspace,
+            copy_collection_items.created_by,
+            Workspace.Roles.CONTRIBUTOR,
+        ):
+            self.assertTrue(task.execute())
+
+        debug_logs_artifact = copy_collection_items.artifact_set.get(
+            category=ArtifactCategory.WORK_REQUEST_DEBUG_LOGS
+        )
+        copy_log = debug_logs_artifact.fileinartifact_set.get()
+        self.assertEqual(copy_log.path, "copy-collection-items.log")
+        fileobj = copy_log.file
+        file_backend = self.source_workspace.scope.download_file_backend(
+            fileobj
+        )
+        with file_backend.get_stream(fileobj) as file:
+            self.assertEqual(
+                file.read().decode(),
+                dedent(
+                    f"""\
+                    Copying to {target_suite}: {binary_package}
+                    Copying to {self.target_build_logs}: {build_log}
+                    """
+                ),
+            )
 
         # The target collections contain the requested items.
         self.assertTrue(
@@ -677,6 +728,57 @@ class CopyCollectionItemsTests(TestCase):
             ).exists()
         )
 
+    def test_execute_copy_fails(self) -> None:
+        self.create_build_log_placeholder()
+        source_items = LookupMultiple.parse_obj(
+            {
+                "collection": self.source_build_logs.id,
+                "child_type": LookupChildType.BARE,
+            }
+        )
+        copy_collection_items = self.playground.create_server_task(
+            task_name="copycollectionitems",
+            task_data=CopyCollectionItemsData(
+                copies=[
+                    CopyCollectionItemsCopies(
+                        source_items=source_items,
+                        target_collection=self.target_build_logs.id,
+                    )
+                ]
+            ),
+        )
+        self.source_workspace.public = False
+        self.source_workspace.save()
+        self.playground.create_group_role(
+            self.target_workspace,
+            Workspace.Roles.CONTRIBUTOR,
+            users=[copy_collection_items.created_by],
+        )
+        worker = self.playground.create_worker(worker_type=WorkerType.CELERY)
+        copy_collection_items.assign_worker(worker)
+        task = copy_collection_items.get_task()
+        self.addCleanup(cleanup_task, task)
+
+        with (
+            self.playground.assign_role(
+                self.source_workspace,
+                copy_collection_items.created_by,
+                Workspace.Roles.CONTRIBUTOR,
+            ),
+            self.assertRaises(CannotCopy),
+        ):
+            task.execute()
+
+        assert task._debug_log_files_directory is not None
+        self.assertEqual(
+            Path(
+                task._debug_log_files_directory.name,
+                "copy-collection-items.log",
+            ).read_text(),
+            f"Copy failed: Copying from {self.source_workspace} to "
+            f"{self.target_workspace} requires unembargo=True\n",
+        )
+
     def test_get_label(self) -> None:
         """Test get_label."""
         task = CopyCollectionItems(
