@@ -20,7 +20,13 @@ from django_pglocks import advisory_lock
 
 from debusine.artifacts.models import CollectionCategory
 from debusine.db.locks import LockError, LockType
-from debusine.db.models import Artifact, Collection, CollectionItem, WorkRequest
+from debusine.db.models import (
+    Artifact,
+    Collection,
+    CollectionItem,
+    File,
+    WorkRequest,
+)
 from debusine.db.playground import scenarios
 from debusine.server.tasks import GenerateSuiteIndexes
 from debusine.server.tasks.generate_suite_indexes import IndexInfo
@@ -220,7 +226,7 @@ class GenerateSuiteIndexesTests(TestCase):
         expected_sources = dedent(
             f"""\
             Package: dpkg
-            Binary: dpkg, dpkg-doc
+            Binary: dpkg, dpkg-doc, libdpkg-perl
             Version: 1.21.22
             Maintainer: Example Maintainer <example@example.org>
             Architecture: any
@@ -233,6 +239,7 @@ class GenerateSuiteIndexesTests(TestCase):
             Package-List:{" "}
              dpkg deb devel optional arch=any
              dpkg-doc deb devel optional arch=any
+             libdpkg-perl deb devel optional arch=any
             Directory: pool/main/d/dpkg
             Priority: source
             Section: devel
@@ -381,24 +388,18 @@ class GenerateSuiteIndexesTests(TestCase):
             temp_path=temp_path, component="main", architecture="amd64"
         )
 
-        dpkg_amd64 = task.suite.manager.lookup("binary:dpkg_amd64")
-        assert dpkg_amd64 is not None
-        assert dpkg_amd64.artifact is not None
-        dpkg_amd64_deb = dpkg_amd64.artifact.fileinartifact_set.get(
-            path="dpkg_1.21.22_amd64.deb"
-        ).file
-        dpkg_doc_all = task.suite.manager.lookup("binary:dpkg-doc_all")
-        assert dpkg_doc_all is not None
-        assert dpkg_doc_all.artifact is not None
-        dpkg_doc_all_deb = dpkg_doc_all.artifact.fileinartifact_set.get(
-            path="dpkg-doc_1.21.22_all.deb"
-        ).file
-        hello_amd64 = task.suite.manager.lookup("binary:hello_amd64")
-        assert hello_amd64 is not None
-        assert hello_amd64.artifact is not None
-        hello_amd64_deb = hello_amd64.artifact.fileinartifact_set.get(
-            path="hello_1.0-1_amd64.deb"
-        ).file
+        debs: dict[str, File] = {}
+        for pkg in (
+            "dpkg_amd64",
+            "dpkg-doc_all",
+            "libdpkg-perl_all",
+            "hello_amd64",
+        ):
+            item = task.suite.manager.lookup(f"binary:{pkg}")
+            assert item is not None
+            assert item.artifact is not None
+            debs[pkg] = item.artifact.fileinartifact_set.get().file
+
         expected_packages = dedent(
             f"""\
             Package: dpkg
@@ -409,8 +410,8 @@ class GenerateSuiteIndexesTests(TestCase):
             Section: devel
             Priority: optional
             Filename: pool/main/d/dpkg/dpkg_1.21.22_amd64.deb
-            Size: {dpkg_amd64_deb.size}
-            SHA256: {dpkg_amd64_deb.sha256.hex()}
+            Size: {debs["dpkg_amd64"].size}
+            SHA256: {debs["dpkg_amd64"].sha256.hex()}
 
             Package: dpkg-doc
             Source: dpkg
@@ -421,8 +422,20 @@ class GenerateSuiteIndexesTests(TestCase):
             Section: devel
             Priority: optional
             Filename: pool/main/d/dpkg/dpkg-doc_1.21.22_all.deb
-            Size: {dpkg_doc_all_deb.size}
-            SHA256: {dpkg_doc_all_deb.sha256.hex()}
+            Size: {debs["dpkg-doc_all"].size}
+            SHA256: {debs["dpkg-doc_all"].sha256.hex()}
+
+            Package: libdpkg-perl
+            Source: dpkg
+            Version: 1.21.22
+            Maintainer: Example Maintainer <example@example.org>
+            Architecture: all
+            Description: Example description
+            Section: devel
+            Priority: optional
+            Filename: pool/main/d/dpkg/libdpkg-perl_1.21.22_all.deb
+            Size: {debs["libdpkg-perl_all"].size}
+            SHA256: {debs["libdpkg-perl_all"].sha256.hex()}
 
             Package: hello
             Version: 1.0-1
@@ -432,8 +445,8 @@ class GenerateSuiteIndexesTests(TestCase):
             Section: devel
             Priority: optional
             Filename: pool/main/h/hello/hello_1.0-1_amd64.deb
-            Size: {hello_amd64_deb.size}
-            SHA256: {hello_amd64_deb.sha256.hex()}
+            Size: {debs["hello_amd64"].size}
+            SHA256: {debs["hello_amd64"].sha256.hex()}
 
             """
         )
@@ -510,6 +523,7 @@ class GenerateSuiteIndexesTests(TestCase):
             Suite: stable
             Codename: bookworm
             Date: {expected_date}
+            Acquire-By-Hash: yes
             Architectures: {" ".join(task.architectures)}
             Components: {" ".join(task.components)}
             SHA256:
@@ -612,6 +626,27 @@ class GenerateSuiteIndexesTests(TestCase):
             [sources_xz_artifact],
         )
 
+    def test_override_acquire_by_hash(self) -> None:
+        """``generate_release`` lets ``Acquire-By-Hash: yes`` be overridden."""
+        scenario = scenarios.UIPlayground(set_current=True)
+        self.playground.build_scenario(scenario, set_current=True)
+        scenario.suite.data["release_fields"]["Acquire-By-Hash"] = False
+        scenario.suite.save()
+
+        suite_collection = f"play_bookworm@{CollectionCategory.SUITE}"
+        task = self.create_task(suite_collection=suite_collection)
+        temp_path = self.create_temporary_directory()
+        task.generate_release(temp_path=temp_path)
+
+        release_item = task.suite.manager.lookup("index:Release")
+        assert release_item is not None
+        assert release_item.artifact is not None
+        release_file = release_item.artifact.files.get()
+        file_backend = scenario.scope.download_file_backend(release_file)
+        with file_backend.get_stream(release_file) as f:
+            got_release = f.read().decode()
+            self.assertNotIn("Acquire-By-Hash:", got_release)
+
     def test_supersede_older_items(self) -> None:
         """Remove superseded old items before generating new ones."""
         bookworm = self.create_suite_collection("bookworm")
