@@ -21,17 +21,12 @@ from channels.routing import URLRouter
 from channels.testing import WebsocketCommunicator
 from django.conf import settings
 from django.utils import timezone
-from prometheus_client import Counter, Gauge, Metric
-from prometheus_client.metrics import MetricWrapperBase
+from django_prometheus.testutils import assert_metric_diff, save_registry
 from rest_framework.fields import DateTimeField
 
 from debusine.db.models import Token, WorkRequest, Worker, Workspace
 from debusine.server import consumers
-from debusine.server.consumers import (
-    WorkerConsumer,
-    active_websocket_connections,
-    websocket_connect_requests,
-)
+from debusine.server.consumers import WorkerConsumer
 from debusine.server.middlewares.token_last_seen_at import (
     TokenLastSeenAtMiddlewareChannels,
 )
@@ -53,44 +48,6 @@ class WebSocketURLTests(TestCase):
         URLRouter(websocket_urlpatterns)  # type: ignore[arg-type]
 
 
-class MetricsTestMixin:
-    """Methods used by metrics tests."""
-
-    def reset_counter(self, counter: Counter) -> None:
-        """
-        Reset all label sets in a counter.
-
-        This is useful to clear out leftovers from previous tests.
-        """
-        [collected] = counter.collect()
-        for sample in collected.samples:
-            counter.labels(**sample.labels).reset()
-
-    def reset_gauge(self, gauge: Gauge) -> None:
-        """
-        Reset all label sets in a gauge.
-
-        This is useful to clear out leftovers from previous tests.
-        """
-        [collected] = gauge.collect()
-        for sample in collected.samples:
-            gauge.labels(**sample.labels).set(0)
-
-    def sample_by_label(self, metric: Metric, labels: dict[str, str]) -> float:
-        """Find the sample for labels in metric."""
-        for sample in metric.samples:
-            if sample.labels == labels:
-                return sample.value
-        raise AssertionError(f"{labels} not found in {metric}")
-
-    def assert_metric_equals(
-        self, metric: MetricWrapperBase, value: float, **labels: str
-    ) -> None:
-        """Assert that there is a sample for labels in metric with value."""
-        [collected] = metric.collect()
-        assert self.sample_by_label(collected, labels) == value
-
-
 class WorkerConsumerMixin(BaseDjangoTestCase):
     """Methods used by different WorkerConsumer tests classes."""
 
@@ -243,9 +200,7 @@ class WorkerConsumerMixin(BaseDjangoTestCase):
         await communicator.disconnect()
 
 
-class WorkerConsumerTransactionTests(
-    WorkerConsumerMixin, MetricsTestMixin, TransactionTestCase
-):
+class WorkerConsumerTransactionTests(WorkerConsumerMixin, TransactionTestCase):
     """
     Tests for WorkerConsumer class.
 
@@ -313,7 +268,7 @@ class WorkerConsumerTransactionTests(
         assert worker.token is not None
 
         await database_sync_to_async(self.playground.create_worker_task)(
-            status=WorkRequest.Statuses.PENDING, worker=worker
+            mark_pending=True, worker=worker
         )
 
         await self.assert_messages_send_on_connect(
@@ -330,7 +285,7 @@ class WorkerConsumerTransactionTests(
         assert worker.token is not None
 
         await database_sync_to_async(self.playground.create_worker_task)(
-            status=WorkRequest.Statuses.RUNNING, worker=worker
+            mark_running=True, worker=worker
         )
 
         await self.assert_messages_send_on_connect(
@@ -391,10 +346,7 @@ class WorkerConsumerTransactionTests(
 
         work_request: WorkRequest = await database_sync_to_async(
             self.playground.create_worker_task
-        )()
-
-        await database_sync_to_async(work_request.assign_worker)(worker)
-        await database_sync_to_async(work_request.mark_running)()
+        )(worker=worker, mark_running=True)
 
         self.assertEqual(work_request.status, work_request.Statuses.RUNNING)
 
@@ -550,51 +502,80 @@ class WorkerConsumerTransactionTests(
         )
 
     async def test_metrics(self) -> None:
-        self.reset_counter(websocket_connect_requests)
-        self.reset_gauge(active_websocket_connections)
+        registry = save_registry()
 
         communicator1, worker1 = await self.connect_to_new_worker(
             fqdn="worker1.example"
         )
-        self.assert_metric_equals(
-            websocket_connect_requests, 1, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_websocket_connect_requests_total",
+            view="ws-connect",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 1, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_active_websocket_connections",
+            view="ws-connect",
         )
 
         communicator2, worker2 = await self.connect_to_new_worker(
             fqdn="worker2.example"
         )
-        self.assert_metric_equals(
-            websocket_connect_requests, 2, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_websocket_connect_requests_total",
+            view="ws-connect",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 2, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_active_websocket_connections",
+            view="ws-connect",
         )
 
         await self.test_connect_invalid_token()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-connect",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 2, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_active_websocket_connections",
+            view="ws-connect",
         )
 
         await communicator1.disconnect()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-connect",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 1, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_active_websocket_connections",
+            view="ws-connect",
         )
 
         await communicator2.disconnect()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-connect",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 0, view="ws-connect"
+        assert_metric_diff(
+            registry,
+            0,
+            "debusine_active_websocket_connections",
+            view="ws-connect",
         )
 
 
@@ -644,7 +625,7 @@ class WorkerConsumerTests(WorkerConsumerMixin, TestCase):
         await worker.disconnect(0)
 
 
-class WorkRequestCompletedConsumerTests(MetricsTestMixin, TransactionTestCase):
+class WorkRequestCompletedConsumerTests(TransactionTestCase):
     """Tests for OnWorkRequestCompletedConsumer class."""
 
     CONNECTED_PAYLOAD = {
@@ -1006,8 +987,7 @@ class WorkRequestCompletedConsumerTests(MetricsTestMixin, TransactionTestCase):
         await communicator.disconnect()
 
     async def test_metrics(self) -> None:
-        self.reset_counter(websocket_connect_requests)
-        self.reset_gauge(active_websocket_connections)
+        registry = save_registry()
         users = [
             await database_sync_to_async(self.playground.create_user)(
                 username=username
@@ -1022,41 +1002,71 @@ class WorkRequestCompletedConsumerTests(MetricsTestMixin, TransactionTestCase):
         ]
 
         communicator1 = await self.connect(token_key=tokens[0].key)
-        self.assert_metric_equals(
-            websocket_connect_requests, 1, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_websocket_connect_requests_total",
+            view="ws-work-request-on-completed",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 1, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_active_websocket_connections",
+            view="ws-work-request-on-completed",
         )
 
         communicator2 = await self.connect(token_key=tokens[1].key)
-        self.assert_metric_equals(
-            websocket_connect_requests, 2, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_websocket_connect_requests_total",
+            view="ws-work-request-on-completed",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 2, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_active_websocket_connections",
+            view="ws-work-request-on-completed",
         )
 
         await self.test_connect_rejected_no_token_header()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-work-request-on-completed",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 2, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            2,
+            "debusine_active_websocket_connections",
+            view="ws-work-request-on-completed",
         )
 
         await communicator1.disconnect()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-work-request-on-completed",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 1, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            1,
+            "debusine_active_websocket_connections",
+            view="ws-work-request-on-completed",
         )
 
         await communicator2.disconnect()
-        self.assert_metric_equals(
-            websocket_connect_requests, 3, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            3,
+            "debusine_websocket_connect_requests_total",
+            view="ws-work-request-on-completed",
         )
-        self.assert_metric_equals(
-            active_websocket_connections, 0, view="ws-work-request-on-completed"
+        assert_metric_diff(
+            registry,
+            0,
+            "debusine_active_websocket_connections",
+            view="ws-work-request-on-completed",
         )
