@@ -179,7 +179,9 @@ class SchedulerTestCase(PlaygroundTestCase):
     ) -> WorkRequest:
         """Create and return a WorkRequest."""
         work_request = self.playground.create_worker_task(
-            task_name='sbuild', task_data=self.SAMPLE_TASK_DATA
+            task_name="sbuild",
+            task_data=self.SAMPLE_TASK_DATA,
+            mark_pending=True,
         )
         if assign_to:
             work_request.assign_worker(assign_to)
@@ -233,9 +235,7 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         WorkRequest.objects.all().delete()
 
         self.playground.create_internal_task(
-            status=WorkRequest.Statuses.PENDING,
-            task_name="unhandled",
-            validate=False,
+            mark_pending=True, task_name="unhandled", validate=False
         )
 
         self.assertEqual(schedule(), [])
@@ -253,12 +253,14 @@ class SchedulerTests(SchedulerTestCase, TestCase):
             for i in range(2)
         ]
         for wr in pending:
-            wr.status = WorkRequest.Statuses.PENDING
-            wr.save()
-        WorkRequest.objects.create_synchronization_point(
-            parent=parent,
-            step="completed",
-        ).mark_completed(result=WorkRequest.Results.SUCCESS)
+            wr.mark_pending()
+        self.playground.advance_work_request(
+            WorkRequest.objects.create_synchronization_point(
+                parent=parent,
+                step="completed",
+            ),
+            result=WorkRequest.Results.SUCCESS,
+        )
 
         self.assertCountEqual(schedule(), pending)
         for wr in pending:
@@ -362,7 +364,7 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         dep = work_request.parent.create_child_worker(task_name="noop")
         work_request.add_dependency(dep)
         self.playground.advance_work_request(
-            dep, mark_running=True, result=WorkRequest.Results.SUCCESS
+            dep, result=WorkRequest.Results.SUCCESS
         )
 
         result = schedule()
@@ -389,7 +391,7 @@ class SchedulerTests(SchedulerTestCase, TestCase):
         )
         work_request.add_dependency(dep)
         self.playground.advance_work_request(
-            dep, mark_running=True, result=WorkRequest.Results.FAILURE
+            dep, result=WorkRequest.Results.FAILURE
         )
 
         result = schedule()
@@ -767,6 +769,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
                 input=LintianInput(source_artifact=1),
                 environment="debian/match:codename=bookworm",
             ),
+            mark_pending=True,
         )
         self.single_lookups = {
             (1, None): ArtifactInfo(
@@ -890,6 +893,7 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
                 task_data=GenerateKeyData(
                     purpose=KeyPurpose.UEFI, description="A UEFI key"
                 ),
+                mark_pending=True,
             )
 
         result = schedule_for_worker(signing_worker)
@@ -938,12 +942,12 @@ class ScheduleForWorkerTests(SchedulerTestCase, TestCase):
 
         for _ in range(3):
             work_request = self.playground.create_server_task(
-                task_name="servernoop"
+                task_name="servernoop", mark_pending=True
             )
             self.assertEqual(work_request, schedule_for_worker(worker))
 
         work_request = self.playground.create_server_task(
-            task_name="servernoop"
+            task_name="servernoop", mark_pending=True
         )
         self.assertIsNone(schedule_for_worker(worker))
 
@@ -1148,7 +1152,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             "server:servernoop:version": ServerNoop.TASK_VERSION,
         }
         work_request = self.playground.create_server_task(
-            task_name="servernoop"
+            task_name="servernoop", mark_pending=True
         )
 
         with mock.patch(
@@ -1169,6 +1173,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
         wr = WorkRequest.objects.create_workflow_callback(
             parent=parent, step="test"
         )
+        wr.mark_pending()
 
         with mock.patch(
             "debusine.server.scheduler.run_workflow_task.apply_async"
@@ -1200,6 +1205,8 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             )
             for workflow in workflows[:2]
         ]
+        for wr in wrs:
+            wr.mark_pending()
 
         with transaction.atomic():
             self.assertEqual(schedule(), wrs)
@@ -1222,6 +1229,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
                 parent=workflows[-1], step="test"
             )
         )
+        wrs[-1].mark_pending()
         # A running work request in the same parent workflow that isn't a
         # workflow callback doesn't stop the workflow callback running.
         self.playground.create_worker_task(
@@ -1261,6 +1269,8 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
             )
             for i in range(2)
         ]
+        for wr in wrs:
+            wr.mark_pending()
 
         with transaction.atomic():
             self.assertEqual(schedule(), [wrs[0]])
@@ -1283,6 +1293,7 @@ class ScheduleForWorkerTransactionTests(SchedulerTestCase, TransactionTestCase):
                 parent=workflow, step="test2"
             )
         )
+        wrs[-1].mark_pending()
 
         with transaction.atomic():
             self.assertEqual(schedule(), [])
