@@ -99,9 +99,9 @@ class TarArtifactTests(TestCase):
         self.assertEqual(
             tar_file.getnames(),
             list(
-                artifact.fileinartifact_set.all()
-                .values_list("path", flat=True)
-                .order_by("path")
+                artifact.fileinartifact_set.values_list(
+                    "path", flat=True
+                ).order_by("path")
             ),
         )
 
