@@ -52,6 +52,7 @@ from debusine.server.views.base import (
     GenericAPIViewBase,
     GetOrCreateAPIView,
     ListAPIViewBase,
+    RetrieveAPIViewBase,
 )
 from debusine.server.views.rest import (
     IsTokenUserAuthenticated,
@@ -203,6 +204,40 @@ class ArtifactView(GenericAPIViewBase[Artifact], BaseAPIView):
         return Response(artifact_response.data, status=status.HTTP_201_CREATED)
 
 
+class WorkspaceArtifactView(RetrieveAPIViewBase[Artifact], BaseAPIView):
+    """View used to get artifacts that may cross boundaries."""
+
+    filter_backends = [CanDisplayFilterBackend]
+    lookup_url_kwarg = "artifact_id"
+
+    def get_queryset(self) -> ArtifactQuerySet[Any]:
+        """Get the query set for this view."""
+        return Artifact.objects.visible_to_current_workspace()
+
+    def get_object(self) -> Artifact:
+        """Override to return more API-friendly errors."""
+        try:
+            return super().get_object()
+        except Http404 as exc:
+            raise DebusineAPIException(
+                title=str(exc), status_code=status.HTTP_404_NOT_FOUND
+            )
+
+    def get(
+        self, request: Request, workspace: str, artifact_id: int  # noqa: U100
+    ) -> Response:
+        """Return information of artifact_id."""
+        self.set_current_workspace(workspace)
+        artifact = self.get_object()
+        self.enforce(artifact.can_display)
+
+        artifact_response = ArtifactSerializerResponse.from_artifact(
+            artifact, request
+        )
+
+        return Response(artifact_response.data, status=status.HTTP_200_OK)
+
+
 class FileUploadParser(BaseParser):
     """View used to upload files into an artifact."""
 
