@@ -10,10 +10,11 @@
 """Views for the server application's OpenMetrics statistics."""
 
 import os
-from collections.abc import Callable, Generator, Iterable, Mapping
+from collections.abc import Callable, Generator, Mapping
 from enum import StrEnum
-from typing import Any, Protocol
+from typing import Any
 
+from celery import current_task
 from django.utils.encoding import smart_str
 from prometheus_client import Metric
 from prometheus_client import exposition as prometheus
@@ -24,8 +25,17 @@ from rest_framework import renderers, status
 from rest_framework.request import Request
 from rest_framework.response import Response
 
+from debusine.db.metrics import (
+    BadMetricType,
+    Collector,
+    MetricNotFound,
+    emit_metric,
+)
+from debusine.server.exceptions import DebusineAPIException
 from debusine.server.open_metrics import DebusineCollector
+from debusine.server.serializers import EmitMetricSerializer
 from debusine.server.views.base import BaseAPIView
+from debusine.server.views.rest import IsWorkerAuthenticated
 
 
 class FixedMultiProcessCollector(MultiProcessCollector):
@@ -51,13 +61,6 @@ class FixedMultiProcessCollector(MultiProcessCollector):
             yield metric
 
 
-class Collector(Protocol):
-    """An object that supports metric collection."""
-
-    def collect(self) -> Iterable[Metric]:
-        """Collect metrics."""
-
-
 class FilteredCollector:
     """A metric collector that only exposes some metrics from a parent."""
 
@@ -173,3 +176,41 @@ class OpenMetricsView(BaseAPIView):
         return Response(
             data, status=status.HTTP_200_OK, content_type=content_type
         )
+
+
+class OpenMetricsEmitView(BaseAPIView):
+    """Emit an additional metric from a separate worker."""
+
+    permission_classes = [IsWorkerAuthenticated]
+
+    def post(self, request: Request) -> Response:
+        """Emit a metric from a worker."""
+        serializer = EmitMetricSerializer(data=request.data)
+        serializer.is_valid(raise_exception=True)
+
+        metric_type = serializer.validated_data["metric_type"]
+        name = serializer.validated_data["name"]
+        labels = serializer.validated_data["labels"]
+        value = serializer.validated_data["value"]
+
+        # Make sure emit_metric doesn't recursively call back to this view.
+        assert not current_task
+
+        try:
+            emit_metric(
+                metric_type=metric_type, name=name, labels=labels, value=value
+            )
+        except MetricNotFound as e:
+            raise DebusineAPIException(
+                title="Metric not found",
+                detail=str(e),
+                status_code=status.HTTP_404_NOT_FOUND,
+            )
+        except BadMetricType as e:
+            raise DebusineAPIException(
+                title="Bad metric type",
+                detail=str(e),
+                status_code=status.HTTP_400_BAD_REQUEST,
+            )
+
+        return Response(status=status.HTTP_204_NO_CONTENT)
