@@ -198,7 +198,7 @@ class ArtifactViewTests(TestCase):
             },
             "data": {"key1": "value1"},
         }
-        self.assertEqual(Artifact.objects.all().count(), 0)
+        self.assertEqual(Artifact.objects.count(), 0)
 
         hostname = "example.com"
 
@@ -237,7 +237,7 @@ class ArtifactViewTests(TestCase):
         )
 
         self.assertEqual(
-            artifact.files.all().count(), len(artifact_serialized["files"])
+            artifact.files.count(), len(artifact_serialized["files"])
         )
 
         file_backend = FileStore.default().get_backend_object()
@@ -895,6 +895,159 @@ class ArtifactViewTests(TestCase):
             ),
         )
 
+    def test_workspace_artifact_cross_scope_public(self) -> None:
+        """Access public artifacts across scopes."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        scope2 = self.playground.get_or_create_scope(name="scope2")
+        public_workspace = self.playground.create_workspace(
+            name="Public", scope=scope2, public=True
+        )
+        self.scenario.workspace.set_inheritance([public_workspace])
+        artifact, _ = self.playground.create_artifact(
+            workspace=public_workspace
+        )
+
+        response = self.client.get(
+            reverse(
+                "api:workspace-artifact",
+                kwargs={
+                    "workspace": self.scenario.workspace.name,
+                    "artifact_id": artifact.id,
+                },
+            ),
+            headers={
+                "Token": self.scenario.user_token.key,
+                "X-Debusine-Scope": self.scenario.scope.name,
+            },
+        )
+
+        with urlconf_scope(artifact.workspace.scope.name):
+            expected = self.artifact_as_dict_response(artifact, "testserver")
+
+        data = self.assertAPIResponseOk(response)
+        self.assertEqual(data, expected)
+
+    def test_workspace_artifact_cross_scope_private(self) -> None:
+        """Access private artifacts across scopes."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        scope2 = self.playground.get_or_create_scope(name="scope2")
+        private_workspace = self.playground.create_workspace(
+            name="Private", scope=scope2
+        )
+        self.playground.create_group_role(
+            private_workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        self.scenario.workspace.set_inheritance([private_workspace])
+        artifact, _ = self.playground.create_artifact(
+            workspace=private_workspace
+        )
+
+        response = self.client.get(
+            reverse(
+                "api:workspace-artifact",
+                kwargs={
+                    "workspace": self.scenario.workspace.name,
+                    "artifact_id": artifact.id,
+                },
+            ),
+            headers={
+                "Token": self.scenario.user_token.key,
+                "X-Debusine-Scope": self.scenario.scope.name,
+            },
+        )
+
+        with urlconf_scope(artifact.workspace.scope.name):
+            expected = self.artifact_as_dict_response(artifact, "testserver")
+
+        data = self.assertAPIResponseOk(response)
+        self.assertEqual(data, expected)
+
+    def test_workspace_artifact_cross_scope_not_inherited(self) -> None:
+        """Only inherited workspaces' artifacts are visible."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        scope2 = self.playground.get_or_create_scope(name="scope2")
+        private_workspace = self.playground.create_workspace(
+            name="Private", scope=scope2
+        )
+        # User has access, but the workspace doesn't inherit private_workspace
+        self.playground.create_group_role(
+            private_workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        artifact, _ = self.playground.create_artifact(
+            workspace=private_workspace
+        )
+
+        response = self.client.get(
+            reverse(
+                "api:workspace-artifact",
+                kwargs={
+                    "workspace": self.scenario.workspace.name,
+                    "artifact_id": artifact.id,
+                },
+            ),
+            headers={
+                "Token": self.scenario.user_token.key,
+                "X-Debusine-Scope": self.scenario.scope.name,
+            },
+        )
+        self.assertResponseProblem(
+            response,
+            "No Artifact matches the given query.",
+            status_code=status.HTTP_404_NOT_FOUND,
+        )
+
+    def test_workspace_artifact_cross_scope_invisible(self) -> None:
+        """Only user-visible artifacts are visible."""
+        self.playground.create_group_role(
+            self.scenario.workspace,
+            Workspace.Roles.VIEWER,
+            users=[self.scenario.user],
+        )
+        scope2 = self.playground.get_or_create_scope(name="scope2")
+        private_workspace = self.playground.create_workspace(
+            name="Private", scope=scope2
+        )
+        self.scenario.workspace.set_inheritance([private_workspace])
+        # User has no access to private_workspace
+        artifact, _ = self.playground.create_artifact(
+            workspace=private_workspace
+        )
+
+        response = self.client.get(
+            reverse(
+                "api:workspace-artifact",
+                kwargs={
+                    "workspace": self.scenario.workspace.name,
+                    "artifact_id": artifact.id,
+                },
+            ),
+            headers={
+                "Token": self.scenario.user_token.key,
+                "X-Debusine-Scope": self.scenario.scope.name,
+            },
+        )
+        self.assertResponseProblem(
+            response,
+            "No Artifact matches the given query.",
+            status_code=status.HTTP_404_NOT_FOUND,
+        )
+
 
 class UploadFileViewTests(TestCase):
     """Tests for UploadFileView class."""
@@ -1112,7 +1265,7 @@ class UploadFileViewTests(TestCase):
 
     def test_put_wrong_whole_file(self) -> None:
         """Server receive a file without Content-Range. Unexpected hash."""
-        files_in_uploads_before = FileUpload.objects.all().count()
+        files_in_uploads_before = FileUpload.objects.count()
 
         self.assertEqual(FileUpload.objects.count(), 0)
 
@@ -1132,9 +1285,7 @@ class UploadFileViewTests(TestCase):
             status_code=status.HTTP_409_CONFLICT,
         )
 
-        self.assertEqual(
-            files_in_uploads_before, FileUpload.objects.all().count()
-        )
+        self.assertEqual(files_in_uploads_before, FileUpload.objects.count())
         self.file_in_artifact.refresh_from_db()
         self.assertFalse(self.file_in_artifact.complete)
 
@@ -1214,7 +1365,7 @@ class UploadFileViewTests(TestCase):
 
     def test_put_two_parts_file_completed(self) -> None:
         """Server receive a file in two parts."""
-        files_in_uploads_before = FileUpload.objects.all().count()
+        files_in_uploads_before = FileUpload.objects.count()
 
         # Upload first part of the file
         start_range = 0
@@ -1238,23 +1389,20 @@ class UploadFileViewTests(TestCase):
         self.assertEqual(response.status_code, status.HTTP_201_CREATED)
 
         # Assert that debusine created the file as expected in the database
-        self.assertEqual(self.artifact.files.all().count(), 1)
+        self.assertEqual(self.artifact.files.count(), 1)
 
         self.file_in_artifact.refresh_from_db()
         file = self.file_in_artifact.file
         self.assertEqual(file.hash_digest.hex(), self.file_hash)
-        self.assertEqual(self.artifact.files.all().count(), 1)
+        self.assertEqual(self.artifact.files.count(), 1)
         self.assertEqual(FileInStore.objects.count(), 1)
 
-        self.assertEqual(
-            files_in_uploads_before,
-            FileUpload.objects.all().count(),
-        )
+        self.assertEqual(files_in_uploads_before, FileUpload.objects.count())
         self.assertTrue(self.file_in_artifact.complete)
 
     def test_put_two_parts_file_completed_wrong_hash(self) -> None:
         """Server receive a file in two parts. Hash mismatch with expected."""
-        files_in_uploads_before = FileUpload.objects.all().count()
+        files_in_uploads_before = FileUpload.objects.count()
         self.assertEqual(FileUpload.objects.count(), 0)
 
         # Upload first part of the file
@@ -1286,11 +1434,8 @@ class UploadFileViewTests(TestCase):
         )
 
         # Temporary uploaded data is removed so the client can try again:
-        self.assertEqual(FileUpload.objects.all().count(), 0)
-        self.assertEqual(
-            files_in_uploads_before,
-            FileUpload.objects.all().count(),
-        )
+        self.assertEqual(FileUpload.objects.count(), 0)
+        self.assertEqual(files_in_uploads_before, FileUpload.objects.count())
         self.file_in_artifact.refresh_from_db()
         self.assertFalse(self.file_in_artifact.complete)
 
@@ -1946,9 +2091,7 @@ class ArtifactRelationsViewTests(TestCase):
                 )
 
                 data = self.assertAPIResponseOk(response)
-                self.assertEqual(
-                    len(data), ArtifactRelation.objects.all().count()
-                )
+                self.assertEqual(len(data), ArtifactRelation.objects.count())
 
                 for artifact_relation in data:
                     ArtifactRelation.objects.get(
@@ -1977,7 +2120,7 @@ class ArtifactRelationsViewTests(TestCase):
         )
 
         data = self.assertAPIResponseOk(response)
-        self.assertEqual(len(data), ArtifactRelation.objects.all().count())
+        self.assertEqual(len(data), ArtifactRelation.objects.count())
 
         response = self.get_relations(
             artifact_id=self.artifact.id,
@@ -2029,7 +2172,7 @@ class ArtifactRelationsViewTests(TestCase):
         )
 
         data = self.assertAPIResponseOk(response)
-        self.assertEqual(len(data), ArtifactRelation.objects.all().count())
+        self.assertEqual(len(data), ArtifactRelation.objects.count())
 
     def test_post_400_artifact_id_does_not_exist(self) -> None:
         """Post the relations for a non-existing artifact: return 400."""
@@ -2061,7 +2204,7 @@ class ArtifactRelationsViewTests(TestCase):
 
     def test_post_201_created(self) -> None:
         """Add a new relation for an artifact."""
-        self.assertEqual(self.artifact.relations.all().count(), 0)
+        self.assertEqual(self.artifact.relations.count(), 0)
         relation = {
             "artifact": self.artifact.id,
             "target": self.artifact_target.id,
@@ -2105,7 +2248,7 @@ class ArtifactRelationsViewTests(TestCase):
 
     def test_post_400_invalid_target_id(self) -> None:
         """No relations added to the artifact: invalid target id."""
-        self.assertEqual(self.artifact.relations.all().count(), 0)
+        self.assertEqual(self.artifact.relations.count(), 0)
 
         relation = {
             "artifact": self.artifact.id,
@@ -2284,7 +2427,7 @@ class ArtifactRelationsViewTests(TestCase):
         response = self.delete_relation(artifact_relation.id)
 
         self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
-        self.assertEqual(self.artifact.relations.all().count(), 0)
+        self.assertEqual(self.artifact.relations.count(), 0)
 
     def test_delete_honours_scope(self) -> None:
         """Deleting relations looks up the artifact in the current scope."""
@@ -2322,7 +2465,7 @@ class ArtifactRelationsViewTests(TestCase):
         )
 
         self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
-        self.assertEqual(self.artifact.relations.all().count(), 0)
+        self.assertEqual(self.artifact.relations.count(), 0)
 
     def test_delete_private_workspace_unauthorized(self) -> None:
         """Deleting relations in private workspaces 404s to the unauthorized."""
@@ -2372,4 +2515,4 @@ class ArtifactRelationsViewTests(TestCase):
         )
 
         self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
-        self.assertEqual(self.artifact.relations.all().count(), 0)
+        self.assertEqual(self.artifact.relations.count(), 0)
