@@ -52,7 +52,7 @@ class EnrollViewTests(TestCase):
         cls.url = reverse("api:enroll")
 
     @contextlib.contextmanager
-    def assertValidationSuccessful(self) -> Generator[None, None, None]:
+    def assertValidationSuccessful(self) -> Generator[None]:
         """Ensure EnrollView passes validation."""
 
         class Reached(BaseException):
@@ -202,6 +202,7 @@ class EnrollViewTests(TestCase):
 
         # Send confirmation
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         confirmation = EnrollConfirmPayload(
             outcome=EnrollOutcome.CONFIRM, token="a" * 32
         )
@@ -241,6 +242,7 @@ class EnrollViewTests(TestCase):
 
         # Send confirmation to complete the first response
         channel_layer = get_channel_layer()
+        assert channel_layer is not None
         confirmation = EnrollConfirmPayload(
             outcome=EnrollOutcome.CONFIRM, token="a" * 32
         )
@@ -306,6 +308,7 @@ class EnrollViewTests(TestCase):
 
             # Send confirmation
             channel_layer = get_channel_layer()
+            assert channel_layer is not None
             confirmation = EnrollConfirmPayload(
                 outcome=EnrollOutcome.CONFIRM, token="a" * 32
             )
