@@ -23,6 +23,7 @@ from rest_framework import status
 from debusine.artifacts.models import RuntimeStatistics
 from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.models import Token, WorkRequest, Worker
+from debusine.db.models.work_requests import MAX_AUTOMATIC_RETRIES
 from debusine.server.views.rest import IsWorkerAuthenticated
 from debusine.server.views.tests.test_work_requests import WorkRequestTestCase
 from debusine.server.views.workers import (
@@ -30,6 +31,7 @@ from debusine.server.views.workers import (
     UpdateWorkRequestAsCompletedView,
     UpdateWorkerDynamicMetadataView,
 )
+from debusine.server.workflows.models import WorkRequestWorkflowData
 from debusine.tasks.models import (
     OutputData,
     SbuildData,
@@ -239,18 +241,74 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         )
 
     def test_get_running_work_request(self) -> None:
-        """A running work request is aborted and retried."""
+        """A running work request with no output is simply retried."""
+        work_request = self.playground.create_worker_task(
+            mark_running=True, worker=self.worker
+        )
+
+        with self.assertLogsContains(
+            f"Re-attempting work request {work_request.id}",
+            logger="debusine.server.views.workers",
+            level=logging.DEBUG,
+        ):
+            response = self.client.get(
+                reverse('api:work-request-get-next'),
+                headers={"token": self.token.key},
+            )
+
+        # The running work request was retried
+        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        work_request.refresh_from_db()
+        self.assertEqual(work_request.status, WorkRequest.Statuses.RUNNING)
+        self.assertFalse(hasattr(work_request, "superseded"))
+        self.assertEqual(work_request.workflow_data.retry_count, 1)
+        self.check_response_for_work_request(response, work_request)
+
+    def test_get_running_work_request_retry_limit(self) -> None:
+        """A work request is not simply retried after the automatic limit."""
+        work_request = self.playground.create_worker_task(
+            mark_running=True,
+            worker=self.worker,
+            workflow_data=WorkRequestWorkflowData(
+                retry_count=MAX_AUTOMATIC_RETRIES
+            ),
+        )
+
+        with self.assertLogsContains(
+            (
+                f"Cannot retry previously-running work request "
+                f"{work_request.id}: automatic retry limit exceeded."
+            ),
+            logger="debusine.server.views.workers",
+            level=logging.DEBUG,
+        ):
+            response = self.client.get(
+                reverse('api:work-request-get-next'),
+                headers={"token": self.token.key},
+            )
+
+        # The running work request was not retried
+        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
+        work_request.refresh_from_db()
+        self.assertEqual(work_request.status, WorkRequest.Statuses.ABORTED)
+        self.assertFalse(hasattr(work_request, "superseded"))
+
+    def test_get_running_partially_completed_work_request(self) -> None:
+        """A partially-completed work request is aborted and retried."""
         # Create WorkRequest pending
         work_request_pending = self.create_work_request(
             dt.datetime(2022, 1, 5, 10, 13, 20, 204242, dt.timezone.utc),
             mark_pending=True,
         )
 
-        # Create WorkRequest running
+        # Create WorkRequest running with output
         work_request_running = self.create_work_request(
             dt.datetime(2022, 1, 5, 11, 14, 22, 242178, dt.timezone.utc),
             mark_running=True,
         )
+        self.playground.create_build_log_artifact(
+            work_request=work_request_running
+        )
 
         # Request
         response = self.client.get(
@@ -267,6 +325,7 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         )
         self.assertTrue(hasattr(work_request_running, "superseded"))
         self.assertIsNone(work_request_running.superseded.worker)
+        self.assertEqual(work_request_running.workflow_data.retry_count, 0)
         self.assertEqual(
             work_request_running.superseded.workflow_data.retry_count, 1
         )
@@ -285,6 +344,9 @@ class GetNextWorkRequestViewTests(WorkRequestTestCase):
         work_request_running.task_data = {"invalid": True}
         work_request_running.configured_task_data = {"invalid": True}
         work_request_running.save()
+        self.playground.create_build_log_artifact(
+            work_request=work_request_running
+        )
 
         with self.assertLogsContains(
             "Cannot retry previously-running work request: "
@@ -439,6 +501,9 @@ class GetNextWorkRequestViewTransactionTests(TransactionTestCase):
                 self.playground.advance_work_request(
                     work_request, mark_running=True
                 )
+                self.playground.create_build_log_artifact(
+                    work_request=work_request
+                )
             assert work_requests[0].worker is not None
             assert work_requests[0].worker.token is not None
             # workflows[2] and workflows[3] have pending work requests
