@@ -16,7 +16,11 @@ from rest_framework.request import Request
 from rest_framework.response import Response
 
 from debusine.db.models import Token, WorkRequest, Worker
-from debusine.db.models.work_requests import CannotRetry, WorkRequestRetryReason
+from debusine.db.models.work_requests import (
+    CannotRetry,
+    MAX_AUTOMATIC_RETRIES,
+    WorkRequestRetryReason,
+)
 from debusine.server.exceptions import DebusineAPIException
 from debusine.server.serializers import (
     WorkRequestCompletedSerializer,
@@ -62,7 +66,7 @@ class RegisterView(BaseAPIView):
             request.auth.delete()
         else:
             Worker.objects.create_with_fqdn(
-                fqdn, token, worker_type=worker_type
+                fqdn, token=token, worker_type=worker_type
             )
 
         logger.info('Client registered. Token key: %s', token_key)
@@ -75,19 +79,44 @@ class GetNextWorkRequestView(BaseAPIView):
 
     permission_classes = [IsWorkerAuthenticated]
 
-    def get(self, request: Request) -> Response:
-        """Return the task to build."""
-        token_key = request.headers['token']
-        worker = Worker.objects.get_worker_by_token_key_or_none(token_key)
-        # This can't be None at this point, since IsWorkerAuthenticated
-        # passed.
-        assert worker is not None
+    def select_work_request(self, worker: Worker) -> WorkRequest | None:
+        """Pick our work request from the assigned queue."""
+        running_work_requests = WorkRequest.objects.running(worker=worker)
+
+        # Did the worker fail before producing anything?
+        # Then it's safe to just re-attempt execution
+        for running in running_work_requests.reassignable(worker=worker):
+            workflow_data = running.workflow_data
+            if workflow_data.retry_count >= MAX_AUTOMATIC_RETRIES:
+                logger.info(
+                    (
+                        "Cannot retry previously-running work request %i: "
+                        "automatic retry limit exceeded."
+                    ),
+                    running.id,
+                )
+                running.mark_aborted()
+                continue
+            logger.info("Re-attempting work request %i", running.id)
+            workflow_data.retry_count += 1
+            running.workflow_data = workflow_data
+            running.save()
+            return running
+
+        pending_work_requests = WorkRequest.objects.pending(worker=worker)
+
+        # Aborting running work requests may change the first pending work
+        # request that we select later, but it will still be in one of these
+        # workflows.
+        (
+            running_work_requests | pending_work_requests[:1]
+        ).lock_workflows_for_update()
 
         # This view is normally only called when a worker is idle, but if
         # the worker was restarted then the server may still think it has a
         # running work request.  In that case, retry the work request since
         # it may be in an unclear intermediate state.
-        for running in WorkRequest.objects.running(worker=worker):
+        for running in running_work_requests:
             running.mark_aborted()
             try:
                 running.retry(reason=WorkRequestRetryReason.WORKER_FAILED)
@@ -96,7 +125,18 @@ class GetNextWorkRequestView(BaseAPIView):
                     "Cannot retry previously-running work request: %s", e
                 )
 
-        work_request = WorkRequest.objects.pending(worker=worker).first()
+        return pending_work_requests.first()
+
+    def get(self, request: Request) -> Response:
+        """Return the task to build."""
+        token_key = request.headers['token']
+        worker = Worker.objects.get_worker_by_token_key_or_none(token_key)
+
+        # This can't be None at this point, since IsWorkerAuthenticated
+        # passed.
+        assert worker is not None
+
+        work_request = self.select_work_request(worker)
 
         status_code: int
         if work_request:
@@ -120,6 +160,10 @@ class UpdateWorkRequestAsCompletedView(BaseAPIView):
 
     def put(self, request: Request, work_request_id: int) -> Response:
         """Mark a work request as completed."""
+        WorkRequest.objects.filter(
+            id=work_request_id
+        ).lock_workflows_for_update()
+
         token_key = request.headers['token']
         worker = Worker.objects.get_worker_by_token_key_or_none(token_key)
 
@@ -142,7 +186,9 @@ class UpdateWorkRequestAsCompletedView(BaseAPIView):
                     output_data = OutputData.parse_obj(data["output_data"])
                 else:
                     output_data = None
-                work_request.mark_completed(data["result"], output_data)
+                work_request.mark_completed(
+                    data["result"], output_data=output_data
+                )
                 content = None
                 status_code = status.HTTP_200_OK
             else:
