@@ -124,6 +124,8 @@ class AWSEC2WorkerPool(
         self._terminate_existing_instances(worker.worker_pool_data)
         worker.worker_pool_data = {}
         worker.instance_created_at = None
+        worker.token = None
+        worker.activation_token = None
         worker.save()
 
     @override
@@ -154,7 +156,7 @@ class AWSEC2WorkerPool(
                 worker.name,
                 instance_id,
             )
-            self.terminate_worker(worker)
+            self.worker_pool.terminate_worker(worker)
 
     def _make_client(
         self, provider_account_data: AWSProviderAccountData
