@@ -139,6 +139,8 @@ class AWSEC2InstanceRequirements(BaseWorkerPoolDataModel):
     BurstablePerformance: AWSInstanceRequirementFilterEnum = (
         AWSInstanceRequirementFilterEnum.EXCLUDED
     )
+    AllowedInstanceTypes: list[str] | None = None
+    ExcludedInstanceTypes: list[str] | None = None
 
 
 class AWSEC2NetworkInterface(BaseWorkerPoolDataModel):
