@@ -20,7 +20,7 @@ from botocore.exceptions import ClientError
 
 from debusine.assets import AWSProviderAccountData, CloudProvidersType
 from debusine.db.context import context
-from debusine.db.models import Asset, WorkerPool
+from debusine.db.models import Asset, Worker, WorkerPool
 from debusine.server.worker_pools.amazon_ec2 import AWSEC2WorkerPool, log
 from debusine.server.worker_pools.cloud_init import FilesystemSetup
 from debusine.server.worker_pools.exceptions import InstanceLaunchException
@@ -751,12 +751,18 @@ class TestAWSEC2WorkerPool(TestCase):
 
     def test_terminate_missing(self) -> None:
         worker_pool = self.get_worker_pool()
+        workers: dict[str, Worker] = {}
         for instance_id in ("instance-1", "instance-2", "instance-3"):
             worker = self.playground.create_worker(
                 worker_pool=worker_pool.worker_pool
             )
             worker.worker_pool_data = {"instance-id": instance_id}
             worker.save()
+            workers[instance_id] = worker
+
+        task = self.playground.create_worker_task(
+            mark_running=True, worker=workers["instance-3"]
+        )
 
         # Inactive pool member
         worker = self.playground.create_worker(
@@ -816,6 +822,8 @@ class TestAWSEC2WorkerPool(TestCase):
             ],
             any_order=True,
         )
+        task.refresh_from_db()
+        self.assertIsNone(task.worker)
 
     def test_terminate_missing_empty(self) -> None:
         worker_pool = self.get_worker_pool()
