@@ -18,7 +18,7 @@ from hcloud.networks.domain import Network
 from responses.matchers import header_matcher, query_string_matcher
 
 from debusine.assets import CloudProvidersType, HetznerProviderAccountData
-from debusine.db.models import Asset, WorkerPool
+from debusine.db.models import Asset, Worker, WorkerPool
 from debusine.server.worker_pools.exceptions import InstanceLaunchException
 from debusine.server.worker_pools.hetzner import HetznerCloudWorkerPool
 from debusine.test.django import TestCase
@@ -293,12 +293,18 @@ class TestHetznerCloudWorkerPool(TestCase):
     @responses.activate
     def test_terminate_missing(self) -> None:
         worker_pool = self.get_worker_pool()
+        workers: dict[int, Worker] = {}
         for instance_id in (11, 12, 13):
             worker = self.playground.create_worker(
                 worker_pool=worker_pool.worker_pool
             )
             worker.worker_pool_data = {"instance-id": instance_id}
             worker.save()
+            workers[instance_id] = worker
+
+        task = self.playground.create_worker_task(
+            mark_running=True, worker=workers[13]
+        )
 
         # Inactive pool member
         worker = self.playground.create_worker(
@@ -338,6 +344,8 @@ class TestHetznerCloudWorkerPool(TestCase):
             ],
             any_order=True,
         )
+        task.refresh_from_db()
+        self.assertIsNone(task.worker)
 
     @responses.activate
     def test_terminate_missing_empty(self) -> None:
