@@ -36,7 +36,7 @@ from debusine.server.workflows.models import (
 from debusine.server.workflows.regression_tracking import (
     RegressionTrackingWorkflow,
 )
-from debusine.server.workflows.workflow_utils import lookup_result_architecture
+from debusine.server.workflows.workflow_utils import lookup_result_architectures
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BaseDynamicTaskData,
@@ -138,12 +138,11 @@ class BlhcWorkflow(
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
         items: list[tuple[str, LookupResult]] = []
         for package_build_log in self._lookup_package_build_logs():
-            items.append(
-                (
-                    lookup_result_architecture(package_build_log),
-                    package_build_log,
-                )
-            )
+            architectures = lookup_result_architectures(package_build_log)
+            assert len(architectures) == 1
+            architecture = architectures.pop()
+            items.append((architecture, package_build_log))
+
         for architecture, package_build_log in sorted(items, key=itemgetter(0)):
             self._populate_single(
                 package_build_log=package_build_log,
