@@ -14,6 +14,7 @@ from typing import NewType
 
 from debian.debian_support import Version
 
+from debusine.artifacts import SourcePackage
 from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
@@ -25,6 +26,7 @@ from debusine.client.models import LookupChildType
 from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
 from debusine.server.collections.lookup import (
     LookupResult,
+    artifact_ids,
     lookup_multiple,
     lookup_single,
     reconstruct_lookup,
@@ -32,17 +34,20 @@ from debusine.server.collections.lookup import (
 from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.models import (
     DebDiffWorkflowData,
+    DebDiffWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
 from debusine.server.workflows.workflow_utils import (
-    lookup_result_architecture,
+    lookup_result_architectures,
     lookup_result_binary_package_name,
 )
 from debusine.tasks.models import (
-    BaseDynamicTaskData,
     DebDiffData,
     DebDiffFlags,
     DebDiffInput,
+    InputArtifact,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LookupMultiple,
     LookupSingle,
 )
@@ -53,21 +58,27 @@ Architecture = NewType("Architecture", str)
 PackageArchLookupMap = dict[PackageName, dict[Architecture, LookupSingle]]
 
 
-class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
+class DebDiffWorkflow(
+    Workflow[DebDiffWorkflowData, DebDiffWorkflowDynamicData]
+):
     """DebDiff workflow."""
 
     TASK_NAME = "debdiff"
 
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    ) -> DebDiffWorkflowDynamicData:
         """
         Compute dynamic data for this workflow.
 
         :subject: package name of ``source_artifact``
         :parameter_summary: name of debian suite: old_version → new_version
+        :source_artifact_id: id of the source artifact
+        :binary_artifacts_ids: ids of the binary artifacts
+        :original_id: id of the original artifact
         """
-        new_source_data = workflow_utils.source_package_data(self)
+        source_artifact = workflow_utils.source_package(self)
+        new_source_data = SourcePackage.create_data(source_artifact.data)
         new_source_package_name = get_source_package_name(new_source_data)
         version_new = new_source_data.version
 
@@ -83,6 +94,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         )
 
         version_original = None
+        original_source_package_id = None
         if original_source_package is not None:
             assert original_source_package.artifact is not None
             version_original = Version(
@@ -90,6 +102,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
                     **original_source_package.artifact.data
                 ).version
             )
+            original_source_package_id = original_source_package.artifact.id
         else:
             original_binary_packages = (
                 self._get_binary_packages_from_source_package(
@@ -116,11 +129,54 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         else:
             parameter_summary = f"{debian_suite.name}: missing → {version_new}"
 
-        return BaseDynamicTaskData(
+        binary_artifacts_results = lookup_multiple(
+            self.data.binary_artifacts,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.workflow_root,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+
+        return DebDiffWorkflowDynamicData(
             subject=new_source_package_name,
             parameter_summary=parameter_summary,
+            source_artifact_id=source_artifact.id,
+            binary_artifacts_ids=artifact_ids(binary_artifacts_results),
+            original_id=original_source_package_id,
         )
 
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return input artifacts."""
+        return [
+            InputArtifactSingle(
+                lookup=self.data.source_artifact,
+                label="source_artifact",
+                artifact_id=(
+                    self.dynamic_data.source_artifact_id
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            ),
+            InputArtifactSingle(
+                lookup=self.data.original,
+                label="original",
+                artifact_id=(
+                    self.dynamic_data.original_id
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            ),
+            InputArtifactMultiple(
+                lookup=self.data.binary_artifacts,
+                label="binary_artifacts",
+                artifact_ids=(
+                    self.dynamic_data.binary_artifacts_ids
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            ),
+        ]
+
     def populate(self) -> None:
         """Create work requests."""
         new_source_data = workflow_utils.source_package_data(self)
@@ -147,7 +203,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
             self._populate_source_artifact_debdiff(
                 original=original_source_lookup,
                 new=self.data.source_artifact,
-                host_architecture=self.data.arch_all_host_architecture,
+                build_architecture=self.data.arch_all_build_architecture,
                 environment=environment,
                 extra_flags=self.data.extra_flags,
             )
@@ -198,8 +254,8 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
             ]
 
         if original_relevants and new_relevants:
-            host_arch = (
-                self.data.arch_all_host_architecture
+            build_arch = (
+                self.data.arch_all_build_architecture
                 if architecture == "all"
                 else architecture
             )
@@ -207,7 +263,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
                 originals=LookupMultiple.parse_obj(sorted(original_relevants)),
                 news=LookupMultiple.parse_obj(sorted(new_relevants)),
                 architecture=architecture,
-                host_architecture=host_arch,
+                build_architecture=build_arch,
                 environment=environment,
                 extra_flags=extra_flags,
             )
@@ -261,7 +317,9 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         ] = defaultdict(list)
 
         for lookup_result in original_binary_results:
-            arch = Architecture(lookup_result_architecture(lookup_result))
+            architectures = lookup_result_architectures(lookup_result)
+            assert len(architectures) == 1
+            arch = Architecture(architectures.pop())
             pkg_name = PackageName(
                 lookup_result_binary_package_name(lookup_result)
             )
@@ -318,9 +376,9 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
 
         match lookup_result.artifact.category:
             case ArtifactCategory.BINARY_PACKAGE:
-                architecture = Architecture(
-                    lookup_result_architecture(lookup_result)
-                )
+                architectures = lookup_result_architectures(lookup_result)
+                assert len(architectures) == 1
+                architecture = Architecture(architectures.pop())
                 pkg_name = PackageName(
                     lookup_result_binary_package_name(lookup_result)
                 )
@@ -417,7 +475,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         *,
         original: LookupSingle,
         new: LookupSingle,
-        host_architecture: str,
+        build_architecture: str,
         environment: LookupSingle,
         extra_flags: list[DebDiffFlags],
     ) -> WorkRequest:
@@ -432,7 +490,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
                     source_artifacts=[original, new],
                 ),
                 environment=environment,
-                host_architecture=host_architecture,
+                build_architecture=build_architecture,
                 extra_flags=extra_flags,
             ),
             workflow_data=WorkRequestWorkflowData(
@@ -451,7 +509,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
         originals: LookupMultiple,
         news: LookupMultiple,
         architecture: str,
-        host_architecture: str,
+        build_architecture: str,
         environment: LookupSingle,
         extra_flags: list[DebDiffFlags],
     ) -> WorkRequest:
@@ -462,7 +520,7 @@ class DebDiffWorkflow(Workflow[DebDiffWorkflowData, BaseDynamicTaskData]):
                     binary_artifacts=[originals, news],
                 ),
                 environment=environment,
-                host_architecture=host_architecture,
+                build_architecture=build_architecture,
                 extra_flags=extra_flags,
             ),
             workflow_data=WorkRequestWorkflowData(
