@@ -12,17 +12,30 @@
 from typing import Any
 
 from debian.debian_support import Version
-from django.db.models import Window
-from django.db.models.fields.json import KT
-from django.db.models.functions import Rank
 from django.utils import timezone
 
-from debusine.artifacts import LintianArtifact
-from debusine.artifacts.models import ArtifactCategory, TaskTypes
-from debusine.db.models import Artifact, CollectionItem, WorkRequest
+from debusine.artifacts import LintianArtifact, SourcePackage
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    DebianBinaryPackage,
+    DebianSourcePackage,
+    DebianUpload,
+    get_source_package_version,
+)
+from debusine.client.models import LookupChildType
+from debusine.db.models import (
+    Artifact,
+    ArtifactRelation,
+    WorkRequest,
+)
+from debusine.server.collections.lookup import (
+    artifact_ids,
+    lookup_multiple,
+)
 from debusine.server.workflows import workflow_utils
 from debusine.server.workflows.models import (
     LintianWorkflowData,
+    LintianWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
 from debusine.server.workflows.regression_tracking import (
@@ -31,7 +44,9 @@ from debusine.server.workflows.regression_tracking import (
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BackendType,
-    BaseDynamicTaskData,
+    InputArtifact,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LintianData,
     LintianFailOnSeverity,
     LintianInput,
@@ -46,14 +61,14 @@ from debusine.tasks.server import TaskDatabaseInterface
 
 
 class LintianWorkflow(
-    RegressionTrackingWorkflow[LintianWorkflowData, BaseDynamicTaskData]
+    RegressionTrackingWorkflow[LintianWorkflowData, LintianWorkflowDynamicData]
 ):
     """Lintian workflow."""
 
     TASK_NAME = "lintian"
 
     def _has_current_reference_qa_result(
-        self, *, architecture: str, host_architecture: str
+        self, *, architecture: str, build_architecture: str
     ) -> bool:
         """
         Return True iff we have a current reference QA result.
@@ -78,7 +93,7 @@ class LintianWorkflow(
             f"latest:lintian_{source_data.name}_{architecture}"
         )
         reference_lintian = self.qa_suite.manager.lookup(
-            f"binary:lintian_{host_architecture}"
+            f"binary:lintian_{build_architecture}"
         )
         return (
             latest_result is not None
@@ -97,7 +112,7 @@ class LintianWorkflow(
 
     def populate(self) -> None:
         """Create work requests."""
-        architectures = workflow_utils.get_architectures(
+        architectures = workflow_utils.get_lookup_architectures(
             self, self.data.binary_artifacts
         )
 
@@ -121,15 +136,15 @@ class LintianWorkflow(
         source_all_analysis_architecture: str | None = None
         if "all" in architectures:
             source_all_analysis_architecture = "all"
-        elif self.data.arch_all_host_architecture in architectures:
+        elif self.data.arch_all_build_architecture in architectures:
             source_all_analysis_architecture = (
-                self.data.arch_all_host_architecture
+                self.data.arch_all_build_architecture
             )
         elif architectures:
             # Alphabetical sorting doesn't necessarily produce good results
             # here, but this is already a last-ditch fallback case so we
             # don't worry about it too much.  If it turns out to be wrong,
-            # it's probably best to just set `arch_all_host_architecture`.
+            # it's probably best to just set `arch_all_build_architecture`.
             source_all_analysis_architecture = sorted(architectures)[0]
 
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
@@ -154,7 +169,7 @@ class LintianWorkflow(
             )
             filtered_binary_artifacts = (
                 workflow_utils.filter_artifact_lookup_by_arch(
-                    self, self.data.binary_artifacts, (arch, "all")
+                    self, self.data.binary_artifacts, {arch, "all"}
                 )
             )
 
@@ -162,8 +177,8 @@ class LintianWorkflow(
                 source_artifact=source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 output=output,
-                host_architecture=(
-                    self.data.arch_all_host_architecture
+                build_architecture=(
+                    self.data.arch_all_build_architecture
                     if arch == "all"
                     else arch
                 ),
@@ -182,7 +197,7 @@ class LintianWorkflow(
         source_artifact: LookupSingle,
         binary_artifacts: LookupMultiple,
         output: LintianOutput,
-        host_architecture: str,
+        build_architecture: str,
         environment: str,
         backend: BackendType,
         include_tags: list[str],
@@ -195,7 +210,7 @@ class LintianWorkflow(
         if (
             self.data.update_qa_results
             and self._has_current_reference_qa_result(
-                architecture=architecture, host_architecture=host_architecture
+                architecture=architecture, build_architecture=build_architecture
             )
         ):
             return
@@ -214,7 +229,7 @@ class LintianWorkflow(
                     binary_artifacts=binary_artifacts,
                 ),
                 output=output,
-                host_architecture=host_architecture,
+                build_architecture=build_architecture,
                 environment=environment,
                 backend=backend,
                 include_tags=include_tags,
@@ -271,36 +286,110 @@ class LintianWorkflow(
             # Checked by a model validator.
             assert self.data.reference_prefix
 
-            callback = WorkRequest.objects.create_workflow_callback(
-                parent=self.work_request,
-                step="regression-analysis",
-                display_name=f"Regression analysis for {architecture}",
-                visible=False,
+            regression_analysis = self.work_request_ensure_child_internal(
+                task_name="workflow",
+                workflow_data=WorkRequestWorkflowData(
+                    allow_dependency_failures=True,
+                    step="regression-analysis",
+                    display_name=f"Regression analysis for {architecture}",
+                    visible=False,
+                ),
             )
             try:
                 self.requires_artifact(
-                    callback,
+                    regression_analysis,
                     f"internal@collections/name:"
                     f"{self.data.reference_prefix}lintian-{architecture}",
                 )
             except KeyError:
                 pass
-            callback.add_dependency(wr)
+            regression_analysis.add_dependency(wr)
+            self.orchestrate_child(regression_analysis)
 
     def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    ) -> LintianWorkflowDynamicData:
         """
         Compute dynamic data for this workflow.
 
         :subject: package name of ``source_artifact``
+        :parameter_summary: package name_version
+        :source_artifact_id: id of the source artifact
+        :binary_artifacts_ids: ids of the binary artifacts
         """
-        source_data = workflow_utils.source_package_data(self)
-        return BaseDynamicTaskData(
+        source_artifact = workflow_utils.source_package(self)
+        source_data = SourcePackage.create_data(source_artifact.data)
+
+        binary_artifacts_results = lookup_multiple(
+            self.data.binary_artifacts,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.workflow_root,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+        return LintianWorkflowDynamicData(
             subject=source_data.name,
             parameter_summary=f"{source_data.name}_{source_data.version}",
+            source_artifact_id=source_artifact.id,
+            binary_artifacts_ids=artifact_ids(binary_artifacts_results),
         )
 
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return source_artifact and the binary artifacts."""
+        return [
+            InputArtifactSingle(
+                lookup=self.data.source_artifact,
+                label="source_artifact",
+                artifact_id=(
+                    self.dynamic_data.source_artifact_id
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            ),
+            InputArtifactMultiple(
+                lookup=self.data.binary_artifacts,
+                label="binary_artifacts",
+                artifact_ids=(
+                    self.dynamic_data.binary_artifacts_ids
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            ),
+        ]
+
+    def _extract_artifact_details(
+        self, artifact: Artifact | None
+    ) -> tuple[str | None, int | None, WorkRequest | None]:
+        """Extract details from a ``debian:lintian`` artifact."""
+        source_version: str | None
+        artifact_id: int | None
+        wr: WorkRequest | None
+        if artifact is None:
+            source_version = None
+            artifact_id = None
+            wr = None
+        else:
+            analyzed_artifact = (
+                artifact.relations.filter(
+                    type=ArtifactRelation.Relations.RELATES_TO
+                )
+                .earliest("id")
+                .target
+            )
+            analyzed_data = analyzed_artifact.create_data()
+            assert isinstance(
+                analyzed_data,
+                (
+                    DebianSourcePackage,
+                    DebianBinaryPackage,
+                    DebianUpload,
+                ),
+            )
+            source_version = get_source_package_version(analyzed_data)
+            artifact_id = artifact.id
+            wr = artifact.created_by_work_request
+        return source_version, artifact_id, wr
+
     @staticmethod
     def compare_qa_results_fine_grained(
         reference: Artifact | None, new: Artifact | None
@@ -336,7 +425,7 @@ class LintianWorkflow(
 
         return status, details
 
-    def callback_regression_analysis(self) -> None:
+    def callback_regression_analysis(self) -> bool:
         """
         Analyze regressions compared to reference results.
 
@@ -347,45 +436,11 @@ class LintianWorkflow(
         to handle cases where there isn't a one-to-one mapping between the
         reference results and the new results.
         """
-        # This workflow callback is only created when
-        # enable_regression_tracking is true, in which case this is checked
-        # by a model validator.
-        assert self.reference_qa_results is not None
-
-        source_data = workflow_utils.source_package_data(self)
         # Select the newest result for each architecture.
-        reference_artifacts = {
-            result.data["architecture"]: result.artifact
-            for result in self.reference_qa_results.child_items.filter(
-                child_type=CollectionItem.Types.ARTIFACT,
-                artifact__isnull=False,
-                data__task_name="lintian",
-                data__package=source_data.name,
-                data__version=source_data.version,
-                data__has_key="architecture",
-            )
-            .annotate(
-                rank_by_architecture=Window(
-                    Rank(),
-                    partition_by=KT("data__architecture"),
-                    order_by="-data__timestamp",
-                )
-            )
-            .filter(rank_by_architecture=1)
-            .select_related("artifact__created_by_work_request")
-        }
-        new_artifacts = {
-            artifact.data["architecture"]: artifact
-            for artifact in Artifact.objects.filter(
-                created_by_work_request__in=(
-                    self.work_request.children.unsuperseded()
-                    .terminated()
-                    .filter(task_type=TaskTypes.WORKER, task_name="lintian")
-                ),
-                category=ArtifactCategory.LINTIAN,
-                data__has_key="architecture",
-            ).select_related("created_by_work_request")
-        }
+        reference_artifacts = self.find_reference_artifacts("lintian")
+        new_artifacts = self.find_new_artifacts(
+            "lintian", ArtifactCategory.LINTIAN
+        )
 
         output_data = self.work_request.output_data or OutputData()
         output_data.regression_analysis = {}
@@ -393,11 +448,13 @@ class LintianWorkflow(
             set(reference_artifacts) | set(new_artifacts)
         ):
             reference = reference_artifacts.get(architecture)
-            reference_wr = (
-                None if reference is None else reference.created_by_work_request
+            reference_source_version, reference_artifact_id, reference_wr = (
+                self._extract_artifact_details(reference)
             )
             new = new_artifacts.get(architecture)
-            new_wr = None if new is None else new.created_by_work_request
+            new_source_version, new_artifact_id, new_wr = (
+                self._extract_artifact_details(new)
+            )
 
             # A single Lintian task may produce artifacts for multiple
             # architectures (e.g. amd64+all); that may result in regressions
@@ -413,13 +470,14 @@ class LintianWorkflow(
                 status = fine_grained_status
 
             output_data.regression_analysis[architecture] = RegressionAnalysis(
-                original_url=(
-                    None if reference is None else reference.get_absolute_url()
-                ),
-                new_url=(None if new is None else new.get_absolute_url()),
+                original_source_version=reference_source_version,
+                original_artifact_id=reference_artifact_id,
+                new_source_version=new_source_version,
+                new_artifact_id=new_artifact_id,
                 status=status,
                 details=details,
             )
 
         self.work_request.output_data = output_data
         self.work_request.save()
+        return True
