@@ -19,7 +19,7 @@ from debusine.artifacts.models import (
     DebianSourcePackage,
     DebianUpload,
 )
-from debusine.client.models import LookupChildType
+from debusine.client.models import LookupChildType, RuntimeParameter
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -28,7 +28,11 @@ from debusine.db.models import (
     WorkflowTemplate,
     Workspace,
 )
-from debusine.server.collections.lookup import lookup_multiple, lookup_single
+from debusine.server.collections.lookup import (
+    artifact_ids,
+    lookup_multiple,
+    lookup_single,
+)
 from debusine.server.tasks.models import (
     CopyCollectionItemsCopies,
     CopyCollectionItemsData,
@@ -36,17 +40,23 @@ from debusine.server.tasks.models import (
 from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.models import (
     PackagePublishWorkflowData,
+    PackagePublishWorkflowDynamicData,
     UpdateSuitesData,
     WorkRequestWorkflowData,
 )
-from debusine.tasks.models import BaseDynamicTaskData, LookupMultiple
+from debusine.tasks.models import (
+    InputArtifact,
+    InputArtifactMultiple,
+    InputArtifactSingle,
+    LookupMultiple,
+)
 from debusine.tasks.server import TaskDatabaseInterface
 
 logger = logging.getLogger(__name__)
 
 
 class PackagePublishWorkflow(
-    Workflow[PackagePublishWorkflowData, BaseDynamicTaskData]
+    Workflow[PackagePublishWorkflowData, PackagePublishWorkflowDynamicData]
 ):
     """Publish packages to a target suite."""
 
@@ -59,7 +69,7 @@ class PackagePublishWorkflow(
             lookup=self.data.target_suite,
             workspace=self.work_request.workspace,
             user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
+            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.COLLECTION,
         ).collection
 
@@ -90,7 +100,7 @@ class PackagePublishWorkflow(
             self.data.source_artifact,
             self.workspace,
             user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
+            workflow_root=self.work_request.workflow_root,
             expect_type=LookupChildType.ARTIFACT,
         ).artifact
         source_artifact = workflow_utils.locate_debian_source_package(
@@ -344,7 +354,7 @@ class PackagePublishWorkflow(
                     self.data.binary_artifacts,
                     self.workspace,
                     user=self.work_request.created_by,
-                    workflow_root=self.work_request.get_workflow_root(),
+                    workflow_root=self.work_request.workflow_root,
                     expect_type=LookupChildType.ARTIFACT,
                 )
             ],
@@ -376,7 +386,7 @@ class PackagePublishWorkflow(
             update_suites_callback.add_dependency(copy_collection_items)
             self.orchestrate_child(update_suites_callback)
 
-    def callback_trigger_update_suites(self) -> None:
+    def callback_trigger_update_suites(self) -> bool:
         """
         Update suites once the items have been copied.
 
@@ -392,18 +402,20 @@ class PackagePublishWorkflow(
             name=WellKnownWorkflowTemplate.UPDATE_SUITES,
             task_name="update_suites",
             workspace=self.target_suite.workspace,
+            runtime_parameters=RuntimeParameter.ANY,
         )
         WorkRequest.objects.create_workflow(
             template=template,
             data=UpdateSuitesData(only_suites=[self.target_suite.name]).dict(
                 exclude_unset=True
             ),
-        )
+        ).mark_pending()
+        return True
 
     def build_dynamic_data(
         self,
         task_database: TaskDatabaseInterface,  # noqa: U100
-    ) -> BaseDynamicTaskData:
+    ) -> PackagePublishWorkflowDynamicData:
         """
         Compute dynamic data for this workflow.
 
@@ -411,10 +423,13 @@ class PackagePublishWorkflow(
           by spaces
         :parameter_summary: source package names of ``binary_artifacts``
           separated by spaces → suite
+        :source_artifact_id: id of the source artifact
+        :binary_artifacts_ids: ids of the binary artifacts
+        :target_suite_id: collection id of the target_suite
         """
         binaries = lookup_multiple(
             self.data.binary_artifacts,
-            workflow_root=self.work_request.get_workflow_root(),
+            workflow_root=self.work_request.workflow_root,
             workspace=self.workspace,
             user=self.work_request.created_by,
             expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
@@ -431,8 +446,54 @@ class PackagePublishWorkflow(
             )
         )
 
-        return BaseDynamicTaskData(
+        target_suite_id = lookup_single(
+            self.data.target_suite,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.workflow_root,
+            expect_type=LookupChildType.COLLECTION,
+        ).collection.id
+
+        try:
+            source_artifact_id = workflow_utils.source_package(self).id
+        except LookupError:
+            source_artifact_id = None
+
+        return PackagePublishWorkflowDynamicData(
             subject=source_package_names,
             parameter_summary=f"{source_package_names} → "
             f"{self.data.target_suite}",
+            source_artifact_id=source_artifact_id,
+            binary_artifacts_ids=artifact_ids(binaries),
+            target_suite_id=target_suite_id,
         )
+
+    def get_input_artifacts(self) -> list[InputArtifact]:
+        """Return input artifacts."""
+        artifacts: list[InputArtifact] = []
+        if self.data.source_artifact is not None:
+            artifacts.append(
+                InputArtifactSingle(
+                    lookup=self.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=(
+                        self.dynamic_data.source_artifact_id
+                        if self.dynamic_data is not None
+                        else None
+                    ),
+                )
+            )
+
+        artifacts.append(
+            InputArtifactMultiple(
+                lookup=self.data.binary_artifacts,
+                label="binary_artifacts",
+                artifact_ids=(
+                    self.dynamic_data.binary_artifacts_ids
+                    if self.dynamic_data is not None
+                    else None
+                ),
+            )
+        )
+
+        return artifacts
