@@ -499,6 +499,26 @@ class BlhcWorkflowTests(
             ],
         )
 
+    def test_populate_with_upload(self) -> None:
+        upload_artifacts = self.playground.create_upload_artifacts()
+        log = self.package_build_log_artifacts[Architecture("amd64")]
+        wr = self.playground.create_workflow(
+            task_name="blhc",
+            task_data=BlhcWorkflowData(
+                source_artifact=upload_artifacts.upload.id,
+                package_build_logs=LookupMultiple.parse_obj([log.id]),
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        self.get_workflow(wr).populate()
+        children = list(wr.children.all())
+        self.assertEqual(len(children), 1)
+        child = children[0]
+        self.assertEqual(
+            child.task_data["input"], {"artifact": f"{log.id}@artifacts"}
+        )
+
     def test_populate_has_current_reference_qa_result(self) -> None:
         """The workflow does nothing with a current reference QA result."""
         self.playground.create_collection("sid", CollectionCategory.SUITE)
