@@ -33,6 +33,7 @@ from debusine.server.workflows.debdiff import Architecture
 from debusine.server.workflows.models import (
     BaseWorkflowData,
     DebDiffWorkflowData,
+    DebDiffWorkflowDynamicData,
     WorkRequestWorkflowData,
 )
 from debusine.server.workflows.tests.helpers import (
@@ -42,6 +43,8 @@ from debusine.server.workflows.tests.helpers import (
 from debusine.tasks.models import (
     BaseDynamicTaskData,
     DebDiffFlags,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LookupMultiple,
     LookupSingle,
     OutputData,
@@ -276,14 +279,12 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
     def archs_for_binaries(self, binaries: list[LookupSingle]) -> set[str]:
         archs = set()
         for lookup in binaries:
-            archs.add(
-                workflow_utils.lookup_result_architecture(
-                    lookup_single(
-                        lookup,
-                        self.workspace,
-                        user=self.playground.get_default_user(),
-                        expect_type=LookupChildType.ARTIFACT,
-                    )
+            archs |= workflow_utils.lookup_result_architectures(
+                lookup_single(
+                    lookup,
+                    self.workspace,
+                    user=self.playground.get_default_user(),
+                    expect_type=LookupChildType.ARTIFACT,
                 )
             )
 
@@ -295,6 +296,11 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
     @preserve_task_registry()
     def test_populate(self) -> None:
+        # We'll deliberately add a binary package newer than the one we're
+        # interested in, so allow this for test purposes
+        self.debian_collection.data["may_rewind_versions"] = True
+        self.debian_collection.save()
+
         # Add a binary package with older version: it will be discarded
         old_package_1 = self.playground.create_minimal_binary_package_artifact(
             package="libhello0",
@@ -444,10 +450,12 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
                             f"{bookworm_lookup}/name:hello_1.0.0_all",
                             f"{bookworm_lookup}/name:hello_1.0.0_amd64",
                         ],
-                        [
-                            self.new_binary_artifact_lookup("amd64"),
-                            self.new_binary_artifact_lookup("all"),
-                        ],
+                        sorted(
+                            [
+                                self.new_binary_artifact_lookup("all"),
+                                self.new_binary_artifact_lookup("amd64"),
+                            ]
+                        ),
                     ]
                 },
             },
@@ -481,8 +489,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
     @preserve_task_registry()
     def test_populate_binary_artifacts_upload(self) -> None:
         upload_artifacts = self.playground.create_upload_artifacts(
-            binaries=["hello", "bye"],
-            bin_architecture="amd64",
+            binaries=[("hello", "amd64"), ("bye", "amd64")],
         )
 
         work_request = self.create_debdiff_workflow(
@@ -730,40 +737,24 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
     @preserve_task_registry()
     def test_populate_package_without_source_in_original(self) -> None:
-        # Add a binary package that has the same name but is older
+        # Add binary packages that have the same name but are older
         # (to exercise returning only the newest)
-        older = self.playground.create_minimal_binary_package_artifact(
-            package="hello",
-            srcpkg_name="hello",
-            srcpkg_version="0.1.0",
-            version="0.1.0",
-            architecture="amd64",
-        )
-        self.debian_collection.manager.add_artifact(
-            older,
-            user=self.playground.get_default_user(),
-            variables={
-                "component": "main",
-                "section": "devel",
-                "priority": "optional",
-            },
-        )
-        even_older = self.playground.create_minimal_binary_package_artifact(
-            package="hello",
-            srcpkg_name="hello",
-            srcpkg_version="0.0.1",
-            version="0.0.1",
-            architecture="amd64",
-        )
-        self.debian_collection.manager.add_artifact(
-            even_older,
-            user=self.playground.get_default_user(),
-            variables={
-                "component": "main",
-                "section": "devel",
-                "priority": "optional",
-            },
-        )
+        for version in ("0.0.1", "0.1.0"):
+            self.debian_collection.manager.add_artifact(
+                self.playground.create_minimal_binary_package_artifact(
+                    package="hello",
+                    srcpkg_name="hello",
+                    srcpkg_version=version,
+                    version=version,
+                    architecture="amd64",
+                ),
+                user=self.playground.get_default_user(),
+                variables={
+                    "component": "main",
+                    "section": "devel",
+                    "priority": "optional",
+                },
+            )
 
         debdiff_workflow = self.create_debdiff_workflow(
             add_origin_source_artifact=False,
@@ -816,11 +807,51 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(work_request)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="bookworm: 1.0.0 → 2.0.0"
+            DebDiffWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="bookworm: 1.0.0 → 2.0.0",
+                original_id=self.origin_source_artifact.id,
+                source_artifact_id=self.new_source_artifact.id,
+                binary_artifacts_ids=[
+                    artifact.id
+                    for artifact in self.new_binary_artifacts.values()
+                ],
             ),
         )
 
+    @preserve_task_registry()
+    def test_get_input_artifacts(self) -> None:
+        work_request = self.create_debdiff_workflow()
+        workflow = self.get_workflow(work_request)
+
+        workflow.dynamic_data = workflow.build_dynamic_data(
+            TaskDatabase(work_request)
+        )
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=self.new_source_artifact.id,
+                ),
+                InputArtifactSingle(
+                    lookup=workflow.data.original,
+                    label="original",
+                    artifact_id=self.origin_source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[
+                        artifact.id
+                        for artifact in self.new_binary_artifacts.values()
+                    ],
+                ),
+            ],
+        )
+
     @preserve_task_registry()
     def test_compute_dynamic_data_from_binary(self) -> None:
         # Add a binary package with older version: it will be discarded
@@ -849,8 +880,15 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(work_request)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="bookworm: 1.0.0 → 2.0.0"
+            DebDiffWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="bookworm: 1.0.0 → 2.0.0",
+                original_id=None,
+                source_artifact_id=self.new_source_artifact.id,
+                binary_artifacts_ids=[
+                    artifact.id
+                    for artifact in self.new_binary_artifacts.values()
+                ],
             ),
         )
 
@@ -867,9 +905,15 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(work_request)),
-            BaseDynamicTaskData(
+            DebDiffWorkflowDynamicData(
                 subject="hello",
                 parameter_summary="bookworm: missing → 2.0.0",
+                original_id=None,
+                source_artifact_id=self.new_source_artifact.id,
+                binary_artifacts_ids=[
+                    artifact.id
+                    for artifact in self.new_binary_artifacts.values()
+                ],
             ),
         )
 
@@ -934,8 +978,7 @@ class DebDiffWorkflowTests(WorkflowTestBase[DebDiffWorkflow]):
     def test_package_arch_pairs_artifact_upload(self) -> None:
         # Introspects the relations
         upload_artifacts = self.playground.create_upload_artifacts(
-            binaries=["bye", "hello"],
-            bin_architecture="all",
+            binaries=[("bye", "all"), ("hello", "all")],
         )
 
         lookup_result = lookup_single(
