@@ -39,6 +39,7 @@ from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
     LintianWorkflowData,
+    LintianWorkflowDynamicData,
     SbuildWorkflowData,
 )
 from debusine.server.workflows.tests.helpers import (
@@ -54,6 +55,8 @@ from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BackendType,
     BaseDynamicTaskData,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LintianFailOnSeverity,
     LintianOutput,
     LookupMultiple,
@@ -87,6 +90,12 @@ class LintianWorkflowTests(
         validate: bool = True,
     ) -> LintianWorkflow:
         """Create a lintian workflow."""
+        binary_artifact_1 = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        binary_artifact_2 = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
         task_data = {
             "source_artifact": self.source_artifact.pk,
             "binary_artifacts": [
@@ -98,9 +107,17 @@ class LintianWorkflowTests(
         }
         if extra_task_data is not None:
             task_data.update(extra_task_data)
+
         wr = self.playground.create_workflow(
-            task_name="lintian", task_data=task_data, validate=validate
+            task_name="lintian",
+            task_data=task_data,
+            validate=validate,
+            internal_collection_artifacts=[
+                (binary_artifact_1, {"name": "build-amd64"}),
+                (binary_artifact_2, {"name": "build-i386"}),
+            ],
         )
+
         return self.get_workflow(wr)
 
     def orchestrate(
@@ -979,6 +996,50 @@ class LintianWorkflowTests(
                 ordered=False,
             )
 
+    def test_populate_uploads(self) -> None:
+        """The workflow accepts debian:upload artifacts in all inputs."""
+        artifacts = self.playground.create_upload_artifacts(
+            binaries=[
+                ("hello", "amd64"),
+                ("hello", "i386"),
+                ("hello-doc", "all"),
+            ]
+        )
+        architectures = ["amd64", "i386"]
+
+        wr = self.playground.create_workflow(
+            task_name="lintian",
+            task_data=LintianWorkflowData(
+                source_artifact=artifacts.upload.id,
+                binary_artifacts=LookupMultiple.parse_obj(
+                    [artifacts.upload.id]
+                ),
+                architectures=architectures,
+                vendor="debian",
+                codename="sid",
+            ),
+        )
+        self.get_workflow(wr).populate()
+        children = list(wr.children.order_by("id").all())
+        self.assertEqual(len(children), len(architectures))
+        for architecture, child in zip(architectures, children):
+            with self.subTest(architecture=architecture):
+                self.assertEqual(
+                    child.task_data["build_architecture"], architecture
+                )
+                self.assertEqual(
+                    child.task_data["input"],
+                    {
+                        "source_artifact": (f"{artifacts.source.id}@artifacts"),
+                        "binary_artifacts": sorted(
+                            f"{artifact.id}@artifacts"
+                            for artifact in artifacts.binaries
+                            if artifact.data["deb_fields"]["Architecture"]
+                            in (architecture, "all")
+                        ),
+                    },
+                )
+
     @preserve_task_registry()
     def test_populate_has_current_reference_qa_result(self) -> None:
         """The workflow does nothing with a current reference QA result."""
@@ -1187,7 +1248,7 @@ class LintianWorkflowTests(
                     )[0]
                 )
             self.playground.advance_work_request(
-                lintian, mark_pending=True, result=WorkRequest.Results.SUCCESS
+                lintian, result=WorkRequest.Results.SUCCESS
             )
 
         for architecture, result in results.items():
@@ -1319,11 +1380,72 @@ class LintianWorkflowTests(
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
-                subject="hello", parameter_summary="hello_1.0-1"
+            LintianWorkflowDynamicData(
+                subject="hello",
+                parameter_summary="hello_1.0-1",
+                source_artifact_id=source_artifact.id,
+                binary_artifacts_ids=[binary_artifact.id],
             ),
         )
 
+    def test_get_input_artifacts(self) -> None:
+        source_artifact = self.playground.create_source_artifact(
+            name="hello", version="1.0-1"
+        )
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+
+        wr = self.playground.create_workflow(
+            task_name="lintian",
+            task_data=LintianWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                vendor="debian",
+                codename="trixie",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        self.assertIsNone(workflow.dynamic_data)
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=None,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=None,
+                ),
+            ],
+        )
+
+        workflow.dynamic_data = LintianWorkflowDynamicData(
+            source_artifact_id=source_artifact.id,
+            binary_artifacts_ids=[binary_artifact.id],
+        )
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactSingle(
+                    lookup=workflow.data.source_artifact,
+                    label="source_artifact",
+                    artifact_id=source_artifact.id,
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[binary_artifact.id],
+                ),
+            ],
+        )
+
     @override_settings(DISABLE_AUTOMATIC_SCHEDULING=True)
     @preserve_task_registry()
     def test_callback_regression_analysis(self) -> None:
