@@ -9,6 +9,7 @@
 
 """Unit tests for the make_signed_source workflow."""
 from collections.abc import Sequence
+from typing import ClassVar
 
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -25,6 +26,7 @@ from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
     MakeSignedSourceWorkflowData,
+    MakeSignedSourceWorkflowDynamicData,
     SbuildWorkflowData,
     WorkRequestWorkflowData,
 )
@@ -34,7 +36,9 @@ from debusine.server.workflows.tests.helpers import (
 )
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
+    BackendType,
     BaseDynamicTaskData,
+    InputArtifactMultiple,
     LookupMultiple,
     SbuildInput,
 )
@@ -44,6 +48,26 @@ from debusine.test.test_utils import preserve_task_registry
 class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
     """Unit tests for :py:class:`MakeSignedSourceWorkflow`."""
 
+    source_artifact: ClassVar[Artifact]
+    binary_amd64: ClassVar[Artifact]
+    binary_i386: ClassVar[Artifact]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Set up common data."""
+        super().setUpTestData()
+        cls.source_artifact = cls.playground.create_source_artifact(
+            name="hello"
+        )
+        cls.binary_amd64 = (
+            cls.playground.create_minimal_binary_package_artifact(
+                architecture="amd64"
+            )
+        )
+        cls.binary_i386 = cls.playground.create_minimal_binary_package_artifact(
+            architecture="i386"
+        )
+
     def create_signing_template_artifact(self, architecture: str) -> Artifact:
         """
         Create a signing template artifact.
@@ -127,7 +151,7 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 self.orchestrate_child(make_signed_source)
 
         root = self.playground.create_workflow(task_name="examplepipeline")
-        orchestrate_workflow(root)
+        self.assertTrue(orchestrate_workflow(root))
 
         return root
 
@@ -162,12 +186,56 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
 
         self.assertEqual(
             workflow.compute_dynamic_data(TaskDatabase(wr)),
-            BaseDynamicTaskData(
+            MakeSignedSourceWorkflowDynamicData(
                 subject="hello linux-base",
                 parameter_summary="hello linux-base for debian:trixie",
+                binary_artifacts_ids=[binary_artifact.id, upload_artifact.id],
+                signing_template_artifacts_ids=[signing_artifact.id],
             ),
         )
 
+    def test_get_input_artifacts(self) -> None:
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact(
+                srcpkg_name="hello"
+            )
+        )
+        signing_artifact = self.create_signing_template_artifact("amd64")
+
+        wr = self.playground.create_workflow(
+            task_name="make_signed_source",
+            task_data=MakeSignedSourceWorkflowData(
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                vendor="debian",
+                codename="trixie",
+                signing_template_artifacts=LookupMultiple.parse_obj(
+                    [signing_artifact.id]
+                ),
+                architectures=["amd64"],
+                purpose=KeyPurpose("uefi"),
+                key="ABC123",
+            ),
+        )
+        workflow = self.get_workflow(wr)
+
+        workflow.dynamic_data = workflow.compute_dynamic_data(TaskDatabase(wr))
+
+        self.assertEqual(
+            workflow.get_input_artifacts(),
+            [
+                InputArtifactMultiple(
+                    lookup=workflow.data.binary_artifacts,
+                    label="binary_artifacts",
+                    artifact_ids=[binary_artifact.id],
+                ),
+                InputArtifactMultiple(
+                    lookup=workflow.data.signing_template_artifacts,
+                    label="signing_template_artifacts",
+                    artifact_ids=[signing_artifact.id],
+                ),
+            ],
+        )
+
     @preserve_task_registry()
     def test_populate(self) -> None:
         """Test populate."""
@@ -393,7 +461,7 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 {
                     "prefix": f"signed-source-{arch}-{template_name}|",
                     "architectures": ["all", "amd64", "i386"],
-                    "backend": "auto",
+                    "backend": BackendType.AUTO,
                     "input": {
                         "source_artifact": (
                             f"internal@collections/"
@@ -427,7 +495,9 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 work_request=extract_for_signing,
             )
             extract_for_signing.refresh_from_db()
-            extract_for_signing.mark_completed(WorkRequest.Results.SUCCESS)
+            self.playground.advance_work_request(
+                extract_for_signing, result=WorkRequest.Results.SUCCESS
+            )
             self.assertEqual(
                 [
                     result.artifact
@@ -451,7 +521,9 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 work_request=sign,
             )
             sign.refresh_from_db()
-            sign.mark_completed(WorkRequest.Results.SUCCESS)
+            self.playground.advance_work_request(
+                sign, result=WorkRequest.Results.SUCCESS
+            )
             self.assertEqual(
                 [
                     result.artifact
@@ -477,7 +549,9 @@ class MakeSignedSourceWorkflowTests(WorkflowTestBase[MakeSignedSourceWorkflow]):
                 work_request=assemble_signed_source,
             )
             assemble_signed_source.refresh_from_db()
-            assemble_signed_source.mark_completed(WorkRequest.Results.SUCCESS)
+            self.playground.advance_work_request(
+                assemble_signed_source, result=WorkRequest.Results.SUCCESS
+            )
             self.assertEqual(
                 lookup_single(
                     sbuild_workflow.task_data["input"]["source_artifact"],
