@@ -737,6 +737,80 @@ class QAWorkflowTests(WorkflowTestBase[QAWorkflow]):
             ).exists()
         )
 
+    def test_populate_uploads(self) -> None:
+        """The workflow accepts debian:upload artifacts in all inputs."""
+        self.playground.create_collection(
+            name="sid", category=CollectionCategory.SUITE
+        )
+        artifacts = self.playground.create_upload_artifacts(
+            binaries=[
+                ("hello", "amd64"),
+                ("hello", "i386"),
+                ("hello-doc", "all"),
+            ]
+        )
+        architectures = ["amd64", "i386"]
+
+        wr = self.playground.create_workflow(
+            task_name="qa",
+            task_data=QAWorkflowData(
+                source_artifact=f"{artifacts.upload.id}@artifacts",
+                binary_artifacts=LookupMultiple.parse_obj(
+                    [artifacts.upload.id]
+                ),
+                package_build_logs=LookupMultiple.parse_obj(
+                    [self.package_build_log.id]
+                ),
+                architectures=architectures,
+                vendor="debian",
+                codename="sid",
+                qa_suite=f"sid@{CollectionCategory.SUITE}",
+                enable_autopkgtest=True,
+                enable_reverse_dependencies_autopkgtest=True,
+                enable_debdiff=True,
+                enable_lintian=True,
+                enable_piuparts=True,
+                enable_blhc=True,
+            ),
+        )
+        self.get_workflow(wr).populate()
+        children = list(wr.children.order_by("id").all())
+        expected_src_artifact = f"{artifacts.upload.id}@artifacts"
+        expected_bin_artifacts = [expected_src_artifact]
+        expected: dict[str, dict[str, Any]] = {
+            "autopkgtest": {
+                "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
+            },
+            "blhc": {
+                "source_artifact": expected_src_artifact,
+            },
+            "debdiff": {
+                "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
+            },
+            "lintian": {
+                "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
+            },
+            "piuparts": {
+                "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
+            },
+            "reverse_dependencies_autopkgtest": {
+                "source_artifact": expected_src_artifact,
+                "binary_artifacts": expected_bin_artifacts,
+            },
+        }
+        self.assertEqual(len(children), len(expected))
+        for child in children:
+            with self.subTest(task_name=child.task_name):
+                expected_data = expected[child.task_name]
+                for k, v in expected_data.items():
+                    self.assertEqual(
+                        child.task_data.get(k), v, f"task_data[{k}]"
+                    )
+
     @preserve_task_registry()
     def test_populate_enable_reverse_dependencies_autopkgtest(self) -> None:
         """Populate can enable reverse_dependencies_autopkgtest."""
