@@ -544,7 +544,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(
         upload_artifacts = self.playground.create_upload_artifacts(
             src_name="python-urllib3",
             version="2.0.7-2",
-            binaries=["python3-urllib3"],
+            binaries=[("python3-urllib3", "all")],
         )
         workflow = self.create_rdep_autopkgtest_workflow(
             extra_task_data={
