@@ -16,6 +16,7 @@ from debusine.artifacts.models import (
     CollectionCategory,
     TaskTypes,
 )
+from debusine.client.models import RuntimeParameter
 from debusine.db.models import (
     TaskDatabase,
     WorkRequest,
@@ -28,6 +29,7 @@ from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.tests.helpers import WorkflowTestBase
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
+    BackendType,
     BaseDynamicTaskData,
 )
 
@@ -40,14 +42,15 @@ class UpdateEnvironmentsWorkflowTests(
     def create_update_environments_template(
         self,
         name: str = "update_environments",
-        task_data: dict[str, Any] | None = None,
+        static_parameters: dict[str, Any] | None = None,
     ) -> WorkflowTemplate:
         """Create an update_environments WorkflowTemplate."""
         return WorkflowTemplate.objects.create(
             name=name,
             workspace=default_workspace(),
             task_name="update_environments",
-            task_data=task_data or {},
+            static_parameters=static_parameters or {},
+            runtime_parameters=RuntimeParameter.ANY,
         )
 
     def test_create_orchestrator(self) -> None:
@@ -76,7 +79,7 @@ class UpdateEnvironmentsWorkflowTests(
     def test_create_work_requests(self) -> None:
         """Create specified work requests."""
         template = self.create_update_environments_template(
-            task_data={
+            static_parameters={
                 "vendor": "debian",
                 "targets": [
                     {
@@ -92,7 +95,10 @@ class UpdateEnvironmentsWorkflowTests(
                         "codenames": ["trixie"],
                         "codename_aliases": {"trixie": ["sid"]},
                         "variants": [None, "autopkgtest", "sbuild"],
-                        "backends": ["unshare", "incus-lxc"],
+                        "backends": [
+                            BackendType.UNSHARE,
+                            BackendType.INCUS_LXC,
+                        ],
                         "architectures": ["amd64", "arm64"],
                         "simplesystemimagebuild_template": {
                             "bootstrap_options": {"variant": "minbase"},
@@ -112,7 +118,7 @@ class UpdateEnvironmentsWorkflowTests(
         )
         wr = self.playground.create_workflow(template, task_data={})
         self.assertEqual(wr.status, WorkRequest.Statuses.PENDING)
-        orchestrate_workflow(wr)
+        self.assertTrue(orchestrate_workflow(wr))
 
         children = list(wr.children.order_by("id"))
         self.assertEqual(len(children), 3)
@@ -176,18 +182,18 @@ class UpdateEnvironmentsWorkflowTests(
                 },
             )
             for codename, variant, backend in (
-                ("trixie", None, "unshare"),
-                ("trixie", None, "incus-lxc"),
-                ("trixie", "autopkgtest", "unshare"),
-                ("trixie", "autopkgtest", "incus-lxc"),
-                ("trixie", "sbuild", "unshare"),
-                ("trixie", "sbuild", "incus-lxc"),
-                ("sid", None, "unshare"),
-                ("sid", None, "incus-lxc"),
-                ("sid", "autopkgtest", "unshare"),
-                ("sid", "autopkgtest", "incus-lxc"),
-                ("sid", "sbuild", "unshare"),
-                ("sid", "sbuild", "incus-lxc"),
+                ("trixie", None, BackendType.UNSHARE),
+                ("trixie", None, BackendType.INCUS_LXC),
+                ("trixie", "autopkgtest", BackendType.UNSHARE),
+                ("trixie", "autopkgtest", BackendType.INCUS_LXC),
+                ("trixie", "sbuild", BackendType.UNSHARE),
+                ("trixie", "sbuild", BackendType.INCUS_LXC),
+                ("sid", None, BackendType.UNSHARE),
+                ("sid", None, BackendType.INCUS_LXC),
+                ("sid", "autopkgtest", BackendType.UNSHARE),
+                ("sid", "autopkgtest", BackendType.INCUS_LXC),
+                ("sid", "sbuild", BackendType.UNSHARE),
+                ("sid", "sbuild", BackendType.INCUS_LXC),
             )
         ]
 
@@ -251,7 +257,7 @@ class UpdateEnvironmentsWorkflowTests(
     def test_event_reaction(self) -> None:
         """The event reaction created by the workflow can be handled."""
         template = self.create_update_environments_template(
-            task_data={
+            static_parameters={
                 "vendor": "debian",
                 "targets": [
                     {
@@ -267,7 +273,7 @@ class UpdateEnvironmentsWorkflowTests(
             }
         )
         wr = self.playground.create_workflow(template, task_data={})
-        orchestrate_workflow(wr)
+        self.assertTrue(orchestrate_workflow(wr))
         child = WorkRequest.objects.get(parent=wr)
         child.status = WorkRequest.Statuses.PENDING
         collection = self.playground.create_collection(
@@ -279,7 +285,9 @@ class UpdateEnvironmentsWorkflowTests(
             work_request=child,
         )
 
-        child.mark_completed(WorkRequest.Results.SUCCESS)
+        self.playground.advance_work_request(
+            child, result=WorkRequest.Results.SUCCESS
+        )
 
         item = collection.manager.lookup(
             "match:format=tarball:codename=bookworm:architecture=amd64"
