@@ -31,6 +31,8 @@ from debusine.tasks import BaseTask
 from debusine.tasks.models import (
     ActionUpdateCollectionWithArtifacts,
     BaseDynamicTaskData,
+    InputArtifactMultiple,
+    InputArtifactSingle,
     LookupMultiple,
 )
 from debusine.test.django import TestCase
@@ -137,6 +139,7 @@ class WorkflowTests(TestCase):
         )
         workflow_root_task = workflow_root.get_task()
         assert isinstance(workflow_root_task, ExampleWorkflow)
+        assert workflow_root.internal_collection is not None
 
         child = workflow_root.create_child_worker("noop")
 
@@ -151,8 +154,8 @@ class WorkflowTests(TestCase):
             child, category, name, data=data, artifact_filters=artifact_filters
         )
 
-        promise = CollectionItem.objects.active().get(
-            parent_collection=workflow_root.internal_collection, name=name
+        promise = workflow_root.internal_collection.child_items.active().get(
+            name=name
         )
         self.assertEqual(promise.child_type, CollectionItem.Types.BARE)
         self.assertEqual(promise.category, BareDataCategory.PROMISE)
@@ -194,8 +197,8 @@ class WorkflowTests(TestCase):
             child, category, name, data=data, artifact_filters=artifact_filters
         )
 
-        item = CollectionItem.objects.active().get(
-            parent_collection=workflow_root.internal_collection, name=name
+        item = workflow_root.internal_collection.child_items.active().get(
+            name=name
         )
         self.assertEqual(item.child_type, CollectionItem.Types.ARTIFACT)
         self.assertEqual(item.category, ArtifactCategory.TEST)
@@ -466,3 +469,40 @@ class WorkflowTests(TestCase):
 
                 # Returned the same as had been created
                 self.assertEqual(wr_created, wr_returned)
+
+    @preserve_task_registry()
+    def test_get_input_artifacts_ids(self) -> None:
+        class ExampleWorkflow(
+            SampleWorkflow[BaseWorkflowData, BaseDynamicTaskData]
+        ):
+            """Concrete workflow instance to use for tests."""
+
+            def populate(self) -> None:
+                """Unused abstract method from Workflow."""
+                raise NotImplementedError()
+
+        wr = self.playground.create_workflow(task_name="exampleworkflow")
+        wf = wr.get_task()
+
+        mock_artifacts = [
+            InputArtifactSingle(
+                artifact_id=1, lookup=1, label="source_artifact"
+            ),
+            InputArtifactMultiple(
+                artifact_ids=None,
+                lookup=LookupMultiple.parse_obj(
+                    ["internal@collections/name:buildlog-amd64"]
+                ),
+                label="package_log",
+            ),
+            InputArtifactMultiple(
+                artifact_ids=[5, 6],
+                lookup=LookupMultiple.parse_obj([5, 6]),
+                label="binary_artifacts",
+            ),
+        ]
+
+        with mock.patch.object(
+            wf, "get_input_artifacts", return_value=mock_artifacts
+        ):
+            self.assertEqual(wf.get_input_artifacts_ids(), [1, 5, 6])
