@@ -10,9 +10,9 @@
 """Utility functions for workflows."""
 import functools
 from collections.abc import Collection as AbcCollection
-from collections.abc import Iterable, Sequence
+from collections.abc import Sequence
 from operator import attrgetter
-from typing import Any, TYPE_CHECKING, get_args
+from typing import Any, TYPE_CHECKING, cast, get_args
 
 from django.db.models import QuerySet
 
@@ -28,7 +28,7 @@ from debusine.artifacts.models import (
     DebianSystemTarball,
     DebianUpload,
     DebusinePromise,
-    get_architecture,
+    get_architectures,
     get_source_package_name,
 )
 from debusine.client.models import LookupChildType
@@ -114,20 +114,20 @@ def lookup_result_artifact_category(result: LookupResult) -> str:
         )
 
 
-def lookup_result_architecture(result: LookupResult) -> str:
-    """Get architecture from result of looking up an artifact."""
+def lookup_result_architectures(result: LookupResult) -> set[str]:
+    """Get binary package architectures present within LookupResult."""
     if result.artifact is not None:
         artifact_data = result.artifact.create_data()
         if not isinstance(artifact_data, get_args(ArtifactDataWithArch)):
             raise ValueError(f"Unexpected type: {type(artifact_data).__name__}")
-        return get_architecture(artifact_data)
+        return get_architectures(artifact_data)
     elif result.collection_item is not None:
         architecture = result.collection_item.data.get("architecture")
         if type(architecture) is not str:
             raise ValueError(
                 f"Cannot determine architecture for lookup result: {result}"
             )
-        return architecture
+        return {architecture}
     else:
         raise ValueError(
             "Unexpected result: must have collection_item or artifact"
@@ -169,7 +169,7 @@ def lookup_result_binary_package_name(result: LookupResult) -> str:
 def filter_artifact_lookup_by_arch(
     workflow: "Workflow[Any, Any]",
     lookup: LookupMultiple,
-    architectures: Iterable[str],
+    architectures: set[str],
 ) -> LookupMultiple:
     """Filter an artifact lookup by architecture."""
     results = lookup_multiple(
@@ -181,25 +181,41 @@ def filter_artifact_lookup_by_arch(
     )
     relevant: list[str] = []
     for result in results:
-        arch_in_lookup = lookup_result_architecture(result)
-        if arch_in_lookup in architectures:
+        arch_in_lookup = lookup_result_architectures(result)
+        if arch_in_lookup.issubset(architectures):
             relevant.append(
                 reconstruct_lookup(
                     result, workflow_root=workflow.work_request.workflow_root
                 )
             )
+        elif arch_in_lookup.intersection(architectures):
+            # Uploads are the only artifacts with multiple architectures at
+            # this time.
+            assert result.artifact is not None
+            assert result.artifact.category == ArtifactCategory.UPLOAD
+            for binary_package in follow_artifact_relations(
+                result.artifact,
+                ArtifactRelation.Relations.EXTENDS,
+                ArtifactCategory.BINARY_PACKAGE,
+            ):
+                pkg_data = cast(
+                    DebianBinaryPackage, binary_package.create_data()
+                )
+                pkg_architectures = get_architectures(pkg_data)
+                if pkg_architectures.issubset(architectures):
+                    relevant.append(f"{binary_package.id}@artifacts")
 
     return LookupMultiple.parse_obj(sorted(relevant))
 
 
-def get_architectures(
+def get_lookup_architectures(
     workflow: "Workflow[Any, Any]", lookup: LookupMultiple
 ) -> set[str]:
     """
     Return set with all the architectures in the artifacts from the lookup.
 
     The architectures are extracted from each lookup result using
-    :py:func:`lookup_result_architecture`.
+    :py:func:`lookup_result_architectures`.
     """
     results = lookup_multiple(
         lookup,
@@ -209,7 +225,10 @@ def get_architectures(
         expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
     )
 
-    return {lookup_result_architecture(result) for result in results}
+    architectures: set[str] = set()
+    for result in results:
+        architectures |= lookup_result_architectures(result)
+    return architectures
 
 
 def follow_artifact_relations(
