@@ -36,7 +36,7 @@ class GnuPGError(Exception):
 @contextmanager
 def gpg_ephemeral_context(
     temp_dir: Path,
-) -> Generator[gpg.Context, None, None]:
+) -> Generator[gpg.Context]:
     """Run gpg, cleaning up properly afterwards."""
     home = temp_dir / "gpg"
     home.mkdir(mode=0o700)
