@@ -173,7 +173,10 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             "debian:source-package\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            source_package_id, destination, tarball=mock.ANY
+            source_package_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_fetch_input_missing_key(self) -> None:
@@ -206,7 +209,10 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             f"Signing key openpgp:{fingerprint} does not exist\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            upload_id, destination, tarball=mock.ANY
+            upload_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_fetch_input_permission_denied(self) -> None:
@@ -252,7 +258,10 @@ class DebsignTests(SignTestMixin, TestCase, DjangoTestCase):
             f"{upload_id} with openpgp:{key.fingerprint}.\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            upload_id, destination, tarball=mock.ANY
+            upload_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_run_error(self) -> None:
