@@ -299,7 +299,10 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             f"{unsigned_id} with openpgp-repository:{fingerprint}.\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            unsigned_id, destination, tarball=mock.ANY
+            unsigned_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_fetch_input_mismatching_permission_check_responses(self) -> None:
@@ -365,7 +368,10 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             f"{asset_permission_checks[1]} != {asset_permission_checks[0]}\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            unsigned_id, destination, tarball=mock.ANY
+            unsigned_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_fetch_input_missing_key(self) -> None:
@@ -405,7 +411,10 @@ class SignRepositoryIndexTests(SignTestMixin, TestCase, DjangoTestCase):
             f"Signing key openpgp-repository:{fingerprint} does not exist\n",
         )
         debusine_mock.download_artifact.assert_called_once_with(
-            unsigned_id, destination, tarball=mock.ANY
+            unsigned_id,
+            destination,
+            tarball=mock.ANY,
+            workspace=task.workspace_name,
         )
 
     def test_run_error(self) -> None:
