@@ -32,7 +32,7 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianBinaryPackage,
-    get_architecture,
+    get_architectures,
     get_binary_package_name,
     get_binary_package_version,
 )
@@ -114,7 +114,9 @@ class AssembleSignedSource(
         assert isinstance(template.data, DebianBinaryPackage)
         package_name = get_binary_package_name(template.data)
         package_version = get_binary_package_version(template.data)
-        package_architecture = get_architecture(template.data)
+        architectures = get_architectures(template.data)
+        assert len(architectures) == 1
+        package_architectures = architectures.pop()
 
         return AssembleSignedSourceDynamicData(
             environment_id=self.get_environment(
@@ -128,7 +130,7 @@ class AssembleSignedSource(
             ).get_ids(),
             subject=package_name,
             parameter_summary=f"{package_name}_{package_version} "
-            f"({package_architecture})",
+            f"({package_architectures})",
         )
 
     def get_input_artifacts_ids(self) -> list[int]:
@@ -160,12 +162,14 @@ class AssembleSignedSource(
         artifact_id: int,
         target_category: ArtifactCategory,
         relation_type: RelationType = RelationType.RELATES_TO,
-    ) -> Generator[int, None, None]:
+    ) -> Generator[int]:
         assert self.debusine is not None
 
         for relation in self.debusine.relation_list(artifact_id=artifact_id):
             if relation.type == relation_type:
-                artifact = self.debusine.artifact_get(relation.target)
+                artifact = self.debusine.artifact_get(
+                    relation.target, workspace=self.workspace_name
+                )
                 if artifact.category == target_category:
                     yield relation.target
 
@@ -248,18 +252,13 @@ class AssembleSignedSource(
         return True
 
     def _run_cmd_or_raise(
-        self,
-        cmd: list[str],
-        execute_directory: PurePath,
-        *,
-        override_cmd_name: str | None = None,
+        self, cmd: list[str], execute_directory: PurePath
     ) -> None:
-        cmd_name = override_cmd_name or cmd[0]
         self.logger.info("Executing: %s", shlex.join(cmd))
         returncode = self.run_cmd(cmd, Path(execute_directory))
-        self.logger.info("%s exited with code %s", cmd_name, returncode)
+        self.logger.info("%s exited with code %s", cmd[0], returncode)
         if returncode != 0:
-            raise AssembleError(f"{cmd_name} exited with code {returncode}")
+            raise AssembleError(f"{cmd[0]} exited with code {returncode}")
 
     def _pull_from_executor(self, source: PurePath, target: Path) -> None:
         """Pull a subdirectory from the executor, preserving timestamps."""
@@ -275,17 +274,7 @@ class AssembleSignedSource(
         source_tar_path = PurePath(f"{source}.tar")
         target_tar_path = Path(f"{target}.tar")
         self._run_cmd_or_raise(
-            [
-                "tar",
-                # Executors don't currently support cwd:
-                # https://salsa.debian.org/freexian-team/debusine/-/issues/434
-                "-C",
-                str(source.parent),
-                "-cf",
-                str(source_tar_path),
-                source.name,
-            ],
-            source.parent,
+            ["tar", "-cf", str(source_tar_path), source.name], source.parent
         )
         self.executor_instance.file_pull(source_tar_path, target_tar_path)
         subprocess.run(
@@ -311,16 +300,7 @@ class AssembleSignedSource(
         )
         self.executor_instance.file_push(source_tar_path, target_tar_path)
         self._run_cmd_or_raise(
-            [
-                "tar",
-                # Executors don't currently support cwd:
-                # https://salsa.debian.org/freexian-team/debusine/-/issues/434
-                "-C",
-                str(target.parent),
-                "--one-top-level",
-                "-xf",
-                str(target_tar_path),
-            ],
+            ["tar", "--one-top-level", "-xf", str(target_tar_path)],
             target.parent,
         )
 
@@ -415,17 +395,7 @@ class AssembleSignedSource(
 
         remote_source_path = self._remote_execute_directory / "source"
         self._push_to_executor(local_source_path, remote_source_path)
-        self._run_cmd_or_raise(
-            [
-                # Executors don't currently support cwd:
-                # https://salsa.debian.org/freexian-team/debusine/-/issues/434
-                "sh",
-                "-c",
-                f"cd {remote_source_path} && dpkg-source -b .",
-            ],
-            local_source_path,
-            override_cmd_name="dpkg-source",
-        )
+        self._run_cmd_or_raise(["dpkg-source", "-b", "."], remote_source_path)
         with open(local_source_path / "debian" / "changelog") as changelog_file:
             changelog = Changelog(changelog_file, max_blocks=1)
         remote_changes_path = (
@@ -433,23 +403,15 @@ class AssembleSignedSource(
             / f"{changelog.package}_{changelog.version}_source.changes"
         )
         distribution = changelog.distributions.split()[0]
-        dpkg_genchanges = [
-            "dpkg-genchanges",
-            "-S",
-            f"-DDistribution={distribution}",
-            "-UCloses",
-            f"-O{remote_changes_path}",
-        ]
         self._run_cmd_or_raise(
             [
-                # Executors don't currently support cwd:
-                # https://salsa.debian.org/freexian-team/debusine/-/issues/434
-                "sh",
-                "-c",
-                f"cd {remote_source_path} && {shlex.join(dpkg_genchanges)}",
+                "dpkg-genchanges",
+                "-S",
+                f"-DDistribution={distribution}",
+                "-UCloses",
+                f"-O{remote_changes_path}",
             ],
-            local_source_path,
-            override_cmd_name="dpkg-genchanges",
+            remote_source_path,
         )
         return remote_changes_path
 
