@@ -20,7 +20,9 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianPackageBuildLog,
+    get_architectures,
     get_source_package_name,
+    get_source_package_version,
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
@@ -56,6 +58,7 @@ class Blhc(
         Resolve artifact lookups for this task.
 
         :subject: source package name out of the analyzed build log
+        :parameter_summary: source package name and version (arch)
         """
         input_artifact = task_database.lookup_single_artifact(
             self.data.input.artifact
@@ -68,6 +71,8 @@ class Blhc(
         )
         assert isinstance(input_artifact.data, DebianPackageBuildLog)
         package_name = get_source_package_name(input_artifact.data)
+        package_version = get_source_package_version(input_artifact.data)
+        architectures = " ".join(sorted(get_architectures(input_artifact.data)))
 
         return BlhcDynamicData(
             environment_id=self.get_environment(
@@ -82,6 +87,8 @@ class Blhc(
                 update={"bd_uninstallable": None}
             ),
             subject=package_name,
+            parameter_summary=f"{package_name}_{package_version} "
+            f"({architectures})",
         )
 
     def get_input_artifacts_ids(self) -> list[int]:
@@ -210,9 +217,3 @@ class Blhc(
         :return: True for success, False failure.
         """
         return returncode in [0, 1]
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        # TODO: copy the source package information in dynamic task data and
-        # use them here if available
-        return "blhc"
