@@ -408,7 +408,9 @@ class IncusExecutorMixin:
     incus_driver: str
     _image_id: str | None = None
 
-    def __init__(self, debusine_api: Debusine, system_image_id: int):
+    def __init__(
+        self, debusine_api: Debusine, system_image_id: int, workspace: str
+    ):
         """
         Instantiate an IncusExecutor.
 
@@ -417,7 +419,7 @@ class IncusExecutorMixin:
         """
         self._extracted: Path | None = None
         self._image_cache = self.image_cache_class(
-            debusine_api, self.image_category
+            debusine_api, self.image_category, workspace=workspace
         )
         self.system_image = self._image_cache.image_artifact(system_image_id)
 
