@@ -25,9 +25,10 @@ from debusine.tasks.executors.base import (
     InstanceInterface,
 )
 from debusine.tasks.executors.images import ImageCache
+from debusine.tasks.models import BackendType
 
 
-class QemuExecutor(ExecutorInterface, backend_name="qemu"):
+class QemuExecutor(ExecutorInterface, backend_name=BackendType.QEMU):
     """Support the qemu(1) executor."""
 
     system_image: ArtifactResponse
@@ -36,14 +37,18 @@ class QemuExecutor(ExecutorInterface, backend_name="qemu"):
 
     image_category = ExecutorImageCategory.IMAGE
 
-    def __init__(self, debusine_api: Debusine, system_image_id: int):
+    def __init__(
+        self, debusine_api: Debusine, system_image_id: int, workspace: str
+    ):
         """
         Instantiate a QemuExecutor.
 
         :param debusine_api: The object to use the debusine client API.
         :param system_image_id: An artifact ID pointing to the system image.
         """
-        self._image_cache = ImageCache(debusine_api, self.image_category)
+        self._image_cache = ImageCache(
+            debusine_api, self.image_category, workspace=workspace
+        )
         self.system_image = self._image_cache.image_artifact(system_image_id)
 
     @classmethod
@@ -63,9 +68,10 @@ class QemuExecutor(ExecutorInterface, backend_name="qemu"):
 
         Return a path to the image or name, as appropriate for the backend.
         """
-        if "python3" not in self.system_image.data["pkglist"]:
+        if "python3-minimal" not in self.system_image.data["pkglist"]:
             raise ImageImportError(
-                "Image doesn't contain an python3, required by qemu backend."
+                "Image doesn't contain a python3-minimal, required by qemu "
+                "backend."
             )
         self._local_path = self._image_cache.download_image(self.system_image)
         return self.image_name()
