@@ -93,7 +93,7 @@ class IncusImageCacheCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
         super().setUp()
         self.debusine_api = create_autospec(Debusine)
         self.image_cache = self.image_cache_cls(
-            self.debusine_api, self.artifact_category
+            self.debusine_api, self.artifact_category, workspace="workspace"
         )
         self.image_cache.image_cache_path = Path("/tmp/not-used")
 
@@ -556,7 +556,9 @@ class IncusExecutorCommonTests(ExternalTaskHelperMixin[Noop], TestCase):
         self.run_incus_cmd = run_incus_cmd_patcher.start()
         self.addCleanup(run_incus_cmd_patcher.stop)
 
-        self.executor = self.executor_cls(self.debusine_api, 42)
+        self.executor = self.executor_cls(
+            self.debusine_api, 42, workspace="workspace"
+        )
 
     def test_available(self) -> None:
         """Test that available() returns True if incus is available."""
