@@ -44,7 +44,9 @@ class UnshareExecutorTests(ExternalTaskHelperMixin[Noop], TestCase):
         self.debusine_api = MagicMock(spec=Debusine)
         self.image_artifact = self.mock_image_download(self.debusine_api)
 
-        self.executor = UnshareExecutor(self.debusine_api, 42)
+        self.executor = UnshareExecutor(
+            self.debusine_api, 42, workspace="workspace"
+        )
 
     def test_backend_name(self) -> None:
         """Test that the backend_name attribute was set."""
