@@ -23,7 +23,10 @@ from debusine.artifacts.models import (
     ArtifactCategory,
     CollectionCategory,
     DebianBinaryPackage,
+    DebianUpload,
+    get_architectures,
     get_binary_package_name,
+    get_binary_package_version,
 )
 from debusine.client.models import RelationType
 from debusine.tasks import BaseTaskWithExecutor, RunCommandTask
@@ -89,6 +92,7 @@ class ExtractForSigning(
         Resolve artifact lookups for this task.
 
         :subject: binary package name of ``template_artifact``
+        :parameter_summary: same as subject (architecture)
         """
         template_artifact = task_database.lookup_single_artifact(
             self.data.input.template_artifact
@@ -101,12 +105,15 @@ class ExtractForSigning(
         )
         assert isinstance(template_artifact.data, DebianBinaryPackage)
         binary_package_name = get_binary_package_name(template_artifact.data)
+        version = get_binary_package_version(template_artifact.data)
 
         binary_artifacts = task_database.lookup_multiple_artifacts(
             self.data.input.binary_artifacts
         )
         individual_binary_artifact_ids = []
         upload_artifact_ids = []
+        architectures = set()
+
         for i, binary_artifact in enumerate(binary_artifacts):
             self.ensure_artifact_categories(
                 configuration_key=f"input.binary_artifacts[{i}]",
@@ -119,12 +126,19 @@ class ExtractForSigning(
             match binary_artifact.category:
                 case ArtifactCategory.BINARY_PACKAGE:
                     individual_binary_artifact_ids.append(binary_artifact.id)
+                    assert isinstance(binary_artifact.data, DebianBinaryPackage)
+                    architectures |= get_architectures(binary_artifact.data)
+
                 case ArtifactCategory.UPLOAD:
                     upload_artifact_ids.append(binary_artifact.id)
+                    assert isinstance(binary_artifact.data, DebianUpload)
+                    architectures |= get_architectures(binary_artifact.data)
+
                 case _ as unreachable:
                     raise AssertionError(
                         f"Unexpected artifact category: {unreachable}"
                     )
+
         if upload_artifact_ids:
             individual_binary_artifact_ids.extend(
                 task_database.find_related_artifacts(
@@ -134,6 +148,8 @@ class ExtractForSigning(
                 ).get_ids()
             )
 
+        architectures_str = ", ".join(sorted(architectures))
+
         return ExtractForSigningDynamicData(
             environment_id=self.get_environment(
                 task_database,
@@ -143,6 +159,8 @@ class ExtractForSigning(
             input_template_artifact_id=template_artifact.id,
             input_binary_artifacts_ids=individual_binary_artifact_ids,
             subject=binary_package_name,
+            parameter_summary=f"{binary_package_name}_{version} "
+            f"({architectures_str})",
         )
 
     def get_input_artifacts_ids(self) -> list[int]:
@@ -218,18 +236,13 @@ class ExtractForSigning(
         return True
 
     def _run_cmd_or_raise(
-        self,
-        cmd: list[str],
-        execute_directory: PurePath,
-        *,
-        override_cmd_name: str | None = None,
+        self, cmd: list[str], execute_directory: PurePath
     ) -> None:
-        cmd_name = override_cmd_name or cmd[0]
         self.logger.info("Executing: %s", shlex.join(cmd))
         returncode = self.run_cmd(cmd, Path(execute_directory))
-        self.logger.info("%s exited with code %s", cmd_name, returncode)
+        self.logger.info("%s exited with code %s", cmd[0], returncode)
         if returncode != 0:
-            raise ExtractError(f"{cmd_name} exited with code {returncode}")
+            raise ExtractError(f"{cmd[0]} exited with code {returncode}")
 
     def _pull_from_executor(self, source: PurePath, target: Path) -> None:
         """Pull a subdirectory from the executor, preserving timestamps."""
@@ -245,17 +258,7 @@ class ExtractForSigning(
         source_tar_path = PurePath(f"{source}.tar")
         target_tar_path = Path(f"{target}.tar")
         self._run_cmd_or_raise(
-            [
-                "tar",
-                # Executors don't currently support cwd:
-                # https://salsa.debian.org/freexian-team/debusine/-/issues/434
-                "-C",
-                str(source.parent),
-                "-cf",
-                str(source_tar_path),
-                source.name,
-            ],
-            source.parent,
+            ["tar", "-cf", str(source_tar_path), source.name], source.parent
         )
         self.executor_instance.file_pull(source_tar_path, target_tar_path)
         subprocess.run(
@@ -401,7 +404,3 @@ class ExtractForSigning(
         if self._local_execute_directory is not None:
             shutil.rmtree(self._local_execute_directory)
         super().cleanup()
-
-    def get_label(self) -> str:
-        """Return the task label."""
-        return "extract signing input"
