@@ -220,7 +220,9 @@ class Sbuild(
         assert self.dynamic_data
 
         source_artifact_id = self.dynamic_data.input_source_artifact_id
-        artifact = self.debusine.artifact_get(source_artifact_id)
+        artifact = self.debusine.artifact_get(
+            source_artifact_id, workspace=self.workspace_name
+        )
         if artifact.category != ArtifactCategory.SOURCE_PACKAGE:
             self.append_to_log_file(
                 "fetch_input.log",
@@ -240,7 +242,9 @@ class Sbuild(
         self.fetch_artifact(source_artifact_id, destination)
 
         for artifact_id in self.dynamic_data.input_extra_binary_artifacts_ids:
-            artifact = self.debusine.artifact_get(artifact_id)
+            artifact = self.debusine.artifact_get(
+                artifact_id, workspace=self.workspace_name
+            )
             if artifact.category in (
                 ArtifactCategory.BINARY_PACKAGE,
                 ArtifactCategory.UPLOAD,
