@@ -91,6 +91,11 @@ class AssembleSignedSourceTests(
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
+        # self.task.workspace_name and self.task.work_request_id are set by
+        # the Worker.
+        self.task.work_request_id = 147
+        self.task.workspace_name = "testing"
+
         self.task_db = FakeTaskDatabase(
             single_lookups={
                 # environment
@@ -211,7 +216,6 @@ class AssembleSignedSourceTests(
 
     def test_fetch_input_template_wrong_category(self) -> None:
         """fetch_input checks the category of the template artifact."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -235,7 +239,6 @@ class AssembleSignedSourceTests(
 
     def test_fetch_input_template_no_files(self) -> None:
         """fetch_input fails if the template has no .deb files."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -260,7 +263,6 @@ class AssembleSignedSourceTests(
 
     def test_fetch_input_template_multiple_deb_files(self) -> None:
         """fetch_input fails if the template has multiple .deb files."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -304,7 +306,6 @@ class AssembleSignedSourceTests(
 
     def test_fetch_input_signed_wrong_category(self) -> None:
         """fetch_input checks the category of signed artifacts."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -329,7 +330,6 @@ class AssembleSignedSourceTests(
 
     def test_fetch_input_signed_gathers_files_and_built_using(self) -> None:
         """fetch_input gathers information from signed artifacts."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -375,7 +375,9 @@ class AssembleSignedSourceTests(
                 ]
             )
 
-        def artifact_get(artifact_id: int) -> ArtifactResponse:
+        def artifact_get(
+            artifact_id: int, workspace: str  # noqa: U100
+        ) -> ArtifactResponse:
             return create_artifact_response(
                 id=6, category=artifact_categories[artifact_id]
             )
@@ -942,7 +944,6 @@ class AssembleSignedSourceTests(
 
     def test_upload_artifacts_execution_failed(self) -> None:
         """upload_artifacts does not upload anything if execution failed."""
-        self.task.work_request_id = 147
         debusine_mock = self.mock_debusine()
         execute_directory = self.create_temporary_directory()
 
@@ -966,8 +967,6 @@ class AssembleSignedSourceTests(
             source="hello-signed",
             version="1.0+1",
         )
-        self.task.work_request_id = 147
-        self.task.workspace_name = "testing"
         self.task._built_using_ids = [3]
         self.task._source_package_artifact = SourcePackage.create(
             name="hello-signed",
@@ -1009,8 +1008,6 @@ class AssembleSignedSourceTests(
 
     def test_execute_end_to_end(self) -> None:
         """End-to-end execution works."""
-        self.task.work_request_id = 147
-        self.task.workspace_name = "testing"
         self.task.dynamic_data = AssembleSignedSourceDynamicData(
             environment_id=1, template_id=11, signed_ids=[12]
         )
@@ -1109,7 +1106,10 @@ class AssembleSignedSourceTests(
         signed_tarball_path = directory / "hello-signed_1.0+1.tar.xz"
 
         def download_artifact(
-            artifact_id: int, destination: Path, tarball: bool = False
+            artifact_id: int,
+            destination: Path,
+            tarball: bool = False,
+            workspace: str | None = None,  # noqa: U100
         ) -> ArtifactResponse:
             assert not tarball
             for file_name in artifacts[artifact_id].files:
@@ -1131,7 +1131,9 @@ class AssembleSignedSourceTests(
                 ]
             )
 
-        def artifact_get(artifact_id: int) -> ArtifactResponse:
+        def artifact_get(
+            artifact_id: int, workspace: str  # noqa: U100
+        ) -> ArtifactResponse:
             return create_artifact_response(
                 id=6, category=artifact_categories[artifact_id]
             )
