@@ -9,11 +9,11 @@
 
 """Unit tests for the Autopkgtest class."""
 import copy
-import datetime
+import datetime as dt
 import shlex
 import textwrap
 from pathlib import Path
-from typing import cast
+from typing import Any, cast
 from unittest.mock import MagicMock, call, create_autospec, patch
 
 try:
@@ -32,7 +32,6 @@ from debusine.artifacts.models import DebianAutopkgtestResultStatus as rs
 from debusine.artifacts.models import (
     DebianAutopkgtestSource,
     DebianBinaryPackage,
-    DebianBinaryPackages,
     DebianSourcePackage,
     DebianUpload,
 )
@@ -48,7 +47,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     WorkerType,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, CollectionInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
@@ -68,7 +67,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             "source_artifact": 5,
             "binary_artifacts": [10, 11],
         },
-        "host_architecture": "amd64",
+        "build_architecture": "amd64",
         "environment": "debian/match:codename=bookworm",
     }
 
@@ -77,22 +76,64 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         super().setUp()
         self.configure_task()
 
-        # self.task.workspace_name and self.task.work_request_id are set by
-        # the Worker.
-        self.task.workspace_name = "testing"
-        self.task.work_request_id = 170
-        self.task.dynamic_data = AutopkgtestDynamicData(
-            environment_id=1,
-            input_source_artifact_id=5,
-            input_binary_artifacts_ids=[10, 11],
-        )
-
     def tearDown(self) -> None:
         """Delete debug log files directory if it exists."""
         if self.task._debug_log_files_directory:
             self.task._debug_log_files_directory.cleanup()
         super().tearDown()
 
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
+    ) -> None:
+        """Perform further setup."""
+        super().configure_task(task_data, override, remove)
+
+        # self.task.workspace_name and self.task.work_request_id are set by
+        # the Worker.
+        self.task.workspace_name = "testing"
+        self.task.work_request_id = 170
+
+        self.task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=autopkgtest",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifact
+                (5, None): ArtifactInfo(
+                    id=5,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (LookupMultiple.parse_obj([10, 11]), None): [
+                    ArtifactInfo(
+                        id=10,
+                        category=ArtifactCategory.BINARY_PACKAGE,
+                        data=self.create_binary_package_data("hello"),
+                    ),
+                    ArtifactInfo(
+                        id=11,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self.create_upload_data("hello"),
+                    ),
+                ],
+            },
+        )
+
     def mock_executor(self) -> None:
         """Set up a fake unshare executor."""
         mock = create_autospec(UnshareExecutor, instance=True)
@@ -287,10 +328,10 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         with self.assertRaises(TaskConfigError):
             self.configure_task(override={"something": "test"})
 
-    def test_configure_host_architecture_is_required(self) -> None:
-        """Missing required field "host_architecture": raise exception."""
+    def test_configure_build_architecture_is_required(self) -> None:
+        """Missing required field "build_architecture": raise exception."""
         with self.assertRaises(TaskConfigError):
-            self.configure_task(remove=["host_architecture"])
+            self.configure_task(remove=["build_architecture"])
 
     def test_configure_environment_is_required(self) -> None:
         """Missing required field "environment": raise exception."""
@@ -299,7 +340,12 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_configure_backend_choices(self) -> None:
         """Field "backend" accept correct values."""
-        for backend in ["auto", "incus-lxc", "incus-vm", "unshare"]:
+        for backend in [
+            BackendType.AUTO,
+            BackendType.INCUS_LXC,
+            BackendType.INCUS_VM,
+            BackendType.UNSHARE,
+        ]:
             self.configure_task(override={"backend": backend})
 
     def test_configure_backend_invalid_choice(self) -> None:
@@ -310,7 +356,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
     def test_configure_backend_default_value(self) -> None:
         """Field "backend" default value is "auto"."""
         self.configure_task(remove=["backend"])
-        self.assertEqual(self.task.data.backend, "auto")
+        self.assertEqual(self.task.data.backend, BackendType.AUTO)
 
     def test_configure_backend_include_tests(self) -> None:
         """Field "include_tests" is allowed."""
@@ -466,7 +512,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_can_run_on(self) -> None:
         """can_run_on returns True if unshare and autopkgtest are available."""
-        architecture = self.task.data.host_architecture
+        architecture = self.task.data.build_architecture
         self.assertTrue(
             self.task.can_run_on(
                 {
@@ -533,18 +579,6 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             deb_control_files=[],
         )
 
-    def create_binary_packages_data(
-        self, srcpkg_name: str
-    ) -> DebianBinaryPackages:
-        """Create sample `debian:binary-packages` artifact data."""
-        return DebianBinaryPackages(
-            srcpkg_name=srcpkg_name,
-            srcpkg_version="1.0",
-            version="1.0",
-            architecture="amd64",
-            packages=[],
-        )
-
     def create_upload_data(self, name: str) -> DebianUpload:
         """Create sample `debian:upload` artifact data."""
         return DebianUpload(
@@ -557,53 +591,15 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=autopkgtest",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=10,
-                        category=ArtifactCategory.BINARY_PACKAGES,
-                        data=self.create_binary_packages_data("hello"),
-                    ),
-                    ArtifactInfo(
-                        id=11,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self.create_upload_data("hello"),
-                    ),
-                ],
-            },
-        )
-
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             AutopkgtestDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
                 input_binary_artifacts_ids=[10, 11],
                 input_context_artifacts_ids=[],
                 subject="hello",
+                parameter_summary="hello_1.0 (amd64)",
                 runtime_context="amd64:unshare",
                 configuration_context="sid",
             ),
@@ -615,75 +611,42 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         self.task.data.input.context_artifacts = LookupMultiple.parse_obj(
             {"collection": "bookworm@debian:suite"}
         )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=autopkgtest",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=10,
-                        category=ArtifactCategory.BINARY_PACKAGE,
-                        data=self.create_binary_package_data("hello"),
-                    ),
-                    ArtifactInfo(
-                        id=11,
-                        category=ArtifactCategory.BINARY_PACKAGES,
-                        data=self.create_binary_packages_data("hello"),
-                    ),
-                    ArtifactInfo(
-                        id=12,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self.create_upload_data("hello"),
-                    ),
-                ],
-                # input.context_artifacts
-                (self.task.data.input.context_artifacts, None): [
-                    ArtifactInfo(
-                        id=2,
-                        category=ArtifactCategory.BINARY_PACKAGE,
-                        data=self.create_binary_package_data("context"),
-                    ),
-                    ArtifactInfo(
-                        id=3,
-                        category=ArtifactCategory.BINARY_PACKAGES,
-                        data=self.create_binary_packages_data("context"),
-                    ),
-                    ArtifactInfo(
-                        id=4,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self.create_upload_data("context"),
-                    ),
-                ],
-            },
-        )
+        self.task_db.multiple_lookups[(binary_artifacts_lookup, None)] = [
+            ArtifactInfo(
+                id=10,
+                category=ArtifactCategory.BINARY_PACKAGE,
+                data=self.create_binary_package_data("hello"),
+            ),
+            ArtifactInfo(
+                id=12,
+                category=ArtifactCategory.UPLOAD,
+                data=self.create_upload_data("hello"),
+            ),
+        ]
+        self.task_db.multiple_lookups[
+            (self.task.data.input.context_artifacts, None)
+        ] = [
+            ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.BINARY_PACKAGE,
+                data=self.create_binary_package_data("context"),
+            ),
+            ArtifactInfo(
+                id=4,
+                category=ArtifactCategory.UPLOAD,
+                data=self.create_upload_data("context"),
+            ),
+        ]
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             AutopkgtestDynamicData(
                 environment_id=1,
                 input_source_artifact_id=5,
-                input_binary_artifacts_ids=[10, 11, 12],
-                input_context_artifacts_ids=[2, 3, 4],
+                input_binary_artifacts_ids=[10, 12],
+                input_context_artifacts_ids=[2, 4],
                 subject="hello",
+                parameter_summary="hello_1.0 (amd64)",
                 runtime_context="amd64:unshare",
                 configuration_context="sid",
             ),
@@ -695,63 +658,33 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         source_artifact's category is unexpected.
         """
-        binary_artifacts_lookup = LookupMultiple.parse_obj([10, 11])
         self.task.data.input.context_artifacts = LookupMultiple.parse_obj(
             {"collection": "bookworm@debian:suite"}
         )
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=autopkgtest",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifact
-                (5, None): ArtifactInfo(
-                    id=5,
-                    category=ArtifactCategory.BINARY_PACKAGE,
-                    data=DebianBinaryPackage(
-                        srcpkg_name="hello",
-                        srcpkg_version="1.0",
-                        deb_fields={},
-                        deb_control_files=[],
-                    ),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=10,
-                        category=ArtifactCategory.BINARY_PACKAGES,
-                        data=self.create_binary_packages_data("hello"),
-                    ),
-                    ArtifactInfo(
-                        id=11,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self.create_upload_data("hello"),
-                    ),
-                ],
-                # input.context_artifacts
-                (self.task.data.input.context_artifacts, None): [
-                    ArtifactInfo(
-                        id=2,
-                        category=ArtifactCategory.BINARY_PACKAGES,
-                        data=self.create_binary_packages_data("context"),
-                    ),
-                    ArtifactInfo(
-                        id=3,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self.create_upload_data("context"),
-                    ),
-                ],
-            },
+        self.task_db.single_lookups[(5, None)] = ArtifactInfo(
+            id=5,
+            category=ArtifactCategory.BINARY_PACKAGE,
+            data=DebianBinaryPackage(
+                srcpkg_name="hello",
+                srcpkg_version="1.0",
+                deb_fields={},
+                deb_control_files=[],
+            ),
         )
+        self.task_db.multiple_lookups[
+            (self.task.data.input.context_artifacts, None)
+        ] = [
+            ArtifactInfo(
+                id=2,
+                category=ArtifactCategory.BINARY_PACKAGE,
+                data=self.create_binary_package_data("context"),
+            ),
+            ArtifactInfo(
+                id=3,
+                category=ArtifactCategory.UPLOAD,
+                data=self.create_upload_data("context"),
+            ),
+        ]
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -759,7 +692,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             "category: 'debian:binary-package'. Valid categories: "
             r"\['debian:source-package', 'debian:upload'\]$",
         ):
-            self.task.compute_dynamic_data(task_db)
+            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -845,6 +778,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                 ],
             }
         )
+        self.set_dynamic_data()
         self.mock_executor()
         assert self.task.executor
         self.task.executor.system_image.data["codename"] = "jessie"
@@ -857,6 +791,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_cmdline_extra_repositories_trixie(self) -> None:
         """Ensure the --copy= parameter is passed to autopkgtest."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
         self.task._prepare_executor()
@@ -877,6 +812,67 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
         )
         self.assertIn("--copy=/tmp/key.asc:/etc/apt/keyrings/key.asc", cmdline)
 
+    def test_cmdline_extra_debusine_repositories(self) -> None:
+        """Extra repositories can be hosted by Debusine."""
+        self.configure_task(
+            override={
+                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
+            }
+        )
+        self.task_db.single_collection_lookups[
+            ("trixie-local@debian:suite", None)
+        ] = CollectionInfo(
+            id=1,
+            scope_name="scope",
+            workspace_name="workspace",
+            category=CollectionCategory.SUITE,
+            name="trixie-local",
+            data={"components": ["main"]},
+        )
+        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
+            "deb.example.com"
+        )
+        self.task_db.suite_signing_keys[1] = "\n".join(
+            (
+                "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                "",
+                "ABCDEFGHI",
+                "-----END PGP PUBLIC KEY BLOCK-----",
+            )
+        )
+        self.set_dynamic_data()
+        debusine_mock = self.mock_debusine()
+        self.mock_image_download(debusine_mock)
+        self.task._prepare_executor()
+        destination = self.create_temporary_directory()
+
+        self.task.write_extra_repository_config("trixie", destination)
+        cmdline = self.task._cmdline()
+
+        self.assertEqual(self.task.extra_repository_keys, [])
+        [extra_repository_source] = self.task.extra_repository_sources
+        self.assertEqual(
+            extra_repository_source.read_text(),
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://deb.example.com/scope/workspace
+                Suites: trixie-local
+                Components: main
+                Signed-By:
+                 -----BEGIN PGP PUBLIC KEY BLOCK-----
+                 .
+                 ABCDEFGHI
+                 -----END PGP PUBLIC KEY BLOCK-----
+                """
+            ),
+        )
+        self.assertIn(
+            f"--copy={extra_repository_source}:"
+            f"/etc/apt/sources.list.d/{extra_repository_source.name}",
+            cmdline,
+        )
+
     def test_cmdline_apt_default_release_not_included(self) -> None:
         """Command line NOT include "--apt-default-release={distribution}."""
         self.mock_executor()
@@ -904,6 +900,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_cmdline_includes_executor(self) -> None:
         """Command line include "-- backend"."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
 
@@ -999,6 +996,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_select_default_backend(self) -> None:
         """We select Autopkgtest.DEFAULT_BACKEND if backend is auto."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
 
@@ -1015,6 +1013,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_select_specified_backend(self) -> None:
         """We select the specified backend if backend is not auto."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
 
@@ -1025,7 +1024,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         assert self.task.executor
 
-        self.assertEqual(self.task.executor.backend_name, "unshare")
+        self.assertEqual(self.task.executor.backend_name, BackendType.UNSHARE)
 
     def test_environment_codename_executor(self) -> None:
         """Test _environment_codename gets the executor distribution."""
@@ -1042,6 +1041,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_fetch_input_no_context_artifacts(self) -> None:
         """Download "source_artifact" and "binary_artifacts"."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
         workspace = "Testing"
@@ -1049,7 +1049,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             id=10,
             workspace=workspace,
             category="Test",
-            created_at=datetime.datetime.now(),
+            created_at=dt.datetime.now(),
             data={},
             download_tar_gz_url=pydantic.parse_obj_as(
                 pydantic.AnyUrl, "https://example.com/not-used"
@@ -1085,7 +1085,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             id=10,
             workspace=workspace,
             category="Test",
-            created_at=datetime.datetime.now(),
+            created_at=dt.datetime.now(),
             data={},
             download_tar_gz_url=pydantic.parse_obj_as(
                 pydantic.AnyUrl, "https://example.com/not-used"
@@ -1098,10 +1098,24 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         self.task.fetch_input(destination)
 
-        calls = [call(data_input.source_artifact, destination, tarball=False)]
+        calls = [
+            call(
+                data_input.source_artifact,
+                destination,
+                tarball=False,
+                workspace="testing",
+            )
+        ]
 
         for binary_artifact_id in data_input.binary_artifacts:
-            calls.append(call(binary_artifact_id, destination, tarball=False))
+            calls.append(
+                call(
+                    binary_artifact_id,
+                    destination,
+                    tarball=False,
+                    workspace="testing",
+                )
+            )
 
         debusine_mock.download_artifact.assert_has_calls(calls)
 
@@ -1110,6 +1124,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
     def test_fetch_input_context_artifacts(self) -> None:
         """Download the "context_artifacts"."""
+        self.set_dynamic_data()
         debusine_mock = self.mock_debusine()
         self.mock_image_download(debusine_mock)
 
@@ -1124,7 +1139,14 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
 
         calls = []
         for context_artifact_id in data_input.context_artifacts:
-            calls.append(call(context_artifact_id, destination, tarball=False))
+            calls.append(
+                call(
+                    context_artifact_id,
+                    destination,
+                    tarball=False,
+                    workspace="testing",
+                )
+            )
 
         debusine_mock.download_artifact.assert_has_calls(calls)
 
@@ -1292,7 +1314,7 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
                         pydantic.AnyUrl, "https://some-url.com/test"
                     ),
                 ),
-                architecture=self.task.data.host_architecture,
+                architecture=self.task.data.build_architecture,
                 distribution="debian:bookworm",
             ),
         )
@@ -1354,26 +1376,3 @@ class AutopkgtestTests(ExternalTaskHelperMixin[Autopkgtest], TestCase):
             self.task.check_directory_for_consistency_errors(build_directory),
             [],
         )
-
-    def test_get_label_dynamic_data_is_none(self) -> None:
-        """Test get_label if dynamic_data.subject is None."""
-        self.assertEqual(self.task.get_label(), "autopkgtest")
-
-    def test_get_label_dynamic_data_subject_is_none(self) -> None:
-        """Test get_label if dynamic_data is None."""
-        self.task.dynamic_data = AutopkgtestDynamicData(
-            environment_id=1,
-            input_source_artifact_id=2,
-            input_binary_artifacts_ids=[],
-        )
-        self.assertEqual(self.task.get_label(), "autopkgtest")
-
-    def test_get_label_dynamic_data_subject_is_hello(self) -> None:
-        """Test get_label if dynamic_data.subject is set."""
-        self.task.dynamic_data = AutopkgtestDynamicData(
-            environment_id=1,
-            input_source_artifact_id=2,
-            input_binary_artifacts_ids=[],
-            subject="hello",
-        )
-        self.assertEqual(self.task.get_label(), "autopkgtest hello")
