@@ -361,7 +361,7 @@ class BlhcTests(ExternalTaskHelperMixin[Blhc], TestCase):
         self.assertTrue(self.task.fetch_input(download_directory))
 
         debusine_mock.download_artifact.assert_called_once_with(
-            1, download_directory, tarball=False
+            1, download_directory, tarball=False, workspace="testing"
         )
 
         f1_contents = "Compiling gkrust v0.1.0 (/build/rust)"
