@@ -10,6 +10,7 @@
 """Unit tests for the debdiff task support on the worker."""
 import itertools
 import re
+from typing import Any
 from unittest.mock import call
 
 from debusine.artifacts import DebDiffArtifact
@@ -41,13 +42,52 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
 
     SAMPLE_TASK_DATA = {
         "input": {"source_artifacts": [421, 123]},
-        "host_architecture": "amd64",
+        "build_architecture": "amd64",
         "environment": "debian/match:codename=bookworm",
     }
 
     def setUp(self) -> None:
         super().setUp()
-        self.task = DebDiff(self.SAMPLE_TASK_DATA)
+        self.configure_task()
+
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
+    ) -> None:
+        """Perform further setup."""
+        super().configure_task(task_data, override, remove)
+
+        self.task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # input.source_artifacts
+                (421, None): ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.0", type="dpkg", dsc_fields={}
+                    ),
+                ),
+                (123, None): ArtifactInfo(
+                    id=123,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=DebianSourcePackage(
+                        name="hello", version="1.1", type="dpkg", dsc_fields={}
+                    ),
+                ),
+            }
+        )
 
     def test_configure_fails_with_missing_required_data(
         self,
@@ -85,104 +125,63 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifacts
-                (421, None): ArtifactInfo(
-                    id=421,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-                (123, None): ArtifactInfo(
-                    id=123,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            }
-        )
-
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=[421, 123],
                 input_binary_artifacts_ids=None,
                 subject="source:hello",
+                parameter_summary="hello: 1.0 → 1.1 (source)",
             ),
         )
 
     def test_compute_dynamic_subject_binary_artifacts(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([500]), None): [
-                    ArtifactInfo(
-                        id=500,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-                (LookupMultiple.parse_obj([501]), None): [
-                    ArtifactInfo(
-                        id=501,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-            },
-        )
-
         self.configure_task(
             override={"input": {"binary_artifacts": [[500], [501]]}}
         )
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([500]), None): [
+                ArtifactInfo(
+                    id=500,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Version": "1.0-1",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+            (LookupMultiple.parse_obj([501]), None): [
+                ArtifactInfo(
+                    id=501,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Version": "1.1-1",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+        }
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=None,
                 input_binary_artifacts_ids=[[500], [501]],
                 subject="binary:hello",
+                parameter_summary="hello: 1.0-1 → 1.1-1 (amd64)",
             ),
         )
 
@@ -193,58 +192,45 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         There is no source package, and binary packages are from different
         source packages.
         """
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([500]), None): [
-                    ArtifactInfo(
-                        id=500,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-                (LookupMultiple.parse_obj([501]), None): [
-                    ArtifactInfo(
-                        id=501,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "linux-base",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-            },
-        )
-
         self.configure_task(
             override={"input": {"binary_artifacts": [[500], [501]]}}
         )
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([500]), None): [
+                ArtifactInfo(
+                    id=500,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Version": "1.0-1",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+            (LookupMultiple.parse_obj([501]), None): [
+                ArtifactInfo(
+                    id=501,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "linux-base",
+                            "Version": "1.1-1",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+        }
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             DebDiffDynamicData(
                 environment_id=1,
                 input_source_artifacts_ids=None,
@@ -253,122 +239,49 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
             ),
         )
 
-    def test_compute_dynamic_data_raise_task_config_no_lookup_source(
-        self,
-    ) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifacts
-                ("internal@111", None): None,
-                ("internal@112", None): None,
-            }
-        )
-
-        self.configure_task(
-            override={
-                "input": {'source_artifacts': ["internal@111", "internal@112"]}
-            }
-        )
-
-        with self.assertRaisesRegex(
-            TaskConfigError,
-            re.escape(
-                "Could not look up one of source_artifacts. "
-                "Source artifacts lookup result: [None, None]"
-            ),
-        ):
-            self.task.compute_dynamic_data(task_db)
-
     def test_compute_dynamic_data_raise_task_config_error_source(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # input.source_artifacts
-                (421, None): ArtifactInfo(
-                    id=421,
-                    category=ArtifactCategory.BINARY_PACKAGES,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-                (123, None): ArtifactInfo(
-                    id=123,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=DebianSourcePackage(
-                        name="hello", version="1.0", type="dpkg", dsc_fields={}
-                    ),
-                ),
-            }
+        self.task_db.single_lookups[(421, None)] = ArtifactInfo(
+            id=421,
+            category=ArtifactCategory.BINARY_PACKAGE,
+            data=DebianSourcePackage(
+                name="hello", version="1.0", type="dpkg", dsc_fields={}
+            ),
         )
 
         with self.assertRaisesRegex(
             TaskConfigError,
             r"^input.source_artifacts: unexpected artifact category: "
-            r"'debian:binary-packages'. "
+            r"'debian:binary-package'. "
             r"Valid categories: \['debian:source-package'\]$",
         ):
-            self.task.compute_dynamic_data(task_db)
+            self.task.compute_dynamic_data(self.task_db)
 
     def test_compute_dynamic_data_raise_task_config_error_binary(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([500]), None): [
-                    ArtifactInfo(
-                        id=500,
-                        category=ArtifactCategory.SOURCE_PACKAGE,
-                        data=EmptyArtifactData(),
-                    ),
-                ],
-                (LookupMultiple.parse_obj([501]), None): [
-                    ArtifactInfo(
-                        id=501,
-                        category=ArtifactCategory.BINARY_PACKAGE,
-                        data=DebianBinaryPackage(
-                            srcpkg_name="linux-base",
-                            srcpkg_version="1.0",
-                            deb_fields={},
-                            deb_control_files=[],
-                        ),
-                    ),
-                ],
-            },
-        )
-
         self.configure_task(
             override={"input": {"binary_artifacts": [[500], [501]]}}
         )
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([500]), None): [
+                ArtifactInfo(
+                    id=500,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                ),
+            ],
+            (LookupMultiple.parse_obj([501]), None): [
+                ArtifactInfo(
+                    id=501,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="linux-base",
+                        srcpkg_version="1.0",
+                        deb_fields={},
+                        deb_control_files=[],
+                    ),
+                ),
+            ],
+        }
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -376,142 +289,112 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
             r"'debian:source-package'. Valid categories: "
             r"\['debian:binary-package', 'debian:upload'\]$",
         ):
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
 
     def test_compute_dynamic_data_raise_task_config_error_categories(
         self,
     ) -> None:
         """Raise TaskConfigError: invalid categories binary artifacts."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([500]), None): [
-                    ArtifactInfo(
-                        id=500,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-                (LookupMultiple.parse_obj([501]), None): [
-                    ArtifactInfo(
-                        id=501,
-                        category=ArtifactCategory.BINARY_PACKAGE,
-                        data=DebianBinaryPackage(
-                            srcpkg_name="linux-base",
-                            srcpkg_version="1.0",
-                            deb_fields={},
-                            deb_control_files=[],
-                        ),
-                    ),
-                ],
-            },
-        )
-
         self.configure_task(
             override={"input": {"binary_artifacts": [[500], [501]]}}
         )
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([500]), None): [
+                ArtifactInfo(
+                    id=500,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+            (LookupMultiple.parse_obj([501]), None): [
+                ArtifactInfo(
+                    id=501,
+                    category=ArtifactCategory.BINARY_PACKAGE,
+                    data=DebianBinaryPackage(
+                        srcpkg_name="linux-base",
+                        srcpkg_version="1.0",
+                        deb_fields={},
+                        deb_control_files=[],
+                    ),
+                ),
+            ],
+        }
 
         with self.assertRaisesRegex(
             TaskConfigError,
             r'^All binary artifacts must have the same category. '
             r'Found "debian:upload" and "debian:binary-package"$',
         ):
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
 
     def test_compute_dynamic_data_raise_invalid_category(self) -> None:
         """Raise TaskConfigError: invalid categories binary artifacts."""
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([500, 501]), None): [
-                    ArtifactInfo(
-                        id=500,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                    ArtifactInfo(
-                        id=501,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-                (LookupMultiple.parse_obj([502, 503]), None): [
-                    ArtifactInfo(
-                        id=502,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                    ArtifactInfo(
-                        id=503,
-                        category=ArtifactCategory.UPLOAD,
-                        data=DebianUpload(
-                            type="dpkg",
-                            changes_fields={
-                                "Architecture": "amd64",
-                                "Source": "hello",
-                                "Files": [{"name": "hello.deb"}],
-                            },
-                        ),
-                    ),
-                ],
-            },
-        )
-
         self.configure_task(
             override={"input": {"binary_artifacts": [[500, 501], [502, 503]]}}
         )
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([500, 501]), None): [
+                ArtifactInfo(
+                    id=500,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+                ArtifactInfo(
+                    id=501,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+            (LookupMultiple.parse_obj([502, 503]), None): [
+                ArtifactInfo(
+                    id=502,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+                ArtifactInfo(
+                    id=503,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Source": "hello",
+                            "Files": [{"name": "hello.deb"}],
+                        },
+                    ),
+                ),
+            ],
+        }
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -519,30 +402,15 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
             r"than one artifact all must be of category debian:binary-package. "
             r"Found: debian:upload$",
         ):
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
 
     def test_compute_dynamic_data_raise_binary_cannot_be_zero(self) -> None:
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # binary artifacts
-                (LookupMultiple.parse_obj([]), None): [],
-                (LookupMultiple.parse_obj([]), None): [],
-            },
-        )
-
         self.configure_task(override={"input": {"binary_artifacts": [[], []]}})
+        self.task_db.multiple_lookups = {
+            # binary artifacts
+            (LookupMultiple.parse_obj([]), None): [],
+            (LookupMultiple.parse_obj([]), None): [],
+        }
 
         with self.assertRaisesRegex(
             TaskConfigError,
@@ -551,7 +419,7 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
                 "cannot have zero artifacts (got 0 and 0)"
             ),
         ):
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
 
     def test_get_input_artifacts_ids_dynamic_dynamic_data_none(self) -> None:
         self.task.dynamic_data = None
@@ -877,7 +745,3 @@ class DebDiffTests(ExternalTaskHelperMixin[DebDiff], TestCase):
         debusine_mock.relation_create.assert_has_calls(
             [call(2, 1, "relates-to"), call(2, 2, "relates-to")]
         )
-
-    def test_get_label(self) -> None:
-        """Test get_label."""
-        self.assertEqual(self.task.get_label(), "debdiff")
