@@ -93,6 +93,11 @@ class ExtractForSigningTests(
         """Perform further setup."""
         super().configure_task(task_data, override, remove)
 
+        # self.task.workspace_name and self.task.work_request_id are set by
+        # the Worker.
+        self.task.work_request_id = 147
+        self.task.workspace_name = "testing"
+
         self.task_db = FakeTaskDatabase(
             single_lookups={
                 # environment
@@ -342,7 +347,6 @@ class ExtractForSigningTests(
 
     def test_fetch_input_template_wrong_category(self) -> None:
         """fetch_input checks the category of the template artifact."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = ExtractForSigningDynamicData(
             environment_id=1,
             input_template_artifact_id=2,
@@ -368,7 +372,6 @@ class ExtractForSigningTests(
 
     def test_fetch_input_template_udeb(self) -> None:
         """fetch_input fails if the template does not have a .deb."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = ExtractForSigningDynamicData(
             environment_id=1,
             input_template_artifact_id=2,
@@ -412,7 +415,6 @@ class ExtractForSigningTests(
 
     def test_fetch_input_gathers_information(self) -> None:
         """fetch_input gathers information from artifacts."""
-        self.task.work_request_id = 147
         self.task.dynamic_data = ExtractForSigningDynamicData(
             environment_id=1,
             input_template_artifact_id=2,
@@ -750,7 +752,6 @@ class ExtractForSigningTests(
 
     def test_upload_artifacts_execution_failed(self) -> None:
         """upload_artifacts does not upload anything if execution failed."""
-        self.task.work_request_id = 147
         debusine_mock = self.mock_debusine()
         execute_directory = self.create_temporary_directory()
 
@@ -764,8 +765,6 @@ class ExtractForSigningTests(
         directory = self.create_temporary_directory()
         (directory / "hello/boot").mkdir(parents=True)
         (directory / "hello/boot/vmlinuz").write_bytes(b"vmlinuz")
-        self.task.work_request_id = 147
-        self.task.workspace_name = "testing"
         self.task.dynamic_data = ExtractForSigningDynamicData(
             environment_id=1,
             input_template_artifact_id=2,
@@ -810,8 +809,6 @@ class ExtractForSigningTests(
 
     def test_execute_end_to_end(self) -> None:
         """End-to-end execution works."""
-        self.task.work_request_id = 147
-        self.task.workspace_name = "testing"
         self.task.dynamic_data = ExtractForSigningDynamicData(
             environment_id=1,
             input_template_artifact_id=2,
@@ -881,7 +878,10 @@ class ExtractForSigningTests(
         output_path = self.create_temporary_directory()
 
         def download_artifact(
-            artifact_id: int, destination: Path, tarball: bool = False
+            artifact_id: int,
+            destination: Path,
+            tarball: bool = False,
+            workspace: str | None = None,  # noqa: U100
         ) -> ArtifactResponse:
             assert not tarball
             for file_name in artifacts[artifact_id].files:
