@@ -1492,6 +1492,7 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
 
         self.configure_task(override={"input": input_override})
         self.task.work_request_id = 147
+        self.task.workspace_name = "testing"
         self.task.dynamic_data = dynamic_data
         debusine_mock = self.mock_debusine()
         destination = self.create_temporary_directory()
@@ -1501,7 +1502,11 @@ class LintianTests(ExternalTaskHelperMixin[Lintian], TestCase):
         expected_artifact_id_to_filenames = {}
 
         for artifact_id in expected_artifact_ids:
-            expected_calls.append(call(artifact_id, destination, tarball=False))
+            expected_calls.append(
+                call(
+                    artifact_id, destination, tarball=False, workspace="testing"
+                )
+            )
 
             file_in_artifact = self.create_temporary_file()
             file_model = FileResponse(
