@@ -8,14 +8,14 @@
 # contained in the LICENSE file.
 
 """Unit tests for the piuparts task support on the worker."""
-import datetime
-import os
+import datetime as dt
 import shlex
 import tarfile
 import textwrap
 from collections.abc import Iterable
 from io import BytesIO
 from pathlib import Path
+from typing import Any
 from unittest import mock
 from unittest.mock import MagicMock, call
 
@@ -48,7 +48,7 @@ from debusine.tasks.models import (
     LookupMultiple,
     PiupartsDynamicData,
 )
-from debusine.tasks.server import ArtifactInfo
+from debusine.tasks.server import ArtifactInfo, CollectionInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
@@ -66,7 +66,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     SAMPLE_TASK_DATA = {
         "input": {"binary_artifacts": [421]},
-        "host_architecture": "amd64",
+        "build_architecture": "amd64",
         "environment": "debian/match:codename=bookworm",
         "base_tgz": 44,
     }
@@ -75,6 +75,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         type="dpkg",
         changes_fields={
             "Architecture": "amd64",
+            "Version": "1.0",
             "Files": [{"name": "hello_1.0_amd64.deb"}],
             "Source": "hello",
         },
@@ -82,10 +83,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def setUp(self) -> None:
         super().setUp()
-        self.task = Piuparts(self.SAMPLE_TASK_DATA)
-        self.task.dynamic_data = PiupartsDynamicData(
-            environment_id=1, input_binary_artifacts_ids=[421], base_tgz_id=44
-        )
+        self.configure_task()
 
     def tearDown(self) -> None:
         """Delete directory to avoid ResourceWarning with python -m unittest."""
@@ -93,6 +91,46 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             self.task._debug_log_files_directory.cleanup()
         super().tearDown()
 
+    def configure_task(
+        self,
+        task_data: dict[str, Any] | None = None,
+        override: dict[str, Any] | None = None,
+        remove: list[str] | None = None,
+    ) -> None:
+        """Perform further setup."""
+        super().configure_task(task_data, override, remove)
+
+        self.task_db = FakeTaskDatabase(
+            single_lookups={
+                # environment
+                (
+                    "debian/match:codename=bookworm:architecture=amd64:"
+                    "format=tarball:backend=unshare:variant=piuparts",
+                    CollectionCategory.ENVIRONMENTS,
+                ): ArtifactInfo(
+                    id=1,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+                # base_tgz
+                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
+                    id=44,
+                    category=ArtifactCategory.SYSTEM_TARBALL,
+                    data=create_system_tarball_data(),
+                ),
+            },
+            multiple_lookups={
+                # input.binary_artifacts
+                (LookupMultiple.parse_obj([421]), None): [
+                    ArtifactInfo(
+                        id=421,
+                        category=ArtifactCategory.UPLOAD,
+                        data=self._binary_upload_data,
+                    )
+                ]
+            },
+        )
+
     def test_configure_fails_with_missing_required_data(
         self,
     ) -> None:
@@ -109,52 +147,21 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         with self.assertRaises(TaskConfigError):
             self.configure_task(remove=["base_tgz"])
 
-    def test_configure_host_architecture_is_required(self) -> None:
-        """Missing required field "host_architecture": raise exception."""
+    def test_configure_build_architecture_is_required(self) -> None:
+        """Missing required field "build_architecture": raise exception."""
         with self.assertRaises(TaskConfigError):
-            self.configure_task(remove=["host_architecture"])
+            self.configure_task(remove=["build_architecture"])
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=piuparts",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # base_tgz
-                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
-                    id=44,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self._binary_upload_data,
-                    )
-                ]
-            },
-        )
-
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             PiupartsDynamicData(
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
                 base_tgz_id=44,
                 subject="hello",
+                parameter_summary="hello_1.0 (amd64)",
                 runtime_context="sid:amd64",
                 configuration_context="sid",
             ),
@@ -162,60 +169,100 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def test_compute_dynamic_data_base_tgz_constraints(self) -> None:
         """Dynamic data applies environment-like constraints to base_tgz."""
-        self.task = Piuparts(
-            {
-                **self.SAMPLE_TASK_DATA,
-                "base_tgz": "debian/match:codename=bookworm",
-            }
-        )
-        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=piuparts",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # base_tgz
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=44,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(codename="bookworm"),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.UPLOAD,
-                        data=self._binary_upload_data,
-                    )
-                ]
-            },
+        self.configure_task(
+            override={"base_tgz": "debian/match:codename=bookworm"}
         )
+        self.task_db.single_lookups = {
+            # environment
+            (
+                "debian/match:codename=bookworm:architecture=amd64:"
+                "format=tarball:backend=unshare:variant=piuparts",
+                CollectionCategory.ENVIRONMENTS,
+            ): ArtifactInfo(
+                id=1,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            # base_tgz
+            (
+                "debian/match:codename=bookworm:architecture=amd64:"
+                "format=tarball",
+                CollectionCategory.ENVIRONMENTS,
+            ): ArtifactInfo(
+                id=44,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(codename="bookworm"),
+            ),
+        }
 
         self.assertEqual(
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
             PiupartsDynamicData(
                 environment_id=1,
                 input_binary_artifacts_ids=[421],
                 base_tgz_id=44,
                 subject="hello",
+                parameter_summary="hello_1.0 (amd64)",
                 runtime_context="bookworm:amd64",
                 configuration_context="bookworm",
             ),
         )
 
+    def test_compute_dynamic_data_multiple_source_packages(self) -> None:
+        self.configure_task(
+            override={"input": {"binary_artifacts": [421, 422]}}
+        )
+
+        self.task_db.single_lookups = {
+            # environment
+            (
+                44,
+                CollectionCategory.ENVIRONMENTS,
+            ): ArtifactInfo(
+                id=44,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(),
+            ),
+            # base_tgz
+            (
+                "debian/match:codename=bookworm:architecture=amd64:"
+                "format=tarball:backend=unshare:variant=",
+                CollectionCategory.ENVIRONMENTS,
+            ): ArtifactInfo(
+                id=45,
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data=create_system_tarball_data(codename="bookworm"),
+            ),
+        }
+        self.task_db.multiple_lookups = {
+            # input.binary_artifacts
+            (LookupMultiple.parse_obj([421, 422]), None): [
+                ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.UPLOAD,
+                    data=self._binary_upload_data,
+                ),
+                ArtifactInfo(
+                    id=422,
+                    category=ArtifactCategory.UPLOAD,
+                    data=DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "amd64",
+                            "Version": "2.0",
+                            "Files": [{"name": "bye_1.0-1_amd64.deb"}],
+                            "Source": "bye",
+                        },
+                    ),
+                ),
+            ]
+        }
+
+        dynamic_data = self.task.compute_dynamic_data(self.task_db)
+
+        self.assertEqual(dynamic_data.subject, "bye hello")
+        self.assertEqual(dynamic_data.parameter_summary, "bye hello (amd64)")
+
     def test_compute_dynamic_raise_config_task_error_wrong_environment(
         self,
     ) -> None:
@@ -224,36 +271,12 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
         environment artifact category is unexpected.
         """
-        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=piuparts",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # base_tgz
-                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
-                    id=44,
-                    category=ArtifactCategory.SOURCE_PACKAGE,
-                    data=EmptyArtifactData(),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.UPLOAD,
-                        data=EmptyArtifactData(),
-                    )
-                ]
-            },
+        self.task_db.single_lookups[(44, CollectionCategory.ENVIRONMENTS)] = (
+            ArtifactInfo(
+                id=44,
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data=EmptyArtifactData(),
+            )
         )
 
         with self.assertRaisesRegex(
@@ -261,7 +284,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             "^base_tgz: unexpected artifact category: 'debian:source-package'. "
             r"Valid categories: \['debian:system-tarball'\]$",
         ):
-            self.task.compute_dynamic_data(task_db),
+            self.task.compute_dynamic_data(self.task_db),
 
     def test_compute_dynamic_raise_config_task_error_wrong_binary_artifact(
         self,
@@ -271,64 +294,17 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
         binary_artifact artifact category is unexpected.
         """
-        binary_artifacts_lookup = LookupMultiple.parse_obj([421])
-        task_db = FakeTaskDatabase(
-            single_lookups={
-                # environment
-                (
-                    "debian/match:codename=bookworm:architecture=amd64:"
-                    "format=tarball:backend=unshare:variant=piuparts",
-                    CollectionCategory.ENVIRONMENTS,
-                ): ArtifactInfo(
-                    id=1,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-                # base_tgz
-                (44, CollectionCategory.ENVIRONMENTS): ArtifactInfo(
-                    id=44,
-                    category=ArtifactCategory.SYSTEM_TARBALL,
-                    data=create_system_tarball_data(),
-                ),
-            },
-            multiple_lookups={
-                # input.binary_artifacts
-                (binary_artifacts_lookup, None): [
-                    ArtifactInfo(
-                        id=421,
-                        category=ArtifactCategory.SYSTEM_TARBALL,
-                        data=EmptyArtifactData(),
-                    )
-                ]
-            },
-        )
+        self.task_db.multiple_lookups[(LookupMultiple.parse_obj([421]), None)][
+            0
+        ].category = ArtifactCategory.SYSTEM_TARBALL
 
         with self.assertRaisesRegex(
             TaskConfigError,
             r"^input.binary_artifacts\[0\]: unexpected artifact category: "
             r"'debian:system-tarball'. Valid categories: "
-            r"\['debian:binary-package', 'debian:binary-packages', "
-            r"'debian:upload'\]$",
+            r"\['debian:binary-package', 'debian:upload'\]$",
         ):
-            self.task.compute_dynamic_data(task_db)
-
-    def test_cmdline_without_usr_sbin(self) -> None:
-        """_cmdline adds `/usr/sbin` to `PATH` if it isn't there."""
-        with mock.patch.dict(os.environ, {"PATH": "/usr/bin:/bin"}):
-            self.assertEqual(
-                self.task._cmdline()[:3],
-                ["env", "PATH=/usr/sbin:/usr/bin:/bin", "piuparts"],
-            )
-
-    def test_cmdline_with_usr_sbin(self) -> None:
-        """_cmdline doesn't add `/usr/sbin` to `PATH` if it's already there."""
-        with mock.patch.dict(
-            os.environ, {"PATH": "/usr/sbin:/sbin:/usr/bin:/bin"}
-        ):
-            self.assertEqual(
-                self.task._cmdline()[:3],
-                ["env", "PATH=/usr/sbin:/sbin:/usr/bin:/bin", "piuparts"],
-            )
+            self.task.compute_dynamic_data(self.task_db)
 
     def test_get_input_artifacts_ids(self) -> None:
         """Test get_input_artifacts_ids."""
@@ -453,7 +429,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 id=artifact_id,
                 workspace="System",
                 category=ArtifactCategory.SYSTEM_TARBALL,
-                created_at=datetime.datetime.now(),
+                created_at=dt.datetime.now(),
                 data=data.dict(),
                 download_tar_gz_url=pydantic.parse_obj_as(
                     pydantic.AnyUrl,
@@ -593,6 +569,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         fetch_artifact_mocked = self.patch_fetch_artifact_for_basetgz(
             with_dev=False, codename=codename, filename=filename
         )
+        self.set_dynamic_data()
 
         # Use mock's side effect to set self.task.executor_instance
         self.patch_prepare_executor_instance()
@@ -617,6 +594,8 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def test_prepare_base_tgz_download_base_tgz_remove_dev_files(self) -> None:
         r"""_prepare_base_tgz download the artifact and remove /dev/\*."""
+        self.set_dynamic_data()
+
         # Use mock's side effect to set self.task.executor_instance
         self.patch_prepare_executor_instance()
         self.task._prepare_executor_instance()
@@ -647,6 +626,8 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def test_prepare_base_tgz_download_base_tgz_resolveconf(self) -> None:
         r"""_prepare_base_tgz download the artifact and install resolv.conf."""
+        self.set_dynamic_data()
+
         # Use mock's side effect to set self.task.executor_instance
         self.patch_prepare_executor_instance()
         self.task._prepare_executor_instance()
@@ -684,6 +665,8 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
 
     def test_prepare_base_tgz_download_base_tgz_dpkg_available(self) -> None:
         r"""_prepare_base_tgz download the artifact and touch dpkg available."""
+        self.set_dynamic_data()
+
         # Use mock's side effect to set self.task.executor_instance
         self.patch_prepare_executor_instance()
         self.task._prepare_executor_instance()
@@ -763,6 +746,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 ]
             }
         )
+        self.set_dynamic_data()
         script = self.prepare_scripts("jessie")
         self.assertIn("mkdir -p /etc/apt/keyrings\n", script)
         key = shlex.quote("KEY A\n")
@@ -796,6 +780,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 ]
             }
         )
+        self.set_dynamic_data()
         script = self.prepare_scripts("buster")
         self.assertIn("mkdir -p /etc/apt/keyrings\n", script)
         key = shlex.quote("KEY A\n")
@@ -836,6 +821,7 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
                 ]
             }
         )
+        self.set_dynamic_data()
         script = self.prepare_scripts("bookworm")
         self.assertNotIn("/etc/apt/keyrings", script)
         source = shlex.quote(
@@ -858,6 +844,60 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             script,
         )
 
+    def test_prepare_scripts_extra_debusine_repositories(self) -> None:
+        """_prepare_scripts with extra repositories hosted by Debusine."""
+        self.configure_task(
+            override={
+                "extra_repositories": [{"suite": "trixie-local@debian:suite"}]
+            }
+        )
+        self.task_db.single_collection_lookups[
+            ("trixie-local@debian:suite", None)
+        ] = CollectionInfo(
+            id=1,
+            scope_name="scope",
+            workspace_name="workspace",
+            category=CollectionCategory.SUITE,
+            name="trixie-local",
+            data={"components": ["main"]},
+        )
+        self.task_db.settings["DEBUSINE_DEBIAN_ARCHIVE_PRIMARY_FQDN"] = (
+            "deb.example.com"
+        )
+        self.task_db.suite_signing_keys[1] = "\n".join(
+            (
+                "-----BEGIN PGP PUBLIC KEY BLOCK-----",
+                "",
+                "ABCDEFGHI",
+                "-----END PGP PUBLIC KEY BLOCK-----",
+            )
+        )
+        self.set_dynamic_data()
+        script = self.prepare_scripts("trixie")
+        self.assertNotIn("/etc/apt/keyrings", script)
+        source = shlex.quote(
+            textwrap.dedent(
+                """\
+                Types: deb
+                URIs: http://deb.example.com/scope/workspace
+                Suites: trixie-local
+                Components: main
+                Signed-By:
+                 -----BEGIN PGP PUBLIC KEY BLOCK-----
+                 .
+                 ABCDEFGHI
+                 -----END PGP PUBLIC KEY BLOCK-----
+                """
+            )
+        )
+        self.assertIn(
+            (
+                f"printf %s {source} > "
+                f"/etc/apt/sources.list.d/extra_repository_0.sources\n"
+            ),
+            script,
+        )
+
     def test_execution_consistency_no_errors(self) -> None:
         """There are no consistency errors."""
         build_directory = self.create_temporary_directory()
@@ -975,23 +1015,20 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
             self.task._deb_versions,
             {"file1": "1.0", "file2": "1.0", "makedev": "2.3.1-97"},
         )
-        with mock.patch.dict(os.environ, {"PATH": "/usr/bin:/bin"}):
-            self.assertEqual(
-                self.task._cmdline(),
-                [
-                    "env",
-                    "PATH=/usr/sbin:/usr/bin:/bin",
-                    "piuparts",
-                    "--keep-sources-list",
-                    "--allow-database",
-                    "--warn-on-leftovers-after-purge",
-                    f"--basetgz={self.task._base_tar}",
-                    f"--scriptsdir={self.task._scripts_dir}",
-                    str(file1),
-                    str(file2),
-                    str(file3),
-                ],
-            )
+        self.assertEqual(
+            self.task._cmdline(),
+            [
+                "piuparts",
+                "--keep-sources-list",
+                "--allow-database",
+                "--warn-on-leftovers-after-purge",
+                f"--basetgz={self.task._base_tar}",
+                f"--scriptsdir={self.task._scripts_dir}",
+                str(file1),
+                str(file2),
+                str(file3),
+            ],
+        )
 
         (download_directory / self.task.CAPTURE_OUTPUT_FILENAME).write_text(
             "piuparts output\n"
@@ -1001,24 +1038,3 @@ class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
         self.task.executor.system_image = self.fake_system_tarball_artifact()
 
         self.task.upload_artifacts(download_directory, execution_success=True)
-
-    def test_get_label_dynamic_data_is_none(self) -> None:
-        """Test get_label if dynamic_data.subject is None."""
-        self.assertEqual(self.task.get_label(), "piuparts")
-
-    def test_get_label_dynamic_data_subject_is_none(self) -> None:
-        """Test get_label if dynamic_data is None."""
-        self.task.dynamic_data = PiupartsDynamicData(
-            environment_id=1, input_binary_artifacts_ids=[], base_tgz_id=2
-        )
-        self.assertEqual(self.task.get_label(), "piuparts")
-
-    def test_get_label_dynamic_data_subject_is_hello(self) -> None:
-        """Test get_label if dynamic_data.subject is set."""
-        self.task.dynamic_data = PiupartsDynamicData(
-            environment_id=1,
-            input_binary_artifacts_ids=[],
-            base_tgz_id=2,
-            subject="hello",
-        )
-        self.assertEqual(self.task.get_label(), "piuparts hello")
