@@ -92,6 +92,10 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
 
         super().configure_task(task_data, override, remove)
 
+        # Set by the worker:
+        self.task.work_request_id = 5
+        self.task.workspace_name = "testing"
+
         self.task_db = FakeTaskDatabase(
             single_lookups={
                 # environment
@@ -253,7 +257,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         directory = self.create_temporary_directory()
         source_artifact = self.fake_debian_source_package_artifact()
         self.configure_task(override={"environment": 42})
-        self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             environment_id=42, input_source_artifact_id=source_artifact.id
         )
@@ -295,7 +298,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 },
             }
         )
-        self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             input_source_artifact_id=source_artifact.id
         )
@@ -343,7 +345,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 },
             }
         )
-        self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             input_source_artifact_id=source_artifact.id
         )
@@ -383,7 +384,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 },
             }
         )
-        self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             input_source_artifact_id=source_artifact.id,
             input_extra_binary_artifacts_ids=[
@@ -393,7 +393,9 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         )
         debusine_mock = self.mock_debusine()
 
-        def artifact_get(artifact_id: int) -> ArtifactResponse:
+        def artifact_get(
+            artifact_id: int, workspace: str  # noqa: U100
+        ) -> ArtifactResponse:
             """Fake for debusine client artifact_get."""
             if artifact_id == source_artifact.id:
                 return source_artifact
@@ -452,7 +454,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                 },
             }
         )
-        self.task.work_request_id = 5
         self.task.dynamic_data = SbuildDynamicData(
             input_source_artifact_id=source_artifact.id,
             input_extra_binary_artifacts_ids=[source_artifact.id],
@@ -960,7 +961,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
             task_data=self.configuration("amd64", ["any"]),
             override={"backend": BackendType.UNSHARE, "environment": 42},
         )
-        self.task.work_request_id = 5
         self.patch_sbuild_debusine()
 
         self.run_execute(["true"])
@@ -1463,19 +1463,19 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
                         architecture=self.task._host_architecture,
                     ),
                     workspace=self.task.workspace_name,
-                    work_request=None,
+                    work_request=self.task.work_request_id,
                 ),
                 call(
                     Upload.create(
                         changes_file=changes_file, exclude_files=set()
                     ),
                     workspace=self.task.workspace_name,
-                    work_request=None,
+                    work_request=self.task.work_request_id,
                 ),
                 call(
                     SigningInputArtifact.create([changes_file], temp_directory),
                     workspace=self.task.workspace_name,
-                    work_request=None,
+                    work_request=self.task.work_request_id,
                 ),
             ]
         )
@@ -1491,8 +1491,6 @@ class SbuildTests(ExternalTaskHelperMixin[Sbuild], TestCase):
         dsc = self.write_dsc_example_file(self.task._dsc_file)
         (log_file := temp_directory / "log.build").write_text("log file")
 
-        self.task.work_request_id = 5
-
         self.task.upload_artifacts(temp_directory, execution_success=False)
 
         # Only one artifact is uploaded: PackageBuildLog
@@ -1916,17 +1914,17 @@ broken-packages: 4
                     architecture=architecture,
                 ),
                 workspace=self.task.workspace_name,
-                work_request=None,
+                work_request=self.task.work_request_id,
             ),
             call(
                 Upload.create(changes_file=changes_file, exclude_files=set()),
                 workspace=self.task.workspace_name,
-                work_request=None,
+                work_request=self.task.work_request_id,
             ),
             call(
                 SigningInputArtifact.create([changes_file], build_directory),
                 workspace=self.task.workspace_name,
-                work_request=None,
+                work_request=self.task.work_request_id,
             ),
         ]
 
@@ -2090,7 +2088,6 @@ broken-packages: 4
         self.write_dsc_example_file(self.task._dsc_file)
         self.write_deb_file(build_directory / "hello_1.0_amd64.deb")
         debusine_mock = self.patch_sbuild_debusine()
-        self.task.work_request_id = 8
 
         self.task.upload_artifacts(build_directory, execution_success=True)
 
@@ -2135,8 +2132,6 @@ broken-packages: 4
         changes_file = temp_directory / "python-network.changes"
         self.write_changes_file(changes_file, [file1, file2])
 
-        self.task.work_request_id = 8
-
         self.task.upload_artifacts(temp_directory, execution_success=True)
 
         workspace_name = self.task.workspace_name
