@@ -34,8 +34,6 @@ import lxml.etree
 import lxml.objectify
 import responses
 import yaml
-from prometheus_client import Counter, Gauge, Metric
-from prometheus_client.metrics import MetricWrapperBase
 
 from debusine.artifacts.local_artifact import deb822dict_to_dict
 from debusine.artifacts.models import DebianSourcePackage
@@ -711,41 +709,3 @@ class TestCase(unittest.TestCase):
         except yaml.parser.ParserError as e:
             self.fail(f"node text {encoded!r} does not parse as YAML: {e}")
         self.assertEqual(actual, data)
-
-
-class MetricsTestMixin:
-    """Methods used by metrics tests."""
-
-    def reset_counter(self, counter: Counter) -> None:
-        """
-        Reset all label sets in a counter.
-
-        This is useful to clear out leftovers from previous tests.
-        """
-        [collected] = counter.collect()
-        for sample in collected.samples:
-            counter.labels(**sample.labels).reset()
-
-    def reset_gauge(self, gauge: Gauge) -> None:
-        """
-        Reset all label sets in a gauge.
-
-        This is useful to clear out leftovers from previous tests.
-        """
-        [collected] = gauge.collect()
-        for sample in collected.samples:
-            gauge.labels(**sample.labels).set(0)
-
-    def sample_by_label(self, metric: Metric, labels: dict[str, str]) -> float:
-        """Find the sample for labels in metric."""
-        for sample in metric.samples:
-            if sample.labels == labels:
-                return sample.value
-        raise AssertionError(f"{labels} not found in {metric}")
-
-    def assert_metric_equals(
-        self, metric: MetricWrapperBase, value: float, **labels: str
-    ) -> None:
-        """Assert that there is a sample for labels in metric with value."""
-        [collected] = metric.collect()
-        assert self.sample_by_label(collected, labels) == value
