@@ -152,16 +152,14 @@ class PermissionOverrideTests(TestCase):
                 ordered=False,
             )
             self.assertQuerySetEqual(
-                Scope.objects.all().can_display(self.user),
+                Scope.objects.can_display(self.user),
                 [self.scope, self.scope1],
                 ordered=False,
             )
         with override_permission(Scope, "can_display", DenyAll):
             self.assertFalse(self.scope.can_display(self.user))
             self.assertQuerySetEqual(Scope.objects.can_display(self.user), [])
-            self.assertQuerySetEqual(
-                Scope.objects.all().can_display(self.user), []
-            )
+            self.assertQuerySetEqual(Scope.objects.can_display(self.user), [])
 
     @override_permission(Scope, "can_display", DenyAll)
     def test_override_permission_decorator(self) -> None:
@@ -243,9 +241,9 @@ class TestCaseTests(TestCase):
             paths, files_size, create_files=True
         )
 
-        self.assertEqual(artifact.fileinartifact_set.all().count(), len(paths))
+        self.assertEqual(artifact.fileinartifact_set.count(), len(paths))
 
-        files_in_artifact = artifact.fileinartifact_set.all().order_by("path")
+        files_in_artifact = artifact.fileinartifact_set.order_by("path")
 
         for file_in_artifact in files_in_artifact:
             file = file_in_artifact.file
@@ -262,7 +260,7 @@ class TestCaseTests(TestCase):
             ["README"], create_files=False
         )
 
-        self.assertEqual(artifact.fileinartifact_set.all().count(), 0)
+        self.assertEqual(artifact.fileinartifact_set.count(), 0)
 
     def test_create_artifact_raise_value_error(self) -> None:
         """create_artifact() raise ValueError: incompatible options."""
