@@ -827,6 +827,27 @@ class DistsByHashFileViewTests(ArchiveFileViewTests):
             response, contents=b"Sources file", mutable=False
         )
 
+    def test_by_hash_matches_multiple_artifacts(self) -> None:
+        contents = b"Sources file"
+        sha256 = hashlib.sha256(contents).hexdigest()
+        for _ in range(2):
+            self.suites["bookworm"].manager.add_artifact(
+                self.playground.create_repository_index(
+                    "Sources.xz", contents=contents
+                ),
+                replace=True,
+                user=self.scenario.user,
+                variables={"path": "main/source/Sources.xz"},
+            )
+
+        response = self.archive_file_get(
+            path=f"dists/bookworm/main/source/by-hash/SHA256/{sha256}",
+        )
+
+        self.assert_response_ok(
+            response, contents=b"Sources file", mutable=False
+        )
+
 
 class DistsFileViewTests(ArchiveFileViewTests):
     """Test retrieving non-``by-hash`` files from ``dists/``."""
