@@ -523,7 +523,7 @@ class FormatYamlTests(TestCase):
         tree = self.assertHTMLValid(format_yaml(42))
         div = self.assertHasElement(tree, "body/div")
         self.assertTextContentEqual(div, "42\n...")
-        self.assertEqual(div.get("class"), "file_highlighted")
+        self.assertEqual(div.get("class"), "file-contents")
 
     def test_format_sorted(self) -> None:
         tree = self.assertHTMLValid(format_yaml({"b": 2, "a": 1}))
