@@ -85,19 +85,19 @@ class WorkflowRuntimeStatusSmallTests(TestCase):
 
         children_needs_input = self.workflow.create_child_wait(
             "waitnoop",
-            status=WorkRequest.Statuses.RUNNING,
             workflow_data=WorkRequestWorkflowData(needs_input=True),
         )
+        self.playground.advance_work_request(
+            children_needs_input, mark_running=True
+        )
 
-        self.workflow.create_child_wait(
+        wr = self.workflow.create_child_wait(
             "waitnoop",
-            status=WorkRequest.Statuses.RUNNING,
             workflow_data=WorkRequestWorkflowData(needs_input=True),
         )
+        self.playground.advance_work_request(wr, mark_running=True)
 
-        self.workflow.create_child_worker(
-            "noop", status=WorkRequest.Statuses.ABORTED
-        )
+        self.workflow.create_child_worker("noop").mark_aborted()
 
         actual = workflow_runtime_status_small(self.workflow)
 
