@@ -431,7 +431,7 @@ class TextFileWidget(FileWidget):
                     lexer = pygments.lexers.get_lexer_for_mimetype("text/plain")
 
             formatter = LinenoHtmlFormatter(
-                cssclass="file_highlighted",
+                cssclass="file-contents",
                 linenos=False,
                 suffix=self.suffix(),
             )
