@@ -1591,7 +1591,7 @@ class CreateArtifactViewTests(TestCase):
         self.assertEqual(artifact.fileinartifact_set.count(), len(files))
 
         for file_in_artifact, file_to_upload in zip(
-            artifact.fileinartifact_set.all().order_by("id"), files
+            artifact.fileinartifact_set.order_by("id"), files
         ):
             file_backend = workspace.scope.download_file_backend(
                 file_in_artifact.file
