@@ -8,7 +8,7 @@
 # contained in the LICENSE file.
 
 """Tests for the auth views."""
-from datetime import timedelta
+import datetime as dt
 from textwrap import dedent
 from typing import Any, ClassVar
 from urllib.parse import urlencode
@@ -17,7 +17,7 @@ import lxml
 from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.template.response import SimpleTemplateResponse
-from django.test import override_settings
+from django.test import RequestFactory, override_settings
 from django.urls import reverse
 from django.utils import timezone
 from rest_framework import status
@@ -112,17 +112,29 @@ class UserDetailViewTests(ViewTestMixin, TestCase):
 
     scenario = scenarios.DefaultScopeUser()
 
-    def test_get_title_full_name(self) -> None:
-        """Test get_title() returns full name."""
+    def test_get_place_full_name(self) -> None:
         view = UserDetailView()
         view.object = self.scenario.user
-        self.assertEqual(view.get_title(), self.scenario.user.get_full_name())
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, self.scenario.user.get_full_name())
+        self.assertEqual(place.url, self.scenario.user.get_absolute_url())
+        self.assertIsNone(place.icon)
+        self.assertEqual(place.description, "User Play Ground <playground>")
+        self.assertIsNone(place.parent)
 
-    def test_get_title_username(self) -> None:
-        """Test get_title() returns username (full_name is not set)"""
+    def test_get_place_username(self) -> None:
+        # Full name is not set
+        user = self.playground.create_user(username="John")
         view = UserDetailView()
-        view.object = self.playground.create_user(username="John")
-        self.assertEqual(view.get_title(), "John")
+        view.object = user
+        view.request = RequestFactory().get("/")
+        place = view.get_place()
+        self.assertEqual(place.title, user.username)
+        self.assertEqual(place.url, user.get_absolute_url())
+        self.assertIsNone(place.icon)
+        self.assertEqual(place.description, "User <John>")
+        self.assertIsNone(place.parent)
 
     def test_anonymous(self) -> None:
         """Test accessing as anonymous user."""
@@ -411,7 +423,7 @@ class UserTokenListViewTests(ViewTestMixin, TestCase):
         )
 
         # Make token1 to be created_at after token2 created_at
-        token1.created_at = token2.created_at + timedelta(minutes=1)
+        token1.created_at = token2.created_at + dt.timedelta(minutes=1)
         token1.save()
 
         response = self.client.get(
@@ -506,11 +518,11 @@ class UserTokenCreateViewTests(ViewTestMixin, TestCase):
                 kwargs={"username": self.user.username},
             )
         )
-
-        self.assertContains(
-            response, "<title>Debusine - Create token</title>", html=True
-        )
-        self.assertContains(response, "<h1>Create token</h1>", html=True)
+        tree = self.assertResponseHTML(response)
+        title = self.assertHasElement(tree, "//title")
+        self.assertTextContentEqual(title, "debusine - testuser - Create token")
+        h1 = self.assertHasElement(tree, "//h1")
+        self.assertTextContentEqual(h1, "Create token")
         self.assertContains(
             response,
             '<input class="btn btn-primary btn-sm" '
@@ -629,11 +641,12 @@ class UserTokenUpdateViewTests(TestCase):
                 },
             )
         )
+        tree = self.assertResponseHTML(response)
+        title = self.assertHasElement(tree, "//title")
+        self.assertTextContentEqual(title, "debusine - playground - Edit token")
+        h1 = self.assertHasElement(tree, "//h1")
+        self.assertTextContentEqual(h1, "Edit token")
 
-        self.assertContains(
-            response, "<title>Debusine - Edit token</title>", html=True
-        )
-        self.assertContains(response, "<h1>Edit token</h1>", html=True)
         self.assertContains(
             response,
             '<input class="btn btn-primary btn-sm" type="submit" value="Edit">',
