@@ -31,7 +31,7 @@ from debusine.test.django import (
     override_permission,
 )
 from debusine.web.views.base import BaseUIView
-from debusine.web.views.places import Place
+from debusine.web.views.places import IndexPlace
 from debusine.web.views.tests.utils import ViewTestMixin
 
 
@@ -132,7 +132,9 @@ class TestBaseUI(ViewTestMixin, TestCase):
         view.request = RequestFactory().get("/")
         ctx = view.get_context_data()
         self.assertEqual(ctx["base_template"], "web/_base.html")
-        self.assertEqual(ctx["place"], Place(title=""))
+        place = ctx["place"]
+        self.assertIsInstance(place, IndexPlace)
+        self.assertEqual(place.url, "/")
         self.assertNotIn("workspace_collections", ctx)
 
     def test_get_context_data_with_workspace(self) -> None:
@@ -147,7 +149,9 @@ class TestBaseUI(ViewTestMixin, TestCase):
         view.request = RequestFactory().get("/")
         ctx = view.get_context_data()
         self.assertEqual(ctx["base_template"], "web/_base.html")
-        self.assertEqual(ctx["place"], Place(title=""))
+        place = ctx["place"]
+        self.assertIsInstance(place, IndexPlace)
+        self.assertEqual(place.url, "/")
         self.assertQuerySetEqual(
             ctx["workspace_collections"],
             list(
