@@ -28,7 +28,8 @@ from debusine.tasks.models import (
     LookupMultiple,
 )
 from debusine.test.django import TestCase
-from debusine.web.views.lintian import LintianViewWorkRequestPlugin
+from debusine.web.views.lintian import LintianTab, LintianViewWorkRequestPlugin
+from debusine.web.views.work_request import Tab
 
 
 class LintianViewTestsWorkRequstPluginTests(TestCase):
@@ -214,13 +215,28 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         ).encode("utf-8")
         return self.create_temporary_file(contents=contents)
 
+    def get_tab(
+        self, source: WorkRequest | LintianViewWorkRequestPlugin
+    ) -> LintianTab:
+        """Return the tab for a work request or a plugin."""
+        if isinstance(source, WorkRequest):
+            source = LintianViewWorkRequestPlugin(source)
+        tabs: list[Tab] = []
+        source.add_tabs(tabs)
+        self.assertEqual(len(tabs), 1)
+        tab = tabs[0]
+        assert isinstance(tab, LintianTab)
+        self.assertEqual(tab.name, "lintian")
+        self.assertEqual(tab.label, "Lintian")
+        return tab
+
     def test_get_context_data(self) -> None:
         """Test get_context_data() return the expected context data."""
         lintian_txt_path = (
             self.binary_artifact.get_absolute_url_download() + "lintian.txt"
         )
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
+        plugin = LintianViewWorkRequestPlugin(self.work_request)
+        context_data = self.get_tab(plugin).get_context_data()
 
         # Verify some keys in separate methods
         context_data.pop("tags")
@@ -233,13 +249,14 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
                 "result": self.work_request.result,
                 "lintian_txt_path": lintian_txt_path,
                 "fail_on_severity": self.fail_on_severity,
+            },
+        )
+
+        self.assertEqual(
+            plugin.get_context_data(),
+            {
                 "description_data": {},
                 "description_template": "web/_lintian-description.html",
-                "specialized_tab": {
-                    "label": "Lintian",
-                    "slug": "lintian",
-                    "template": "web/_lintian-work_request-detail.html",
-                },
             },
         )
 
@@ -273,9 +290,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         }
         self.source_artifact.save()
 
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
-
+        context_data = self.get_tab(self.work_request).get_context_data()
         self.assertEqual(
             context_data["tags_count_severity"],
             {
@@ -297,8 +312,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
             self.create_analysis().read_bytes(),
         )
 
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
+        context_data = self.get_tab(self.work_request).get_context_data()
 
         expected = {
             "hello_2.10-3.dsc": {
@@ -356,8 +370,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         ).dict(exclude_unset=True)
         self.work_request.save()
 
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
+        context_data = self.get_tab(self.work_request).get_context_data()
 
         expected = {
             "source_artifact": {
@@ -395,9 +408,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         """If there is no dynamic data, context data omits artifact info."""
         task_data_input = self.work_request.task_data["input"]
 
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
-
+        context_data = self.get_tab(self.work_request).get_context_data()
         self.assertEqual(
             context_data["request_data"]["source_artifact"],
             {
@@ -423,9 +434,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         source_artifact = Artifact.objects.get(id=source_artifact_id)
         source_artifact.files.clear()
         source_artifact.delete()
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
-
+        context_data = self.get_tab(self.work_request).get_context_data()
         self.assertEqual(
             context_data["request_data"]["source_artifact"],
             {"lookup": source_artifact_id, "id": None, "files": []},
@@ -443,9 +452,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
         binary_artifact_0 = Artifact.objects.get(id=binary_artifact_0_id)
         binary_artifact_0.files.clear()
         binary_artifact_0.delete()
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
-
+        context_data = self.get_tab(self.work_request).get_context_data()
         self.assertEqual(
             context_data["request_data"]["binary_artifacts"],
             {
@@ -478,9 +485,7 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
             complete=False,
         )
 
-        lintian_view = LintianViewWorkRequestPlugin(self.work_request)
-        context_data = lintian_view.get_context_data()
-
+        context_data = self.get_tab(self.work_request).get_context_data()
         self.assertIsNone(context_data["lintian_txt_path"])
         self.assertEqual(context_data["tags"], {})
 
@@ -593,17 +598,13 @@ class LintianViewTestsWorkRequstPluginTests(TestCase):
 
     def test_find_lintian_txt_url_path_no_artifacts(self) -> None:
         """_find_lintian_txt_url_path() with no artifacts: return None."""
-        self.assertIsNone(
-            LintianViewWorkRequestPlugin._find_lintian_txt_url_path([])
-        )
+        self.assertIsNone(LintianTab._find_lintian_txt_url_path([]))
 
     def test_find_lintian_txt_url_path_artifact_no_files(self) -> None:
         """_find_lintian_txt_url_path(), artifact no files: return None."""
         artifact, _ = self.playground.create_artifact()
 
-        self.assertIsNone(
-            LintianViewWorkRequestPlugin._find_lintian_txt_url_path([artifact])
-        )
+        self.assertIsNone(LintianTab._find_lintian_txt_url_path([artifact]))
 
     def test_template_output_no_lintian_txt(self) -> None:
         """The view renders OK if no lintian.txt was output."""
