@@ -12,13 +12,14 @@ import abc
 import datetime as dt
 import functools
 import itertools
-from typing import Any, cast
+from typing import Any, cast, override
 
 from django.contrib import messages
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ValidationError
 from django.http import HttpRequest, HttpResponse
 from django.shortcuts import redirect, render
+from django.template.context import Context
 from django.template.defaultfilters import filesizeformat
 from django.utils.duration import duration_string
 from django.views.generic.base import View
@@ -51,6 +52,7 @@ from debusine.web.views.base import (
     FormMixinBase,
     ListViewBase,
     SingleObjectMixinBase,
+    Widget,
     WorkspaceView,
 )
 from debusine.web.views.http_errors import HttpError400, catch_http_errors
@@ -61,6 +63,33 @@ from debusine.web.views.tables import ArtifactTable, WorkRequestTable
 from debusine.web.views.view_utils import format_json, format_yaml
 
 
+class Tab(Widget):
+    """Define a tab."""
+
+    def __init__(
+        self, name: str, label: str, context: dict[str, Any] | None = None
+    ) -> None:
+        """Store name and label."""
+        self.name = name
+        self.label = label
+        # TODO: this is here only to ease transition from previous WorkRequest
+        # plugins. Eventually plugins will move their get_context_data to a Tab
+        # subclass
+        self.context = context or {}
+
+    def get_context_data(self) -> dict[str, Any]:
+        """Return extra context for this tab."""
+        return self.context
+
+    @override
+    def render(self, context: Context) -> str:
+        template_name = f"web/work_request-detail-{self.name}.html"
+        assert context.template is not None
+        template = context.template.engine.get_template(template_name)
+        with context.update(self.get_context_data()):
+            return template.render(context)
+
+
 class WorkRequestObjectMixin(WorkspaceView):
     """
     Add permission filtering to a Single/MultipleObjectMixin.
@@ -105,6 +134,11 @@ class WorkRequestDetailView(
         else:
             return None
 
+    @functools.cached_property
+    def plugin(self) -> "WorkRequestPlugin | None":
+        """Return the work request plugin, if defined."""
+        return WorkRequestPlugin.plugin_for(self.object)
+
     def get_main_ui_shortcuts(self) -> list[ui_shortcuts.UIShortcut]:
         """Return a list of UI shortcuts for this view."""
         actions = super().get_main_ui_shortcuts()
@@ -177,35 +211,47 @@ class WorkRequestDetailView(
 
         return Task.__doc__
 
+    def get_tabs(self) -> list[Tab]:
+        """Return the tabs to show in the view."""
+        tabs: list[Tab] = [
+            Tab(name="work_request", label="Work request"),
+        ]
+        if not self.object.is_workflow or self.object.output_data:
+            tabs += [
+                Tab(name="output", label="Output"),
+            ]
+        tabs += [
+            Tab(name="dependencies", label="Dependencies"),
+            Tab(name="internals", label="Internals"),
+        ]
+
+        # After the initial tab list is set, allow the plugin to add to it
+        if self.plugin is not None:
+            self.plugin.add_tabs(tabs)
+        return tabs
+
     def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
         """Return context for this view's tab."""
-        context = super().get_context_data(**kwargs)
+        ctx = super().get_context_data(**kwargs)
 
-        plugin_view = WorkRequestPlugin.plugin_for(self.object)
+        ctx["tabs"] = self.get_tabs()
 
-        context["description_template"] = (
+        ctx["description_template"] = (
             "web/_work_request-generic-description.html"
         )
-        context["description_data"] = {
+        ctx["description_data"] = {
             "default_description": self.get_default_description()
         }
 
-        if plugin_view is not None:
-            # Add specific specialized plugin information
-            context.update(
-                **plugin_view.get_context_data(),
-                **{"specialized_view": True},
-            )
-
-        context["manual_unblock_log_entries"] = self.get_manual_unblock_log()
-        context["manual_unblock_form"] = WorkRequestUnblockForm()
+        ctx["manual_unblock_log_entries"] = self.get_manual_unblock_log()
+        ctx["manual_unblock_form"] = WorkRequestUnblockForm()
 
         if self._validation_error_message is not None:
-            context["validation_error"] = self._validation_error_message
-        elif plugin_view:
+            ctx["validation_error"] = self._validation_error_message
+        elif self.plugin:
             # plugin_view for WorkRequest.task_name exists, but the
             # response will return the generic WorkRequest view
-            context["specialized_view_path"] = self.request.path
+            ctx["specialized_view_path"] = self.request.path
 
         try:
             task = self.object.get_task()
@@ -214,9 +260,9 @@ class WorkRequestDetailView(
 
         if self.object.requires_confirmation:
             assert isinstance(task, Confirm)
-            context["confirm_form"] = WorkRequestConfirmForm()
-            context["confirm_label"] = task.data.confirm_label
-            context["deny_label"] = task.data.deny_label
+            ctx["confirm_form"] = WorkRequestConfirmForm()
+            ctx["confirm_label"] = task.data.confirm_label
+            ctx["deny_label"] = task.data.deny_label
 
         input_artifacts: ArtifactQuerySet[Any] = Artifact.objects.none()
         if task and (input_artifact_ids := task.get_input_artifacts_ids()):
@@ -233,7 +279,7 @@ class WorkRequestDetailView(
         input_artifacts_table_paginator = input_artifacts_table.get_paginator(
             per_page=20
         )
-        context["input_artifacts_table"] = input_artifacts_table_paginator
+        ctx["input_artifacts_table"] = input_artifacts_table_paginator
 
         built_artifacts_table = ArtifactTable(
             self.request,
@@ -246,7 +292,7 @@ class WorkRequestDetailView(
         built_artifacts_table_paginator = built_artifacts_table.get_paginator(
             per_page=20
         )
-        context["built_artifacts_table"] = built_artifacts_table_paginator
+        ctx["built_artifacts_table"] = built_artifacts_table_paginator
 
         # Generate UI shortcuts for artifacts
         for artifact in itertools.chain(
@@ -259,39 +305,39 @@ class WorkRequestDetailView(
                 ui_shortcuts.create_artifact_download(artifact),
             )
 
-        context["task_data"] = format_yaml(self.object.task_data)
+        ctx["task_data"] = format_yaml(self.object.task_data)
 
         if (dynamic_task_data := self.object.dynamic_task_data) is not None:
-            context["show_task_data_dynamic"] = True
-            context["task_data_dynamic"] = format_yaml(dynamic_task_data)
+            ctx["show_task_data_dynamic"] = True
+            ctx["task_data_dynamic"] = format_yaml(dynamic_task_data)
         else:
-            context["show_task_data_dynamic"] = False
+            ctx["show_task_data_dynamic"] = False
 
         if (
             configured_task_data := self.object.configured_task_data
         ) is not None:
-            context["show_task_data_configured"] = True
-            context["task_data_configured"] = format_yaml(configured_task_data)
+            ctx["show_task_data_configured"] = True
+            ctx["task_data_configured"] = format_yaml(configured_task_data)
         else:
-            context["show_task_data_configured"] = False
+            ctx["show_task_data_configured"] = False
 
-        context["work_request_output_data"] = self._format_output_data(
+        ctx["work_request_output_data"] = self._format_output_data(
             self.object.output_data
         )
-        context["work_request_event_reactions_json"] = format_json(
+        ctx["work_request_event_reactions_json"] = format_json(
             self.object.event_reactions_json
         )
-        context["work_request_output_data_json"] = format_json(
+        ctx["work_request_output_data_json"] = format_json(
             self.object.output_data_json
         )
-        context["work_request_workflow_data_json"] = format_json(
+        ctx["work_request_workflow_data_json"] = format_json(
             self.object.workflow_data_json
         )
 
-        context["work_request_dependencies"] = WorkRequestTable(
+        ctx["work_request_dependencies"] = WorkRequestTable(
             self.request, self.object.dependencies.all(), prefix="dependencies"
         ).get_paginator(per_page=10)
-        context["work_request_reverse_dependencies"] = WorkRequestTable(
+        ctx["work_request_reverse_dependencies"] = WorkRequestTable(
             self.request,
             self.object.reverse_dependencies.all(),
             prefix="reverse_dependencies",
@@ -303,9 +349,13 @@ class WorkRequestDetailView(
             workflow_parents.append(parent)
             parent = parent.parent
 
-        context["workflow_parents"] = reversed(workflow_parents)
+        ctx["workflow_parents"] = reversed(workflow_parents)
 
-        return context
+        # Let the plugin override parts of the context, if needed
+        if self.plugin:
+            ctx.update(self.plugin.get_context_data())
+
+        return ctx
 
     @staticmethod
     def _format_output_data(
@@ -410,6 +460,10 @@ class WorkRequestPlugin(abc.ABC):
         """Check if the tab should appear."""
         return True
 
+    def add_tabs(self, tabs: list[Tab]) -> None:
+        """Add extra tabs to the tab list."""
+        pass
+
     def get_description_data(self) -> dict[str, Any]:
         """Return data for the description or {} if dynamic_data is None."""
         if self.task.dynamic_data is None:
