@@ -16,6 +16,7 @@ from django.shortcuts import get_object_or_404
 
 from debusine.db.models import WorkerPool
 from debusine.web.views.base import BaseUIView, DetailViewBase, ListViewBase
+from debusine.web.views.places import Place
 from debusine.web.views.table import TableMixin
 from debusine.web.views.tables import WorkerPoolTable, WorkerTable
 
@@ -53,9 +54,9 @@ class WorkerPoolDetailView(BaseUIView, DetailViewBase[WorkerPool]):
         queryset = self.get_queryset()
         return get_object_or_404(queryset, name=self.kwargs["name"])
 
-    def get_title(self) -> str:
-        """Get the title for the page."""
-        return f"Worker pool {self.object.name}"
+    def get_place(self) -> Place:
+        """Return the Place for this page."""
+        return self.object.ui(self.request).place
 
     def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
         """Add to context data."""
