@@ -23,9 +23,10 @@ class TestSystemInformation(TestCase):
     """Test for system_information.py functions."""
 
     def setUp(self) -> None:
-        """Ensure that host_architecture() is not cached."""
-        system_information.host_architecture.cache_clear()
-        self.addCleanup(system_information.host_architecture.cache_clear)
+        """Ensure that native_architecture() is not cached."""
+        super().setUp()
+        system_information.native_architecture.cache_clear()
+        self.addCleanup(system_information.native_architecture.cache_clear)
 
     def test_total_physical_memory(self) -> None:
         """Test total_physical_memory function."""
@@ -56,9 +57,9 @@ class TestSystemInformation(TestCase):
         with mock.patch("psutil.cpu_count", return_value=None):
             self.assertEqual(system_information.cpu_count(), 1)
 
-    def test_host_architecture(self) -> None:
-        """Test host_architecture() function."""
-        architecture = system_information.host_architecture()
+    def test_native_architecture(self) -> None:
+        """Test native_architecture() function."""
+        architecture = system_information.native_architecture()
 
         # Assert that architecture "looks good": type, length, no end of line
         assert isinstance(architecture, str)
@@ -79,21 +80,21 @@ class TestSystemInformation(TestCase):
         self.addCleanup(patcher.stop)
         return mocked
 
-    def test_host_architecture_dpkg_not_available(self) -> None:
+    def test_native_architecture_dpkg_not_available(self) -> None:
         """
-        Test host_architecture function: return None.
+        Test native_architecture function: return None.
 
         dpkg is not available or returns an error.
         """
         mocked = self.patch_subprocess_check_output()
         mocked.side_effect = FileNotFoundError()
-        self.assertIsNone(system_information.host_architecture())
+        self.assertIsNone(system_information.native_architecture())
 
-        system_information.host_architecture.cache_clear()
+        system_information.native_architecture.cache_clear()
         mocked.side_effect = subprocess.CalledProcessError(
             1, ["dpkg", "--print-architecture"]
         )
-        self.assertIsNone(system_information.host_architecture())
+        self.assertIsNone(system_information.native_architecture())
 
     def test_kernel_supported_architectures_arch_test_fails(self) -> None:
         """
@@ -119,18 +120,18 @@ class TestSystemInformation(TestCase):
             system_information.kernel_supported_architectures(), architectures
         )
 
-    def test_system_metadata_without_host_kernel_supported_architectures(
+    def test_system_metadata_without_native_kernel_supported_architectures(
         self,
     ) -> None:
         """
         system_metadata() does not include architectures related keys.
 
-        Functions host_architecture and kernel_supported_architectures returned
-        None.
+        Functions native_architecture and kernel_supported_architectures
+        returned None.
         """
         with (
             mock.patch(
-                "debusine.worker.system_information.host_architecture",
+                "debusine.worker.system_information.native_architecture",
                 return_value=None,
             ) as architecture_mocked,
             mock.patch(
@@ -144,21 +145,21 @@ class TestSystemInformation(TestCase):
         architecture_mocked.assert_called()
         kernel_supported_architectures_mocked.assert_called()
 
-        self.assertNotIn("system:host_architecture", metadata)
+        self.assertNotIn("system:native_architecture", metadata)
         self.assertNotIn("system:architectures", metadata)
 
-    def test_system_metadata_architectures_same_as_host_architecture(
+    def test_system_metadata_architectures_same_as_native_architecture(
         self,
     ) -> None:
         """
         system_metadata() returns valid metadata for an external worker.
 
         Method system_information.kernel_supported_architectures() returns
-        None: architectures is the same as host_architecture.
+        None: architectures is the same as native_architecture.
         """
         total_physical_memory = system_information.total_physical_memory()
         cpu_count = system_information.cpu_count()
-        host_architecture = system_information.host_architecture()
+        native_architecture = system_information.native_architecture()
 
         with mock.patch(
             "debusine.worker.system_information.kernel_supported_architectures",
@@ -170,16 +171,16 @@ class TestSystemInformation(TestCase):
                     "system:cpu_count": cpu_count,
                     "system:total_physical_memory": total_physical_memory,
                     "system:worker_type": WorkerType.EXTERNAL,
-                    "system:host_architecture": host_architecture,
-                    "system:architectures": [host_architecture],
+                    "system:native_architecture": native_architecture,
+                    "system:architectures": [native_architecture],
                 },
             )
 
-    def test_system_metadata_architectures_different_to_host_architecture(
+    def test_system_metadata_architectures_different_to_native_architecture(
         self,
     ) -> None:
         """system_metadata() populates architectures."""
-        host_architecture = system_information.host_architecture()
+        native_architecture = system_information.native_architecture()
         kernel_supported_architectures = ["x1", "x2"]
         with mock.patch(
             "debusine.worker.system_information.kernel_supported_architectures",
@@ -188,7 +189,7 @@ class TestSystemInformation(TestCase):
             self.assertDictContainsAll(
                 system_information.system_metadata(WorkerType.EXTERNAL),
                 {
-                    "system:host_architecture": host_architecture,
+                    "system:native_architecture": native_architecture,
                     "system:architectures": kernel_supported_architectures,
                 },
             )
@@ -197,7 +198,7 @@ class TestSystemInformation(TestCase):
         """system_metadata() returns valid metadata for a Celery worker."""
         total_physical_memory = system_information.total_physical_memory()
         cpu_count = system_information.cpu_count()
-        host_architecture = system_information.host_architecture()
+        native_architecture = system_information.native_architecture()
 
         system_metadata = system_information.system_metadata(WorkerType.CELERY)
 
@@ -210,7 +211,7 @@ class TestSystemInformation(TestCase):
                 "system:cpu_count": cpu_count,
                 "system:total_physical_memory": total_physical_memory,
                 "system:worker_type": WorkerType.CELERY,
-                "system:host_architecture": host_architecture,
+                "system:native_architecture": native_architecture,
             },
         )
 
@@ -218,7 +219,7 @@ class TestSystemInformation(TestCase):
         """system_metadata() returns valid metadata for a signing worker."""
         total_physical_memory = system_information.total_physical_memory()
         cpu_count = system_information.cpu_count()
-        host_architecture = system_information.host_architecture()
+        native_architecture = system_information.native_architecture()
 
         system_metadata = system_information.system_metadata(WorkerType.SIGNING)
 
@@ -231,6 +232,6 @@ class TestSystemInformation(TestCase):
                 "system:cpu_count": cpu_count,
                 "system:total_physical_memory": total_physical_memory,
                 "system:worker_type": WorkerType.SIGNING,
-                "system:host_architecture": host_architecture,
+                "system:native_architecture": native_architecture,
             },
         )
